@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2001, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2001, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -27,13 +27,15 @@
 #include <mysqld_error.h>
 #include <sys/types.h>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "client/check/mysqlcheck.h"
-#include "client/client_priv.h"
-#include "m_ctype.h"
+#include "client/include/client_priv.h"
+#include "m_string.h"
 #include "my_default.h"
 #include "my_inttypes.h"
+#include "mysql/strings/m_ctype.h"
 
 using namespace Mysql::Tools::Check;
 
@@ -130,14 +132,14 @@ static inline void escape_str(const string &src, size_t start, size_t end,
 }
 
 static string escape_table_name(const string &src) {
-  string res = "";
+  string res;
 
   escape_str(src, 0, src.length(), res);
   return res;
 }
 
 static string escape_db_table_name(const string &src, size_t dot_pos) {
-  string res = "";
+  string res;
 
   /* Escape database name. */
   escape_str(src, 0, dot_pos - 1, res);
@@ -171,7 +173,7 @@ static int process_all_tables_in_db(const string &database) {
     /* Skip views if we don't perform renaming. */
     if ((num_columns == 2) && (strcmp(row[1], "VIEW") == 0)) continue;
 
-    table_names.push_back(row[0]);
+    table_names.emplace_back(row[0]);
   }
   mysql_free_result(res);
 
@@ -190,7 +192,7 @@ static int run_query(const string &query) {
 
 static int rebuild_table(const string &name) {
   int rc = 0;
-  string query = "ALTER TABLE " + name + " FORCE";
+  string const query = "ALTER TABLE " + name + " FORCE";
   if (mysql_real_query(sock, query.c_str(), (ulong)query.length())) {
     fprintf(stderr, "Failed to %s\n", query.c_str());
     fprintf(stderr, "Error: %s\n", mysql_error(sock));
@@ -254,7 +256,7 @@ static int handle_request_for_tables(const string &tables) {
       break;
   }
 
-  string query = operation + " TABLE " + tables + " " + options;
+  string const query = operation + " TABLE " + tables + " " + options;
 
   if (mysql_real_query(sock, query.c_str(), (ulong)query.length())) {
     DBError(sock,
@@ -280,8 +282,8 @@ static void print_result() {
   prev_alter[0] = 0;
 
   while ((row = mysql_fetch_row(res))) {
-    int changed = strcmp(prev, row[0]);
-    bool status = !strcmp(row[2], "status");
+    const int changed = strcmp(prev, row[0]);
+    const bool status = !strcmp(row[2], "status");
 
     if (status) {
       /*
@@ -290,10 +292,10 @@ static void print_result() {
         list
       */
       if (found_error && opt_auto_repair && what_to_do != DO_REPAIR &&
-          strcmp(row[3], "OK")) {
+          strcmp(row[3], "OK") != 0) {
         if (table_rebuild) {
           if (prev_alter[0])
-            alter_table_cmds.push_back(prev_alter);
+            alter_table_cmds.emplace_back(prev_alter);
           else
             tables4rebuild.push_back(escape_db_table_name(prev, dot_pos));
         } else {
@@ -314,7 +316,7 @@ static void print_result() {
       } else {
         printf("%s\n%-9s: %s", row[0], row[2], row[3]);
       }
-      if (opt_auto_repair && strcmp(row[2], "note")) {
+      if (opt_auto_repair && strcmp(row[2], "note") != 0) {
         const char *alter_txt = strstr(row[3], "ALTER TABLE");
         found_error = true;
         if (alter_txt) {
@@ -345,7 +347,7 @@ static void print_result() {
   if (found_error && opt_auto_repair && what_to_do != DO_REPAIR) {
     if (table_rebuild) {
       if (prev_alter[0])
-        alter_table_cmds.push_back(prev_alter);
+        alter_table_cmds.emplace_back(prev_alter);
       else
         tables4rebuild.push_back(escape_db_table_name(prev, dot_pos));
     } else {
@@ -355,15 +357,18 @@ static void print_result() {
   mysql_free_result(res);
 }
 
-void Mysql::Tools::Check::mysql_check(
-    MYSQL *connection, int what_to_do, bool opt_alldbs,
-    bool opt_check_only_changed, bool opt_extended, bool opt_databases,
-    bool opt_fast, bool opt_medium_check, bool opt_quick, bool opt_all_in_1,
-    bool opt_silent, bool opt_auto_repair, bool ignore_errors, bool opt_frm,
-    bool opt_fix_table_names, bool opt_fix_db_names, bool opt_upgrade,
-    bool opt_write_binlog, uint verbose, std::string opt_skip_database,
-    std::vector<std::string> arguments,
-    void (*dberror)(MYSQL *mysql, const std::string &when)) {
+namespace Mysql::Tools::Check {
+
+void mysql_check(MYSQL *connection, int what_to_do, bool opt_alldbs,
+                 bool opt_check_only_changed, bool opt_extended,
+                 bool opt_databases, bool opt_fast, bool opt_medium_check,
+                 bool opt_quick, bool opt_all_in_1, bool opt_silent,
+                 bool opt_auto_repair, bool ignore_errors, bool opt_frm,
+                 bool opt_fix_table_names, bool opt_fix_db_names,
+                 bool opt_upgrade, bool opt_write_binlog, uint verbose,
+                 std::string opt_skip_database,
+                 std::vector<std::string> arguments,
+                 void (*dberror)(MYSQL *mysql, const std::string &when)) {
   ::sock = connection;
   ::what_to_do = what_to_do;
   ::opt_alldbs = opt_alldbs;
@@ -383,7 +388,7 @@ void Mysql::Tools::Check::mysql_check(
   ::opt_upgrade = opt_upgrade;
   ::opt_write_binlog = opt_write_binlog;
   ::verbose = verbose;
-  ::opt_skip_database = opt_skip_database;
+  ::opt_skip_database = std::move(opt_skip_database);
   ::DBError = dberror;
 
   if (!::opt_write_binlog) {
@@ -396,7 +401,7 @@ void Mysql::Tools::Check::mysql_check(
   if (::opt_alldbs) process_all_databases();
   /* Only one database and selected table(s) */
   else if (arguments.size() > 1 && !::opt_databases) {
-    string db_name = arguments[0];
+    string const db_name = arguments[0];
     arguments.erase(arguments.begin());
     process_selected_tables(db_name, arguments);
   }
@@ -420,6 +425,8 @@ void Mysql::Tools::Check::mysql_check(
   }
 }
 
+}  // namespace Mysql::Tools::Check
+
 Program::Program()
     : m_what_to_do(0),
       m_auto_repair(false),
