@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -28,8 +28,8 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "log_service_imp.h"
 
-#include <assert.h>
 #include <mysqld_error.h>  // ER_*
+#include <cassert>
 #include "my_compiler.h"
 
 #include <mysql/components/services/log_builtins.h>
