@@ -25,39 +25,46 @@
 #include "my_byteorder.h"
 #include "mysql/binlog/event/control_events.h"
 #include "mysql/binlog/event/statement_events.h"
+#include "mysql/gtids/gtids.h"  // Gtid_set
 
 namespace cs::reader {
 
 State &State::operator=(const State &other) {
-  m_gtid_set = other.get_gtids();
+  if (m_gtid_set.assign(other.get_gtids()) != mysql::utils::Return_status::ok)
+    throw std::bad_alloc{};
   return *this;
 }
 
-State::State(const State &other) { m_gtid_set = other.get_gtids(); }
-
-std::string State::to_string() const { return m_gtid_set.to_string(); }
-
-void State::reset() { return m_gtid_set.reset(); }
-
-const mysql::gtid::Gtid_set &State::get_gtids() const { return m_gtid_set; }
-
-bool State::set_gtids(const mysql::gtid::Gtid_set &gtids) {
-  return m_gtid_set.add(gtids);
+State::State(const State &other) {
+  if (m_gtid_set.assign(other.get_gtids()) != mysql::utils::Return_status::ok)
+    throw std::bad_alloc{};
 }
 
-void State::add_gtid_set(const mysql::gtid::Gtid_set &gtids) {
-  m_gtid_set.add(gtids);
+std::string State::to_string() const {
+  return mysql::strconv::throwing::encode_text(m_gtid_set);
 }
 
-void State::add_gtid(const mysql::gtid::Gtid &gtid) { m_gtid_set.add(gtid); }
+void State::reset() { m_gtid_set.clear(); }
+
+const mysql::gtids::Gtid_set &State::get_gtids() const { return m_gtid_set; }
+
+void State::add_gtid_set(const mysql::gtids::Gtid_set &gtids) {
+  if (m_gtid_set.inplace_union(gtids) != mysql::utils::Return_status::ok)
+    throw std::bad_alloc{};
+}
+
+void State::add_gtid(const mysql::gtids::Gtid &gtid) {
+  if (m_gtid_set.insert(gtid) != mysql::utils::Return_status::ok)
+    throw std::bad_alloc{};
+}
 
 std::ostringstream &operator<<(std::ostringstream &out, const State &in) {
-  out << in.m_gtid_set.to_string() << std::flush;
+  out << mysql::strconv::throwing::encode_text(in.get_gtids()) << std::flush;
   return out;
 }
 
 std::stringstream &operator<<(std::stringstream &out, const State &in) {
-  out << in.m_gtid_set.to_string() << std::flush;
+  out << mysql::strconv::throwing::encode_text(in.get_gtids()) << std::flush;
   return out;
 }
 
