@@ -43,6 +43,7 @@ our @EXPORT = qw(
   mtr_report_stats
   mtr_report_test
   mtr_report_test_passed
+  mtr_report_test_opt_passed
   mtr_report_test_skipped
   mtr_summary_file_init
   mtr_verbose
@@ -167,6 +168,12 @@ sub mtr_report_test_skipped ($) {
   mtr_report_test($tinfo);
 }
 
+sub mtr_report_test_opt_passed ($) {
+  my ($tinfo) = @_;
+  $tinfo->{'result'} = 'MTR_RES_OPT_PASSED';
+  mtr_report_test($tinfo);
+}
+
 sub mtr_report_test_passed ($) {
   my ($tinfo) = @_;
 
@@ -201,6 +208,7 @@ sub mtr_report_test ($) {
   my $retry    = $tinfo->{'retries'} ? "retry-" : "";
   my $warnings = $tinfo->{'warnings'};
   my $skip_ignored   = $tinfo->{'skip_ignored'};
+  my $opt_pass = $tinfo->{'opt_pass'};
 
   if ($::opt_test_progress and $tinfo->{'name'} and !$retry) {
     $tests_completed = $tests_completed + 1;
@@ -278,6 +286,20 @@ sub mtr_report_test ($) {
         mtr_report($report, "[ skipped ]");
       }
     }
+  } elsif ($result eq 'MTR_RES_OPT_PASSED') {
+    if ($comment) {
+      if ($::opt_quiet) {
+        mtr_buffered_report(sprintf("%-60s%-s", $report, "[ opt-pass ]"));
+      } else {
+        mtr_report($report, "[ opt-pass ]  $comment");
+      }
+    } else {
+      if ($::opt_quiet) {
+        mtr_buffered_report(sprintf("%-60s%-s", $report, "[ opt-pass ]"));
+      } else {
+        mtr_report($report, "[ opt-pass ]");
+      }
+    }
   } elsif ($result eq 'MTR_RES_PASSED') {
 
     my $pass = $skip_ignored ? "noskip-${retry}pass" : "${retry}pass";
@@ -591,7 +613,8 @@ sub mtr_report_stats ($$;$) {
       # Test was skipped (disabled not counted)
       $tot_skipped++ unless $tinfo->{'disable'};
       $tot_skipdetect++ if $tinfo->{'skip_detected_by_test'};
-    } elsif ($tinfo->{'result'} eq 'MTR_RES_PASSED') {
+    } elsif ($tinfo->{'result'} eq 'MTR_RES_PASSED' or
+             $tinfo->{'result'} eq 'MTR_RES_OPT_PASSED') {
       # Test passed
       $tot_tests++;
       $tot_passed++;
