@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -37,18 +37,12 @@ using std::map;
 using std::set;
 using std::string;
 
-#include <assert.h>
 #include <algorithm>
+#include <cassert>
 #include <iostream>
 #include <iterator>
 #include <list>
 
-/*
-  This is used to disable the arbitrator hack in XCOM just
-  in case it is not disabled by default.
-*/
-extern int ARBITRATOR_HACK;
-
 // Suspicion timeout in 100s of nanoseconds (60 seconds)
 static const uint64_t NON_MEMBER_EXPEL_TIMEOUT = 60 * 10000000;
 
@@ -66,7 +60,7 @@ static bool is_terminate_suspicion_thread() {
 }
 
 static void *suspicions_processing_thread(void *ptr) {
-  Gcs_xcom_control *gcs_ctrl = (Gcs_xcom_control *)ptr;
+  auto *gcs_ctrl = (Gcs_xcom_control *)ptr;
   Gcs_suspicions_manager *mgr = gcs_ctrl->get_suspicions_manager();
 
   while (!is_terminate_suspicion_thread()) {
@@ -80,9 +74,9 @@ static void *suspicions_processing_thread(void *ptr) {
 }
 
 static void *xcom_taskmain_startup(void *ptr) {
-  Gcs_xcom_control *gcs_ctrl = (Gcs_xcom_control *)ptr;
+  auto *gcs_ctrl = (Gcs_xcom_control *)ptr;
   Gcs_xcom_proxy *proxy = gcs_ctrl->get_xcom_proxy();
-  xcom_port port = gcs_ctrl->get_node_address()->get_member_port();
+  xcom_port const port = gcs_ctrl->get_node_address()->get_member_port();
   bool error = true;
 
   auto *xcom_interface =
@@ -119,14 +113,15 @@ map<int, const Gcs_control_event_listener &>
 Gcs_xcom_control::Gcs_xcom_control(
     Gcs_xcom_node_address *xcom_node_address,
     std::vector<Gcs_xcom_node_address *> &xcom_peers,
-    Gcs_group_identifier m_group_identifier, Gcs_xcom_proxy *xcom_proxy,
+    const Gcs_group_identifier &m_group_identifier, Gcs_xcom_proxy *xcom_proxy,
     Gcs_xcom_group_management *xcom_group_management,
     Gcs_xcom_engine *gcs_engine,
     Gcs_xcom_state_exchange_interface *state_exchange,
     Gcs_xcom_view_change_control_interface *view_control, bool boot,
     My_xp_socket_util *socket_util,
     std::unique_ptr<Network_provider_operations_interface>
-        comms_operation_interface)
+        comms_operation_interface,
+    Gcs_xcom_statistics_manager_interface *stats_mgr)
     : m_gid(nullptr),
       m_gid_hash(0),
       m_xcom_proxy(xcom_proxy),
@@ -143,6 +138,7 @@ Gcs_xcom_control::Gcs_xcom_control(
       m_suspicions_processing_thread(),
       m_sock_probe_interface(nullptr),
       m_comms_operation_interface(std::move(comms_operation_interface)),
+      m_stats_mgr(stats_mgr),
       m_xcom_running(false),
       m_leave_view_requested(false),
       m_leave_view_delivered(false),
@@ -269,7 +265,7 @@ enum_gcs_error Gcs_xcom_control::join() {
 
   Gcs_xcom_notification *notification =
       new Control_notification(do_function_join, this);
-  bool scheduled = m_gcs_engine->push(notification);
+  bool const scheduled = m_gcs_engine->push(notification);
   if (!scheduled) {
     MYSQL_GCS_LOG_DEBUG(
         "Tried to enqueue a join request but the member is about to stop.")
@@ -319,16 +315,17 @@ static bool skip_own_peer_address(
     std::string &peer_address, int peer_port) {
   std::vector<std::string> peer_rep_ip;
 
-  bool resolve_error = resolve_ip_addr_from_hostname(peer_address, peer_rep_ip);
+  bool const resolve_error =
+      resolve_ip_addr_from_hostname(peer_address, peer_rep_ip);
   if (resolve_error) {
     MYSQL_GCS_LOG_WARN("Unable to resolve peer address " << peer_address.c_str()
                                                          << ". Skipping...")
     return true;
   }
 
-  for (auto &local_node_info_str_ip_entry : my_own_addresses) {
+  for (const auto &local_node_info_str_ip_entry : my_own_addresses) {
     for (auto &peer_rep_ip_entry : peer_rep_ip) {
-      if (peer_rep_ip_entry.compare(local_node_info_str_ip_entry.first) == 0 &&
+      if (peer_rep_ip_entry == local_node_info_str_ip_entry.first &&
           peer_port == my_own_port) {
         // Skip own address if configured in the peer list
         return true;
@@ -351,8 +348,8 @@ static constexpr const char *get_signaling_error() {
          "restarting Group Replication on this server.";
 }
 
-static const std::string get_connection_test_error(const std::string &address,
-                                                   int port) {
+static std::string get_connection_test_error(const std::string &address,
+                                             int port) {
   std::stringstream retval;
   retval << "The group communication engine failed to test connectivity to the "
             "local group communication engine on "
@@ -370,7 +367,7 @@ static const std::string get_connection_test_error(const std::string &address,
 
 enum_gcs_error Gcs_xcom_control::retry_do_join() {
   /* Used to initialize xcom */
-  int local_port = m_local_node_address->get_member_port();
+  int const local_port = m_local_node_address->get_member_port();
   int comm_status = XCOM_COMM_STATUS_UNDEFINED;
   enum_gcs_error is_xcom_ready = GCS_NOK;
   bool xcom_input_open = false;
@@ -565,7 +562,7 @@ bool Gcs_xcom_control::try_send_add_node_request_to_seeds(
     connection_descriptor *con = nullptr;
     std::tie(connected, con) = connect_to_peer(peer, my_addresses);
 
-    if (bool finalized = m_view_control->is_finalized();
+    if (bool const finalized = m_view_control->is_finalized();
         !finalized && connected) {
       MYSQL_GCS_LOG_INFO("Sucessfully connected to peer "
                          << peer.get_member_ip().c_str() << ":"
@@ -678,7 +675,7 @@ enum_gcs_error Gcs_xcom_control::leave() {
 
   Gcs_xcom_notification *notification =
       new Control_notification(do_function_leave, this);
-  bool scheduled = m_gcs_engine->push(notification);
+  bool const scheduled = m_gcs_engine->push(notification);
   if (!scheduled) {
     MYSQL_GCS_LOG_DEBUG(
         "Tried to enqueue a leave request but the member is about to stop.")
@@ -708,7 +705,7 @@ enum_gcs_error Gcs_xcom_control::do_leave() {
   /*
     Wait until the XCOM's thread exits.
   */
-  int is_xcom_exit = m_xcom_proxy->xcom_wait_exit();
+  int const is_xcom_exit = m_xcom_proxy->xcom_wait_exit();
 
   if (is_xcom_exit == GCS_NOK) {
     MYSQL_GCS_LOG_ERROR("The member has failed to gracefully leave the group.")
@@ -782,7 +779,7 @@ connection_descriptor *Gcs_xcom_control::get_connection_to_node(
   for (it = peers_list->begin(); (con == nullptr) && it != peers_list->end();
        it++) {
     Gcs_xcom_node_address *peer = *(it);
-    std::string peer_rep_ip;
+    std::string const peer_rep_ip;
 
     if (skip_own_peer_address(local_node_info_str_ips,
                               m_local_node_address->get_member_port(),
@@ -791,7 +788,7 @@ connection_descriptor *Gcs_xcom_control::get_connection_to_node(
       continue;
     }
 
-    xcom_port port = peer->get_member_port();
+    xcom_port const port = peer->get_member_port();
     const char *addr = peer->get_member_ip().c_str();
 
     MYSQL_GCS_LOG_TRACE(
@@ -815,7 +812,7 @@ void Gcs_xcom_control::do_remove_node_from_group() {
   if (m_view_control->is_leaving() || !m_view_control->belongs_to_group())
     return;
 
-  int local_port = m_local_node_address->get_member_port();
+  int const local_port = m_local_node_address->get_member_port();
   bool rm_ret = false;
   connection_descriptor *con = nullptr;
 
@@ -840,9 +837,8 @@ void Gcs_xcom_control::do_remove_node_from_group() {
 
     for (it = current_view->get_members().begin();
          !con && it != current_view->get_members().end(); it++) {
-      std::string peer_rep_ip;
-      Gcs_xcom_node_address *peer =
-          new Gcs_xcom_node_address(it->get_member_id());
+      std::string const peer_rep_ip;
+      auto *peer = new Gcs_xcom_node_address(it->get_member_id());
 
       view_members.push_back(peer);
     }
@@ -909,8 +905,6 @@ void Gcs_xcom_control::do_remove_node_from_group() {
 
   MYSQL_GCS_LOG_DEBUG("do_remove_node_from_group finished! Returning %d",
                       rm_ret);
-
-  return;
 }
 
 bool Gcs_xcom_control::belongs_to_group() {
@@ -931,7 +925,7 @@ int Gcs_xcom_control::add_event_listener(
   int handler_key = 0;
   do {
     handler_key = rand();
-  } while (event_listeners.count(handler_key) != 0);
+  } while (event_listeners.contains(handler_key));
 
   event_listeners.emplace(handler_key, event_listener);
 
@@ -964,8 +958,7 @@ void Gcs_xcom_control::build_total_members(
     /*
       Build the member identifier from the address reported.
     */
-    Gcs_member_identifier *member_id =
-        new Gcs_member_identifier((*nodes_it).get_member_id());
+    auto *member_id = new Gcs_member_identifier((*nodes_it).get_member_id());
 
     /*
       Check whether the node is reported as alive or faulty.
@@ -1115,7 +1108,7 @@ bool Gcs_xcom_control::is_killer_node(
     members.
   */
   assert(alive_members.size() != 0 && alive_members[0] != nullptr);
-  bool ret = get_local_member_identifier() == *alive_members[0];
+  bool const ret = get_local_member_identifier() == *alive_members[0];
   MYSQL_GCS_LOG_DEBUG("The member %s will be responsible for killing: %d",
                       get_local_member_identifier().get_member_id().c_str(),
                       ret)
@@ -1160,7 +1153,7 @@ bool Gcs_xcom_control::xcom_receive_local_view(synode_no const config_id,
       /*
         Build the member identifier from the address reported.
       */
-      Gcs_member_identifier member_id((*nodes_it).get_member_id());
+      Gcs_member_identifier const member_id((*nodes_it).get_member_id());
 
       // filter out those that are not yet in the current view
       // delivered to the application. For example, they might
@@ -1266,6 +1259,11 @@ bool Gcs_xcom_control::xcom_receive_local_view(synode_no const config_id,
                                 "suspicious in the cluster: %s",
                                 node_no, (*it)->get_member_id().c_str());)
 
+    // Register suspicious per node, for statistical purposes
+    for (auto &&suspect_member_id : unreachable) {
+      m_stats_mgr->add_suspicious_for_a_node(suspect_member_id.get_member_id());
+    }
+
     // always notify local views
     for (callback_it = event_listeners.begin();
          callback_it != event_listeners.end(); callback_it++) {
@@ -1308,7 +1306,7 @@ void Gcs_xcom_control::install_leave_view(
   Gcs_view *current_view = m_view_control->get_unsafe_current_view();
 
   // Create the new view id here, based in the previous one plus 1
-  Gcs_xcom_view_identifier *new_view_id = new Gcs_xcom_view_identifier(
+  auto *new_view_id = new Gcs_xcom_view_identifier(
       static_cast<const Gcs_xcom_view_identifier &>(
           current_view->get_view_id()));
   new_view_id->increment_by_one();
@@ -1333,7 +1331,7 @@ void Gcs_xcom_control::install_leave_view(
 
   MYSQL_GCS_LOG_DEBUG("Installing leave view.")
 
-  Gcs_group_identifier gid(current_view->get_group_id().get_group_id());
+  Gcs_group_identifier const gid(current_view->get_group_id().get_group_id());
   install_view(new_view_id, gid, nullptr, total, left, joined, error_code);
 
   set<Gcs_member_identifier *>::iterator total_it;
@@ -1527,7 +1525,7 @@ bool Gcs_xcom_control::xcom_receive_global_view(synode_no const config_id,
     If the execution has managed to pass these steps, the node is alive and
     it is time to start building the view.
   */
-  if (do_not_deliver_to_client || failed_members.size() != 0) {
+  if (do_not_deliver_to_client || !failed_members.empty()) {
     MYSQL_GCS_LOG_TRACE(
         "(My node_id is %d) ::xcom_receive_global_view():: "
         "Discarding view because nothing has changed. "
@@ -1628,15 +1626,13 @@ end:
   return ret;
 }
 
-extern uint32_t get_my_xcom_id();
-
 void Gcs_xcom_control::process_control_message(
     Gcs_message *msg, Gcs_protocol_version maximum_supported_protocol_version,
     Gcs_protocol_version used_protocol_version) {
   MYSQL_GCS_LOG_TRACE(
       "::process_control_message():: Received a control message")
 
-  Xcom_member_state *ms_info = new Xcom_member_state(
+  auto *ms_info = new Xcom_member_state(
       maximum_supported_protocol_version, msg->get_message_data().get_payload(),
       msg->get_message_data().get_payload_length());
 
@@ -1687,9 +1683,9 @@ void Gcs_xcom_control::process_control_message(
     return;
   }
 
-  Gcs_member_identifier pid(msg->get_origin());
+  Gcs_member_identifier const pid(msg->get_origin());
   // takes ownership of ms_info
-  bool state_exchange_done = m_state_exchange->process_member_state(
+  bool const state_exchange_done = m_state_exchange->process_member_state(
       ms_info, pid, maximum_supported_protocol_version, used_protocol_version);
 
   // If state exchange has finished
@@ -1719,8 +1715,7 @@ void Gcs_xcom_control::process_control_message(
     Gcs_xcom_view_identifier *provided_view_id =
         m_state_exchange->get_new_view_id();
 
-    Gcs_xcom_view_identifier *new_view_id =
-        new Gcs_xcom_view_identifier(*provided_view_id);
+    auto *new_view_id = new Gcs_xcom_view_identifier(*provided_view_id);
 
     new_view_id->increment_by_one();
 
@@ -1772,11 +1767,11 @@ void Gcs_xcom_control::install_view(
   build_member_list(join, &joined_members);
 
   // Build the new view id and the group id
-  Gcs_xcom_view_identifier v_id(*new_view_id);
+  Gcs_xcom_view_identifier const v_id(*new_view_id);
 
   // Create the new view
-  Gcs_view *current_view = new Gcs_view(members, v_id, left_members,
-                                        joined_members, group, error_code);
+  auto *current_view = new Gcs_view(members, v_id, left_members, joined_members,
+                                    group, error_code);
 
   // Build the exchanged data
   Exchanged_data data_to_deliver;
@@ -1786,8 +1781,7 @@ void Gcs_xcom_control::install_view(
       MYSQL_GCS_LOG_DEBUG(
           "Processing exchanged data while installing the new view")
 
-      Gcs_member_identifier *member_id =
-          new Gcs_member_identifier((*states_it).first);
+      auto *member_id = new Gcs_member_identifier((*states_it).first);
 
       Xcom_member_state *data_exchanged = (*states_it).second;
 
@@ -1800,7 +1794,7 @@ void Gcs_xcom_control::install_view(
                                       data_exchanged->get_data_size());
       }
 
-      std::pair<Gcs_member_identifier *, Gcs_message_data *> state_pair(
+      std::pair<Gcs_member_identifier *, Gcs_message_data *> const state_pair(
           member_id, data_exchanged_holder);
 
       data_to_deliver.push_back(state_pair);
@@ -1822,8 +1816,7 @@ void Gcs_xcom_control::install_view(
   */
   m_view_control->set_belongs_to_group(true);
 
-  map<int, const Gcs_control_event_listener &>::iterator callback_it =
-      event_listeners.begin();
+  auto callback_it = event_listeners.begin();
 
   while (callback_it != event_listeners.end()) {
     (*callback_it).second.on_view_changed(*current_view, data_to_deliver);
@@ -1852,7 +1845,7 @@ void Gcs_xcom_control::build_member_list(
   std::set<Gcs_member_identifier *>::iterator it;
 
   for (it = origin->begin(); it != origin->end(); it++) {
-    Gcs_member_identifier member_id(*(*it));
+    Gcs_member_identifier const member_id(*(*it));
 
     to_fill->push_back(member_id);
   }
@@ -1866,7 +1859,7 @@ void Gcs_xcom_control::init_me() {
 void Gcs_xcom_control::set_node_address(
     Gcs_xcom_node_address *xcom_node_address) {
   m_local_node_address = xcom_node_address;
-  string address = xcom_node_address->get_member_address();
+  string const address = xcom_node_address->get_member_address();
   delete m_local_node_info;
   /*
     We don't care at this point about the unique identifier associated to
@@ -1875,7 +1868,7 @@ void Gcs_xcom_control::set_node_address(
   */
   m_local_node_info = new Gcs_xcom_node_information(address);
 
-  Network_configuration_parameters params;
+  Network_configuration_parameters params{};
   params.port = xcom_node_address->get_member_port();
   m_comms_operation_interface->configure_active_provider(params);
 }
@@ -1986,12 +1979,13 @@ void Gcs_suspicions_manager::clear_suspicions() {
 
 void Gcs_suspicions_manager::process_view(
     synode_no const config_id, Gcs_xcom_nodes *xcom_nodes,
-    std::vector<Gcs_member_identifier *> alive_nodes,
-    std::vector<Gcs_member_identifier *> left_nodes,
-    std::vector<Gcs_member_identifier *> member_suspect_nodes,
-    std::vector<Gcs_member_identifier *> non_member_suspect_nodes,
+    const std::vector<Gcs_member_identifier *> &alive_nodes,
+    const std::vector<Gcs_member_identifier *> &left_nodes,
+    const std::vector<Gcs_member_identifier *> &member_suspect_nodes,
+    const std::vector<Gcs_member_identifier *> &non_member_suspect_nodes,
     bool is_killer_node, synode_no max_synode) {
   bool should_wake_up_manager = false;
+  std::vector<Gcs_member_identifier *> back_to_healthy;
 
   m_suspicions_mutex.lock();
 
@@ -1999,8 +1993,27 @@ void Gcs_suspicions_manager::process_view(
 
   m_config_id = config_id;
 
+  /* Remove the information about expels of members left because they
+   * have already taken effect(left). */
   m_expels_in_progress.forget_expels_that_have_taken_effect(config_id,
                                                             left_nodes);
+
+  /* Remove the information about expels of members alive because they
+   * have already taken effect(rejoined). */
+  if (m_expels_in_progress.size() && !alive_nodes.empty()) {
+    for (Gcs_member_identifier *const &live : alive_nodes) {
+      if (m_expels_in_progress.contains(*live)) {
+        MYSQL_GCS_LOG_DEBUG(
+            "%s: Expelled node %s, rejoined the group immediately.", __func__,
+            live->get_member_id().c_str());
+        back_to_healthy.push_back(live);
+      }
+    }
+    if (!back_to_healthy.empty())
+      m_expels_in_progress.forget_expels_that_have_taken_effect(
+          config_id, back_to_healthy);
+  }
+
   MYSQL_GCS_DEBUG_EXECUTE({
     /* Sanity check: all members in `m_expels_in_progress` must still be in
        `xcom_nodes` (the XCom view) at this point. Otherwise there is a bug in
@@ -2067,7 +2080,7 @@ bool Gcs_suspicions_manager::add_suspicions(
   bool member_suspicions_added = false;
 
   // Get current timestamp
-  uint64_t current_ts = My_xp_util::getsystime();
+  uint64_t const current_ts = My_xp_util::getsystime();
 
   for (susp_it = non_member_suspect_nodes.begin();
        susp_it != non_member_suspect_nodes.end(); ++susp_it) {
@@ -2162,11 +2175,11 @@ void Gcs_suspicions_manager::run_process_suspicions(bool lock) {
   uint64_t node_timeout;
 
   // Get current timestamp
-  uint64_t current_ts = My_xp_util::getsystime();
+  uint64_t const current_ts = My_xp_util::getsystime();
 
   // Get suspicions timeouts
-  uint64_t non_member_expel_timeout = get_non_member_expel_timeout();
-  uint64_t member_expel_timeout = get_member_expel_timeout();
+  uint64_t const non_member_expel_timeout = get_non_member_expel_timeout();
+  uint64_t const member_expel_timeout = get_member_expel_timeout();
 
   // Cycle through the suspicions
   std::vector<Gcs_xcom_node_information>::iterator susp_it;
@@ -2201,7 +2214,7 @@ void Gcs_suspicions_manager::run_process_suspicions(bool lock) {
       m_suspicions.remove_node(*susp_it);
       /* purecov: end */
     } else {
-      std::string node_id = susp_it->get_member_id().get_member_id();
+      std::string const node_id = susp_it->get_member_id().get_member_id();
       if (susp_it->is_member() && !susp_it->has_lost_messages() &&
           synode_gt(m_cache_last_removed, susp_it->get_max_synode())) {
         const_cast<Gcs_xcom_node_information *>(m_suspicions.get_node(node_id))
@@ -2278,7 +2291,7 @@ uint64_t Gcs_suspicions_manager::get_non_member_expel_timeout() {
 void Gcs_suspicions_manager::set_non_member_expel_timeout_seconds(
     unsigned long sec) {
   m_suspicions_parameters_mutex.lock();
-  m_non_member_expel_timeout = sec * 10000000ul;
+  m_non_member_expel_timeout = sec * 10000000UL;
   MYSQL_GCS_LOG_DEBUG("Set non-member expel timeout to %lu seconds (%lu  ns).",
                       sec, m_non_member_expel_timeout * 100);
   m_suspicions_parameters_mutex.unlock();
@@ -2295,7 +2308,7 @@ uint64_t Gcs_suspicions_manager::get_member_expel_timeout() {
 void Gcs_suspicions_manager::set_member_expel_timeout_seconds(
     unsigned long sec) {
   m_suspicions_parameters_mutex.lock();
-  m_member_expel_timeout = sec * 10000000ul;
+  m_member_expel_timeout = sec * 10000000UL;
   MYSQL_GCS_LOG_DEBUG("Set member expel timeout to %lu seconds (%lu  ns).", sec,
                       m_member_expel_timeout * 100);
   m_suspicions_parameters_mutex.unlock();
@@ -2318,7 +2331,7 @@ void Gcs_suspicions_manager::wake_suspicions_processing_thread(bool terminate) {
   set_terminate_suspicion_thread(terminate);
 
   // Wake up suspicions processing thread
-  int ret = m_suspicions_cond.signal();
+  int const ret = m_suspicions_cond.signal();
   MYSQL_GCS_LOG_DEBUG(
       "wake_suspicions_processing_thread: Signaled cond! Return= %d", ret);
 
