@@ -1,16 +1,17 @@
 /* Copyright (C) 2007 Google Inc.
-   Copyright (c) 2008, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2008, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -124,6 +125,14 @@ unsigned int ActiveTranx::get_hash_value(const char *log_file_name,
 
 int ActiveTranx::compare(const char *log_file_name1, my_off_t log_file_pos1,
                          const char *log_file_name2, my_off_t log_file_pos2) {
+  unsigned int len1 = strlen(log_file_name1);
+  unsigned int len2 = strlen(log_file_name2);
+  if (len1 > len2)
+    return 1;
+  else if (len1 < len2)
+    return -1;
+
+  assert(len1 == len2);
   int cmp = strcmp(log_file_name1, log_file_name2);
 
   if (cmp != 0) return cmp;
@@ -333,9 +342,9 @@ int ActiveTranx::clear_active_tranx_nodes(const char *log_file_name,
   return function_exit(kWho, 0);
 }
 
-int ReplSemiSyncMaster::reportReplyPacket(uint32 server_id, const uchar *packet,
+int ReplSemiSyncSource::reportReplyPacket(uint32 server_id, const uchar *packet,
                                           ulong packet_len) {
-  const char *kWho = "ReplSemiSyncMaster::reportReplyPacket";
+  const char *kWho = "ReplSemiSyncSource::reportReplyPacket";
   int result = -1;
   char log_file_name[FN_REFLEN + 1];
   my_off_t log_file_pos;
@@ -344,7 +353,7 @@ int ReplSemiSyncMaster::reportReplyPacket(uint32 server_id, const uchar *packet,
   function_enter(kWho);
 
   if (unlikely(packet[REPLY_MAGIC_NUM_OFFSET] !=
-               ReplSemiSyncMaster::kPacketMagicNum)) {
+               ReplSemiSyncSource::kPacketMagicNum)) {
     LogErr(ERROR_LEVEL, ER_SEMISYNC_REPLY_MAGIC_NO_ERROR);
     goto l_end;
   }
@@ -376,36 +385,38 @@ l_end:
 
 /*******************************************************************************
  *
- * <ReplSemiSyncMaster> class: the basic code layer for sync-replication master.
- * <ReplSemiSyncSlave>  class: the basic code layer for sync-replication slave.
+ * <ReplSemiSyncSource> class: the basic code layer for sync-replication source.
+ * <ReplSemiSyncReplica>  class: the basic code layer for sync-replication
+ * replica.
  *
  * The most important functions during semi-syn replication listed:
  *
- * Master:
+ * Source:
  *  . reportReplyBinlog():  called by the binlog dump thread when it receives
- *                          the slave's status information.
+ *                          the replica's status information.
  *  . updateSyncHeader():   based on transaction waiting information, decide
- *                          whether to request the slave to reply.
+ *                          whether to request the replica to reply.
  *  . writeTranxInBinlog(): called by the transaction thread when it finishes
  *                          writing all transaction events in binlog.
- *  . commitTrx():          transaction thread wait for the slave reply.
+ *  . commitTrx():          transaction thread wait for the replica reply.
  *
- * Slave:
- *  . slaveReadSyncHeader(): read the semi-sync header from the master, get the
- *                           sync status and get the payload for events.
- *  . slaveReply():          reply to the master about the replication progress.
+ * Replica:
+ *  . replicaReadSyncHeader(): read the semi-sync header from the source, get
+ *                             the sync status and get the payload for events.
+ *  . replicaReply():          reply to the source about the replication
+ *                             progress.
  *
  ******************************************************************************/
 
-ReplSemiSyncMaster::ReplSemiSyncMaster() {
+ReplSemiSyncSource::ReplSemiSyncSource() {
   reply_file_name_[0] = '\0';
   wait_file_name_[0] = '\0';
   commit_file_name_[0] = '\0';
 }
 
-int ReplSemiSyncMaster::initObject() {
+int ReplSemiSyncSource::initObject() {
   int result;
-  const char *kWho = "ReplSemiSyncMaster::initObject";
+  const char *kWho = "ReplSemiSyncSource::initObject";
 
   if (init_done_) {
     LogErr(WARNING_LEVEL, ER_SEMISYNC_FUNCTION_CALLED_TWICE, kWho);
@@ -423,25 +434,27 @@ int ReplSemiSyncMaster::initObject() {
 
   /*
     rpl_semi_sync_source_wait_for_replica_count may be set through mysqld
-    option. So call setWaitSlaveCount to initialize the internal ack container.
+    option. So call setWaitReplicaCount to initialize the internal ack
+    container.
   */
-  if (setWaitSlaveCount(rpl_semi_sync_source_wait_for_replica_count)) return 1;
+  if (setWaitReplicaCount(rpl_semi_sync_source_wait_for_replica_count))
+    return 1;
 
   if (rpl_semi_sync_source_enabled)
-    result = enableMaster();
+    result = enableSource();
   else
-    result = disableMaster();
+    result = disableSource();
 
   return result;
 }
 
-int ReplSemiSyncMaster::enableMaster() {
+int ReplSemiSyncSource::enableSource() {
   int result = 0;
 
   /* Must have the lock when we do enable of disable. */
   lock();
 
-  if (!getMasterEnabled()) {
+  if (!getSourceEnabled()) {
     if (active_tranxs_ == nullptr)
       active_tranxs_ = new ActiveTranx(&LOCK_binlog_, trace_level_);
 
@@ -450,7 +463,7 @@ int ReplSemiSyncMaster::enableMaster() {
       reply_file_name_inited_ = false;
       wait_file_name_inited_ = false;
 
-      set_master_enabled(true);
+      set_source_enabled(true);
       /*
         state_ will be set off when users don't want to wait(
         rpl_semi_sync_source_wait_no_replica == 0) if there is no enough active
@@ -471,11 +484,11 @@ int ReplSemiSyncMaster::enableMaster() {
   return result;
 }
 
-int ReplSemiSyncMaster::disableMaster() {
+int ReplSemiSyncSource::disableSource() {
   /* Must have the lock when we do enable of disable. */
   lock();
 
-  if (getMasterEnabled()) {
+  if (getSourceEnabled()) {
     /* Switch off the semi-sync first so that waiting transaction will be
      * waken up.
      */
@@ -492,7 +505,7 @@ int ReplSemiSyncMaster::disableMaster() {
 
     ack_container_.clear();
 
-    set_master_enabled(false);
+    set_source_enabled(false);
     LogErr(INFORMATION_LEVEL, ER_SEMISYNC_DISABLED_ON_SOURCE);
   }
 
@@ -501,7 +514,7 @@ int ReplSemiSyncMaster::disableMaster() {
   return 0;
 }
 
-ReplSemiSyncMaster::~ReplSemiSyncMaster() {
+ReplSemiSyncSource::~ReplSemiSyncSource() {
   if (init_done_) {
     mysql_mutex_destroy(&LOCK_binlog_);
   }
@@ -509,26 +522,26 @@ ReplSemiSyncMaster::~ReplSemiSyncMaster() {
   delete active_tranxs_;
 }
 
-void ReplSemiSyncMaster::lock() { mysql_mutex_lock(&LOCK_binlog_); }
+void ReplSemiSyncSource::lock() { mysql_mutex_lock(&LOCK_binlog_); }
 
-void ReplSemiSyncMaster::unlock() { mysql_mutex_unlock(&LOCK_binlog_); }
+void ReplSemiSyncSource::unlock() { mysql_mutex_unlock(&LOCK_binlog_); }
 
-void ReplSemiSyncMaster::add_slave() {
+void ReplSemiSyncSource::add_replica() {
   lock();
   rpl_semi_sync_source_clients++;
   unlock();
 }
 
-void ReplSemiSyncMaster::remove_slave() {
+void ReplSemiSyncSource::remove_replica() {
   lock();
   rpl_semi_sync_source_clients--;
 
   /* Only switch off if semi-sync is enabled and is on */
-  if (getMasterEnabled() && is_on()) {
+  if (getSourceEnabled() && is_on()) {
     /*
-      If user has chosen not to wait if no enough semi-sync slave available
-      and after a slave exists, turn off semi-semi master immediately if active
-      slaves are less then required slave numbers.
+      If user has chosen not to wait if no enough semi-sync replica available
+      and after a replica exists, turn off semi-semi source immediately if
+      active replicas are less then required replica numbers.
     */
     if ((rpl_semi_sync_source_clients ==
          rpl_semi_sync_source_wait_for_replica_count - 1) &&
@@ -547,16 +560,16 @@ void ReplSemiSyncMaster::remove_slave() {
   unlock();
 }
 
-bool ReplSemiSyncMaster::is_semi_sync_slave() {
+bool ReplSemiSyncSource::is_semi_sync_replica() {
   int null_value;
   long long val = 0;
   get_user_var_int("rpl_semi_sync_replica", &val, &null_value);
   return val;
 }
 
-void ReplSemiSyncMaster::reportReplyBinlog(const char *log_file_name,
+void ReplSemiSyncSource::reportReplyBinlog(const char *log_file_name,
                                            my_off_t log_file_pos) {
-  const char *kWho = "ReplSemiSyncMaster::reportReplyBinlog";
+  const char *kWho = "ReplSemiSyncSource::reportReplyBinlog";
   int cmp;
   bool can_release_threads = false;
   bool need_copy_send_pos = true;
@@ -564,15 +577,15 @@ void ReplSemiSyncMaster::reportReplyBinlog(const char *log_file_name,
   function_enter(kWho);
   mysql_mutex_assert_owner(&LOCK_binlog_);
 
-  if (!getMasterEnabled()) goto l_end;
+  if (!getSourceEnabled()) goto l_end;
 
   if (!is_on()) /* We check to see whether we can switch semi-sync ON. */
     try_switch_on(log_file_name, log_file_pos);
 
   /* The position should increase monotonically, if there is only one
-   * thread sending the binlog to the slave.
+   * thread sending the binlog to the replica.
    * In reality, to improve the transaction availability, we allow multiple
-   * sync replication slaves.  So, if any one of them get the transaction,
+   * sync replication replicas.  So, if any one of them get the transaction,
    * the transaction session in the primary can move forward.
    */
   if (reply_file_name_inited_) {
@@ -581,11 +594,11 @@ void ReplSemiSyncMaster::reportReplyBinlog(const char *log_file_name,
 
     /* If the requested position is behind the sending binlog position,
      * would not adjust sending binlog position.
-     * We based on the assumption that there are multiple semi-sync slave,
+     * We based on the assumption that there are multiple semi-sync replica,
      * and at least one of them shou/ld be up to date.
-     * If all semi-sync slaves are behind, at least initially, the primary
+     * If all semi-sync replicas are behind, at least initially, the primary
      * can find the situation after the waiting timeout.  After that, some
-     * slaves should catch up quickly.
+     * replicas should catch up quickly.
      */
     if (cmp < 0) {
       /* If the position is behind, do not copy it. */
@@ -632,15 +645,15 @@ l_end:
   function_exit(kWho, 0);
 }
 
-int ReplSemiSyncMaster::commitTrx(const char *trx_wait_binlog_name,
+int ReplSemiSyncSource::commitTrx(const char *trx_wait_binlog_name,
                                   my_off_t trx_wait_binlog_pos) {
-  const char *kWho = "ReplSemiSyncMaster::commitTrx";
+  const char *kWho = "ReplSemiSyncSource::commitTrx";
 
   function_enter(kWho);
   PSI_stage_info old_stage;
 
 #if defined(ENABLED_DEBUG_SYNC)
-  /* debug sync may not be initialized for a master */
+  /* debug sync may not be initialized for a source */
   if (current_thd->debug_sync_control)
     DEBUG_SYNC(current_thd, "rpl_semisync_source_commit_trx_before_lock");
 #endif
@@ -659,14 +672,14 @@ int ReplSemiSyncMaster::commitTrx(const char *trx_wait_binlog_name,
   THD_ENTER_COND(nullptr, thd_cond, &LOCK_binlog_,
                  &stage_waiting_for_semi_sync_ack_from_replica, &old_stage);
 
-  if (getMasterEnabled() && trx_wait_binlog_name) {
+  if (getSourceEnabled() && trx_wait_binlog_name) {
     struct timespec start_ts;
     struct timespec abstime;
     int wait_result;
 
     set_timespec(&start_ts, 0);
     /* This is the real check inside the mutex. */
-    if (!getMasterEnabled() || !is_on()) goto l_end;
+    if (!getSourceEnabled() || !is_on()) goto l_end;
 
     if (trace_level_ & kTraceDetail) {
       LogErr(INFORMATION_LEVEL, ER_SEMISYNC_SOURCE_TRX_WAIT_POS, kWho,
@@ -689,7 +702,7 @@ int ReplSemiSyncMaster::commitTrx(const char *trx_wait_binlog_name,
             ActiveTranx::compare(reply_file_name_, reply_file_pos_,
                                  trx_wait_binlog_name, trx_wait_binlog_pos);
         if (cmp >= 0) {
-          /* We have already sent the relevant binlog to the slave: no need to
+          /* We have already sent the relevant binlog to the replica: no need to
            * wait here.
            */
           if (trace_level_ & kTraceDetail)
@@ -707,9 +720,9 @@ int ReplSemiSyncMaster::commitTrx(const char *trx_wait_binlog_name,
         'Entry' object created for the transaction being committed and at a
         later stage it was enabled. In this case trx_wait_binlog_name and
         trx_wait_binlog_pos are set but the 'Entry' object is not present. Hence
-        dump thread will not wait for reply from slave and it will not update
+        dump thread will not wait for reply from replica and it will not update
         reply_file_name. In such case the committing transaction should not wait
-        for an ack from slave and it should be considered as an async
+        for an ack from replica and it should be considered as an async
         transaction.
       */
       if (!entry) {
@@ -750,7 +763,7 @@ int ReplSemiSyncMaster::commitTrx(const char *trx_wait_binlog_name,
 
       /* In semi-synchronous replication, we wait until the binlog-dump
        * thread has received the reply on the relevant binlog segment from the
-       * replication slave.
+       * replication replica.
        *
        * Let us suspend this thread to wait on the condition;
        * when replication has progressed far enough, we will release
@@ -831,29 +844,29 @@ int ReplSemiSyncMaster::commitTrx(const char *trx_wait_binlog_name,
   THD_EXIT_COND(nullptr, &old_stage);
   return function_exit(kWho, 0);
 }
-void ReplSemiSyncMaster::set_wait_no_replica(const void *val) {
+void ReplSemiSyncSource::set_wait_no_replica(const void *val) {
   lock();
   char set_switch = *static_cast<const char *>(val);
   if (set_switch == 0) {
     if ((rpl_semi_sync_source_clients == 0) && (is_on())) switch_off();
   } else {
-    if (!is_on() && getMasterEnabled()) force_switch_on();
+    if (!is_on() && getSourceEnabled()) force_switch_on();
   }
   unlock();
 }
 
-void ReplSemiSyncMaster::force_switch_on() { state_ = true; }
+void ReplSemiSyncSource::force_switch_on() { state_ = true; }
 
 /* Indicate that semi-sync replication is OFF now.
  *
  * What should we do when it is disabled?  The problem is that we want
- * the semi-sync replication enabled again when the slave catches up
- * later.  But, it is not that easy to detect that the slave has caught
+ * the semi-sync replication enabled again when the replica catches up
+ * later.  But, it is not that easy to detect that the replica has caught
  * up.  This is caused by the fact that MySQL's replication protocol is
- * asynchronous, meaning that if the master does not use the semi-sync
- * protocol, the slave would not send anything to the master.
- * Still, if the master is sending (N+1)-th event, we assume that it is
- * an indicator that the slave has received N-th event and earlier ones.
+ * asynchronous, meaning that if the source does not use the semi-sync
+ * protocol, the replica would not send anything to the source.
+ * Still, if the source is sending (N+1)-th event, we assume that it is
+ * an indicator that the replica has received N-th event and earlier ones.
  *
  * If semi-sync is disabled, all transactions still update the wait
  * position with the last position in binlog.  But no transactions will
@@ -862,8 +875,8 @@ void ReplSemiSyncMaster::force_switch_on() { state_ = true; }
  * up with last wait position.  If it does match, semi-sync will be
  * switched on again.
  */
-int ReplSemiSyncMaster::switch_off() {
-  const char *kWho = "ReplSemiSyncMaster::switch_off";
+int ReplSemiSyncSource::switch_off() {
+  const char *kWho = "ReplSemiSyncSource::switch_off";
 
   function_enter(kWho);
   state_ = false;
@@ -879,15 +892,15 @@ int ReplSemiSyncMaster::switch_off() {
   return function_exit(kWho, 0);
 }
 
-int ReplSemiSyncMaster::try_switch_on(const char *log_file_name,
+int ReplSemiSyncSource::try_switch_on(const char *log_file_name,
                                       my_off_t log_file_pos) {
-  const char *kWho = "ReplSemiSyncMaster::try_switch_on";
+  const char *kWho = "ReplSemiSyncSource::try_switch_on";
   bool semi_sync_on = false;
 
   function_enter(kWho);
 
   /* If the current sending event's position is larger than or equal to the
-   * 'largest' commit transaction binlog position, the slave is already
+   * 'largest' commit transaction binlog position, the replica is already
    * catching up now and we can switch semi-sync on here.
    * If commit_file_name_inited_ indicates there are no recent transactions,
    * we can enable semi-sync immediately.
@@ -911,17 +924,17 @@ int ReplSemiSyncMaster::try_switch_on(const char *log_file_name,
   return function_exit(kWho, 0);
 }
 
-int ReplSemiSyncMaster::reserveSyncHeader(unsigned char *header,
+int ReplSemiSyncSource::reserveSyncHeader(unsigned char *header,
                                           unsigned long size) {
-  const char *kWho = "ReplSemiSyncMaster::reserveSyncHeader";
+  const char *kWho = "ReplSemiSyncSource::reserveSyncHeader";
   function_enter(kWho);
 
   int hlen = 0;
   {
-    /* No enough space for the extra header, disable semi-sync master */
+    /* No enough space for the extra header, disable semi-sync source */
     if (sizeof(kSyncHeader) > size) {
       LogErr(WARNING_LEVEL, ER_SEMISYNC_NO_SPACE_IN_THE_PKT);
-      disableMaster();
+      disableSource();
       return 0;
     }
 
@@ -934,25 +947,25 @@ int ReplSemiSyncMaster::reserveSyncHeader(unsigned char *header,
   return function_exit(kWho, hlen);
 }
 
-int ReplSemiSyncMaster::updateSyncHeader(unsigned char *packet,
+int ReplSemiSyncSource::updateSyncHeader(unsigned char *packet,
                                          const char *log_file_name,
                                          my_off_t log_file_pos,
                                          uint32 server_id) {
-  const char *kWho = "ReplSemiSyncMaster::updateSyncHeader";
+  const char *kWho = "ReplSemiSyncSource::updateSyncHeader";
   int cmp = 0;
   bool sync = false;
 
-  /* If the semi-sync master is not enabled, do not request replies from the
-     slave.
+  /* If the semi-sync source is not enabled, do not request replies from the
+     replica.
    */
-  if (!getMasterEnabled()) return 0;
+  if (!getSourceEnabled()) return 0;
 
   function_enter(kWho);
 
   lock();
 
   /* This is the real check inside the mutex. */
-  if (!getMasterEnabled()) goto l_end;  // sync= false at this point in time
+  if (!getSourceEnabled()) goto l_end;  // sync= false at this point in time
 
   if (is_on()) {
     /* semi-sync is ON */
@@ -1014,9 +1027,9 @@ l_end:
   return function_exit(kWho, 0);
 }
 
-int ReplSemiSyncMaster::writeTranxInBinlog(const char *log_file_name,
+int ReplSemiSyncSource::writeTranxInBinlog(const char *log_file_name,
                                            my_off_t log_file_pos) {
-  const char *kWho = "ReplSemiSyncMaster::writeTranxInBinlog";
+  const char *kWho = "ReplSemiSyncSource::writeTranxInBinlog";
   int result = 0;
 
   function_enter(kWho);
@@ -1024,7 +1037,7 @@ int ReplSemiSyncMaster::writeTranxInBinlog(const char *log_file_name,
   lock();
 
   /* This is the real check inside the mutex. */
-  if (!getMasterEnabled()) goto l_end;
+  if (!getSourceEnabled()) goto l_end;
 
   /* Update the 'largest' transaction commit position seen so far even
    * though semi-sync is switched off.
@@ -1068,10 +1081,11 @@ l_end:
   return function_exit(kWho, result);
 }
 
-int ReplSemiSyncMaster::skipSlaveReply(const char *event_buf, uint32 server_id,
-                                       const char *skipped_log_file,
-                                       my_off_t skipped_log_pos) {
-  const char *kWho = "ReplSemiSyncMaster::skipSlaveReply";
+int ReplSemiSyncSource::skipReplicaReply(const char *event_buf,
+                                         uint32 server_id,
+                                         const char *skipped_log_file,
+                                         my_off_t skipped_log_pos) {
+  const char *kWho = "ReplSemiSyncSource::skipReplicaReply";
 
   function_enter(kWho);
 
@@ -1092,8 +1106,8 @@ l_end:
   return function_exit(kWho, 0);
 }
 
-int ReplSemiSyncMaster::readSlaveReply(NET *net, const char *event_buf) {
-  const char *kWho = "ReplSemiSyncMaster::readSlaveReply";
+int ReplSemiSyncSource::readReplicaReply(NET *net, const char *event_buf) {
+  const char *kWho = "ReplSemiSyncSource::readReplicaReply";
   int result = -1;
 
   function_enter(kWho);
@@ -1122,8 +1136,8 @@ l_end:
   return function_exit(kWho, result);
 }
 
-int ReplSemiSyncMaster::resetMaster() {
-  const char *kWho = "ReplSemiSyncMaster::resetMaster";
+int ReplSemiSyncSource::resetSource() {
+  const char *kWho = "ReplSemiSyncSource::resetSource";
   int result = 0;
 
   function_enter(kWho);
@@ -1152,7 +1166,7 @@ int ReplSemiSyncMaster::resetMaster() {
   return function_exit(kWho, result);
 }
 
-void ReplSemiSyncMaster::setExportStats() {
+void ReplSemiSyncSource::setExportStats() {
   lock();
 
   rpl_semi_sync_source_status = state_;
@@ -1170,11 +1184,11 @@ void ReplSemiSyncMaster::setExportStats() {
   unlock();
 }
 
-int ReplSemiSyncMaster::setWaitSlaveCount(unsigned int new_value) {
+int ReplSemiSyncSource::setWaitReplicaCount(unsigned int new_value) {
   const AckInfo *ackinfo = nullptr;
   int result = 0;
 
-  const char *kWho = "ReplSemiSyncMaster::updateWaitSlaves";
+  const char *kWho = "ReplSemiSyncSource::updateWaitReplicas";
   function_enter(kWho);
 
   lock();
@@ -1204,7 +1218,7 @@ const AckInfo *AckContainer::insert(int server_id, const char *log_file_name,
     goto l_end;
   }
 
-  /* Update the slave's ack position if it is in the ack array */
+  /* Update the replica's ack position if it is in the ack array */
   if (updateIfExist(server_id, log_file_name, log_file_pos) < m_size)
     goto l_end;
 
@@ -1218,7 +1232,9 @@ const AckInfo *AckContainer::insert(int server_id, const char *log_file_name,
     if (likely(min_ack == nullptr)) {
       m_greatest_ack.set(server_id, log_file_name, log_file_pos);
 
-      /* Remove all slaves which have minimum ack position from the ack array */
+      /*
+       * Remove all replicas which have minimum ack position from the ack array
+       */
       remove_all(log_file_name, log_file_pos);
 
       /* Don't insert current ack into container if it is the minimum ack. */
