@@ -27,10 +27,12 @@
 #define MYSQLD_MOCK_X_MOCK_SESSION_INCLUDED
 
 #include <memory>
-#include <mutex>
-#include <set>
 
+#include "mysql/harness/destination_endpoint.h"
+#include "mysql/harness/destination_socket.h"
+#include "mysql/harness/logging/logger.h"
 #include "mysql/harness/net_ts/impl/socket_constants.h"
+#include "mysql/harness/tls_server_context.h"
 #include "mysql_server_mock.h"
 #include "router/src/mock_server/src/mock_session.h"
 #include "router/src/mock_server/src/statement_reader.h"
@@ -85,8 +87,8 @@ class MySQLServerMockSessionX : public MySQLServerMockSession {
   using clock_type = std::chrono::steady_clock;
 
   MySQLServerMockSessionX(
-      ProtocolBase::socket_type client_sock,
-      ProtocolBase::endpoint_type client_ep, TlsServerContext &tls_server_ctx,
+      mysql_harness::DestinationSocket sock,
+      mysql_harness::DestinationEndpoint ep, TlsServerContext &tls_ctx,
       std::unique_ptr<StatementReaderBase> statement_processor,
       const bool debug_mode, bool with_tls);
 
@@ -135,6 +137,8 @@ class MySQLServerMockSessionX : public MySQLServerMockSession {
   clock_type::time_point start_time_{};
 
   net::steady_timer notice_timer_{protocol_.io_context()};
+
+  mysql_harness::logging::DomainLogger logger_;
 };
 
 }  // namespace server_mock
