@@ -16,8 +16,16 @@ if (mysqld.global.config_defaults_stored_is_null === undefined) {
   mysqld.global.config_defaults_stored_is_null = 0;
 }
 
+if (mysqld.global.server_version === undefined) {
+  // Let's keep the default server version as some known compatible version.
+  // If there is a need to some specific compatibility checks, this should be
+  // overwritten from the test.
+  mysqld.global.server_version = "8.3.0";
+}
+
 var options = {
   cluster_type: "gr",
+  bootstrap_target_type: "clusterset",
 
   metadata_schema_version: [2, 1, 0],
   clusterset_present: 1,
@@ -27,6 +35,7 @@ var options = {
   clusterset_simulate_cluster_not_found:
       mysqld.global.simulate_cluster_not_found,
   router_expected_target_cluster: mysqld.global.router_expected_target_cluster,
+  router_expected_local_cluster: mysqld.global.router_expected_local_cluster,
   group_replication_name:
       mysqld.global.clusterset_data
           .clusters[mysqld.global.clusterset_data.this_cluster_id]
@@ -53,6 +62,7 @@ var common_responses = common_stmts.prepare_statement_responses(
       "router_start_transaction",
       "router_commit",
       "router_rollback",
+      "get_routing_guidelines_version",
 
       // account verification
       //"router_select_metadata_v2_gr",
@@ -66,6 +76,7 @@ var common_responses = common_stmts.prepare_statement_responses(
       "router_clusterset_present",
       "router_clusterset_id_current",
       "router_clusterset_view_id",
+      "get_local_cluster_name",
     ],
     options);
 
@@ -77,9 +88,11 @@ var common_responses_regex = common_stmts.prepare_statement_responses_regex(
       "router_grant_on_pfs_db",
       "router_grant_on_routers",
       "router_grant_on_v2_routers",
+      "router_grant_on_router_stats",
       "router_update_router_options_in_metadata",
       "router_clusterset_cluster_info_by_name_unknown",
       "router_select_config_defaults_stored_clusterset",
+      "router_update_local_cluster_in_metadata",
     ],
     options);
 
@@ -100,7 +113,8 @@ var router_store_config_defaults_clusterset =
     auth: {
       username: "root",
       password: "fake-pass",
-    }
+    },
+    greeting: {server_version: mysqld.global.server_version}
   },
   stmts: function(stmt) {
     var res;
