@@ -1,4 +1,4 @@
-/* Copyright (c) 2022, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2022, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -80,7 +80,7 @@ int main(int argc, char **argv) {
     return EXIT_FAILURE;
   }
 
-  std::string_view arg_1(argv[1]);
+  std::string_view const arg_1(argv[1]);
 
   int signum;
 
