@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+Copyright (c) 2019, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -88,14 +88,7 @@ class NotifyTest : public RestApiComponentTest {
   }
 
   bool wait_signal_handler_ready(ProcessWrapper &router) {
-#ifdef _WIN32
-    UNREFERENCED_PARAMETER(router);
-    return true;
-#else
-    // log_reopen service reports readiness after the signal handler is
-    // initialized in the current implementation
-    return wait_log_contains(router, "ready 'log_reopen'", 5s);
-#endif
+    return wait_log_contains(router, "ready 'signal_handler'", 5s);
   }
 
   std::string create_config_file(
@@ -276,11 +269,12 @@ TEST_F(NotifyTest, NotifyReadyMetadataCache) {
       "node)");
   auto md_server_port = port_pool_.get_next_available();
   auto md_server_http_port = port_pool_.get_next_available();
-  const std::string json_metadata =
-      get_data_dir().join("metadata_1_node_repeat_v2_gr.js").str();
 
-  /*auto &metadata_server = */ launch_mysql_server_mock(
-      json_metadata, md_server_port, EXIT_SUCCESS, false, md_server_http_port);
+  /*auto &metadata_server = */ mock_server_spawner().spawn(
+      mock_server_cmdline("metadata_1_node_repeat_v2_gr.js")
+          .port(md_server_port)
+          .http_port(md_server_http_port)
+          .args());
 
   SCOPED_TRACE(
       "// Launch the Router with the routing and metadata_cache configuration");
@@ -356,11 +350,12 @@ TEST_F(NotifyTest, NotifyReadyManyPlugins) {
       "node)");
   auto md_server_port = port_pool_.get_next_available();
   auto md_server_http_port = port_pool_.get_next_available();
-  const std::string json_metadata =
-      get_data_dir().join("metadata_1_node_repeat_v2_gr.js").str();
 
-  /*auto &metadata_server = */ launch_mysql_server_mock(
-      json_metadata, md_server_port, EXIT_SUCCESS, false, md_server_http_port);
+  /*auto &metadata_server = */ mock_server_spawner().spawn(
+      mock_server_cmdline("metadata_1_node_repeat_v2_gr.js")
+          .port(md_server_port)
+          .http_port(md_server_http_port)
+          .args());
 
   SCOPED_TRACE("// Launch the Router with multiple plugins");
 
@@ -904,11 +899,15 @@ TEST_P(NotifyBootstrapNotAffectedTest, NotifyBootstrapNotAffected) {
   TempDirectory temp_test_dir;
 
   SCOPED_TRACE("// Launch our metadata server we bootstrap against");
-  const auto trace_file = get_data_dir().join("bootstrap_gr.js").str();
   const auto metadata_server_port = port_pool_.get_next_available();
   const auto http_port = port_pool_.get_next_available();
-  /*auto &md_server =*/ProcessManager::launch_mysql_server_mock(
-      trace_file, metadata_server_port, EXIT_SUCCESS, true, http_port);
+
+  /*auto &metadata_server = */ mock_server_spawner().spawn(
+      mock_server_cmdline("bootstrap_gr.js")
+          .port(metadata_server_port)
+          .http_port(http_port)
+          .args());
+
   set_mock_metadata(http_port, "00000000-0000-0000-0000-0000000000g1",
                     classic_ports_to_gr_nodes({metadata_server_port}), 0,
                     {metadata_server_port});
