@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2020, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,6 +42,7 @@
 #include <rapidjson/schema.h>
 #include <rapidjson/stringbuffer.h>
 
+#include "mock_server_testutils.h"
 #include "mysqlrouter/cluster_metadata.h"
 #include "rest_api_testutils.h"
 #include "router_component_test.h"
@@ -70,6 +72,8 @@ class RouterComponentClusterSetTest : public RestApiComponentTest {
 
     ProcessWrapper *process{nullptr};
     bool is_read_replica{false};
+    std::optional<bool> is_hidden;
+    std::optional<bool> disconnect_when_hidden;
   };
 
   struct ClusterData {
@@ -83,10 +87,11 @@ class RouterComponentClusterSetTest : public RestApiComponentTest {
     bool invalid{false};
 
     std::vector<ClusterNode> nodes;
+    std::vector<GRNode> gr_nodes;
     unsigned primary_node_id;
   };
 
-  struct ClusterSetData {
+  struct ClusterSetTopology {
     std::string uuid{"clusterset-uuid"};
     std::vector<ClusterData> clusters;
     unsigned primary_cluster_id;
@@ -132,17 +137,29 @@ class RouterComponentClusterSetTest : public RestApiComponentTest {
   constexpr static unsigned kClustersNumber = 3;
   constexpr static unsigned kGRNodesPerClusterNumber = 3;
 
-  void create_clusterset(
-      uint64_t view_id, int target_cluster_id, int primary_cluster_id,
-      const std::string &tracefile, const std::string &router_options = "",
-      const std::string &expected_target_cluster = ".*",
-      bool simulate_cluster_not_found = false,
-      bool use_gr_notifications = false,
-      const std::vector<size_t> &read_replicas_number = {},
-      const mysqlrouter::MetadataSchemaVersion &metadata_version =
-          mysqlrouter::MetadataSchemaVersion{2, 2, 0});
+  struct ClusterSetOptions {
+    uint64_t view_id{0};
+    int target_cluster_id{0};
+    int primary_cluster_id{0};
+    unsigned int primary_node_id{0};
+    std::string tracefile;
+    std::string router_options{""};
+    std::string expected_target_cluster{".*"};
+    std::string expected_local_cluster{".*"};
+    bool simulate_cluster_not_found{false};
+    bool simulate_config_defaults_stored_is_null{false};
+    bool simulate_router_options_no_rows{false};
+    bool use_gr_notifications{false};
+    std::vector<size_t> gr_nodes_number{3, 3, 3};
+    std::vector<size_t> read_replicas_number{};
+    mysqlrouter::MetadataSchemaVersion metadata_version{
+        mysqlrouter::MetadataSchemaVersion{2, 4, 0}};
+    ClusterSetTopology topology;
+  };
 
-  void change_clusterset_primary(ClusterSetData &clusterset_data,
+  void create_clusterset(ClusterSetOptions &cs_options);
+
+  void change_clusterset_primary(ClusterSetTopology &cs_topology,
                                  const unsigned new_primary_id);
 
   void add_json_str_field(JsonValue &json_doc, const std::string &field,
@@ -152,28 +169,22 @@ class RouterComponentClusterSetTest : public RestApiComponentTest {
                           const int value);
 
   void add_clusterset_data_field(JsonValue &json_doc, const std::string &field,
-                                 const ClusterSetData &clusterset_data,
+                                 const ClusterSetTopology &cs_topology,
                                  const unsigned this_cluster_id,
                                  const unsigned this_node_id);
 
-  void set_mock_metadata(
-      uint64_t view_id, unsigned this_cluster_id, unsigned this_node_id,
-      unsigned target_cluster_id, uint16_t http_port,
-      const ClusterSetData &clusterset_data,
-      const std::string &router_options = "",
-      const std::string &expected_target_cluster = ".*",
-      const mysqlrouter::MetadataSchemaVersion &metadata_version = {2, 1, 0},
-      bool simulate_cluster_not_found = false);
+  void set_mock_clusterset_metadata(
+      uint16_t http_port, unsigned this_cluster_id, unsigned this_node_id,
+      const ClusterSetOptions &cs_options,
+      const std::optional<std::string> &routing_guidelines = std::nullopt);
 
   void set_mock_metadata_on_all_cs_nodes(
-      uint64_t view_id, unsigned target_cluster_id,
-      const ClusterSetData &clusterset_data,
-      const std::string &router_options = "",
-      const std::string &expected_target_cluster = ".*",
-      const mysqlrouter::MetadataSchemaVersion &metadata_version = {2, 1, 0},
-      bool simulate_cluster_not_found = false);
+      const ClusterSetOptions &cs_options,
+      const std::optional<std::string> &routing_guidelines = std::nullopt);
+
+  std::string get_node_attributes_as_string(
+      const RouterComponentClusterSetTest::ClusterNode &node_data) const;
 
-  ClusterSetData clusterset_data_;
   JsonAllocator json_allocator;
 };
 
