@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -212,8 +212,7 @@ bool initialize(THD *thd);
 
   @return       Upon failure, return true, otherwise false.
 */
-
-bool restart(THD *thd);
+bool restart_dictionary(THD *thd);
 
 /**
   Iterate through all the plugins, and store IS table meta data
@@ -234,16 +233,6 @@ bool store_plugin_IS_table_metadata(THD *thd);
 */
 bool setup_dd_objects_and_collations(THD *thd);
 
-/**
-  This function is used in case of crash during upgrade.
-  It tries to initialize dictionary and calls DDSE_dict_recover.
-  InnoDB should do the recovery and empty undo log. Upgrade
-  process will do the cleanup and exit.
-
-  @param thd    Thread context.
-*/
-void recover_innodb_upon_upgrade(THD *thd);
-
 /**
   Initialize InnoDB for
   - creating new data directory : InnoDB creates system tablespace and
@@ -271,13 +260,12 @@ bool DDSE_dict_init(THD *thd, dict_init_mode_t dict_init_mode, uint version);
   and in place upgrade.
 
   @param thd            Thread context.
-  @param is_dd_upgrade  Flag to indicate if it is in place upgrade.
   @param d              Dictionary instance
 
   @return       Upon failure, return true, otherwise false.
 
 */
-bool initialize_dictionary(THD *thd, bool is_dd_upgrade, Dictionary_impl *d);
+bool initialize_dictionary(THD *thd, Dictionary_impl *d);
 
 }  // namespace bootstrap
 
@@ -396,11 +384,10 @@ bool update_properties(THD *thd, const std::set<String_type> *create_set,
   This function is used during initialize and during server upgrade.
 
   @param thd               Thread context.
-  @param is_dd_upgrade_57  Flag to indicate if it is an upgrade from 5.7.
 
   @return       Upon failure, return true, otherwise false.
 */
-bool update_versions(THD *thd, bool is_dd_upgrade_57);
+bool update_versions(THD *thd);
 
 }  // namespace dd
 #endif  // DD__BOOTSTRAPPER_INCLUDED
