@@ -74,6 +74,7 @@
 #include "sql/sql_const.h"
 #include "sql/sql_error.h"
 #include "sql/sql_executor.h"
+#include "sql/sql_foreign_key_constraint.h"
 #include "sql/sql_lex.h"
 #include "sql/sql_list.h"
 #include "sql/sql_opt_exec_shared.h"
@@ -122,6 +123,12 @@ bool DeleteCurrentRowAndProcessTriggers(THD *thd, TABLE *table,
     }
   }
 
+  if (use_sql_fk_checks_for_table(thd, table)) {
+    if (check_all_child_fk_ref(thd, table, enum_fk_dml_type::FK_DELETE)) {
+      return thd->is_error();
+    }
+  }
+
   if (const int delete_error = table->file->ha_delete_row(table->record[0]);
       delete_error != 0) {
     myf error_flags = MYF(0);
