@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,6 +25,8 @@
 /* Copy data from a text file to table */
 
 #include "sql/sql_load.h"
+#include "my_sqlcommand.h"
+#include "sql/sql_rename.h"
 
 #include <fcntl.h>
 #include <limits.h>
@@ -36,8 +39,8 @@
 #include <algorithm>
 #include <atomic>
 #include <limits>
+#include <sstream>
 
-#include "libbinlogevents/include/load_data_events.h"
 #include "my_base.h"
 #include "my_bitmap.h"
 #include "my_dbug.h"
@@ -47,6 +50,7 @@
 #include "my_macros.h"
 #include "my_sys.h"
 #include "my_thread_local.h"
+#include "mysql/binlog/event/load_data_events.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/my_loglevel.h"
 #include "mysql/psi/mysql_file.h"
@@ -55,10 +59,16 @@
 #include "mysql/thread_type.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
+
 #include "nulls.h"
+#include "scope_guard.h"
+
 #include "sql/auth/auth_acls.h"
 #include "sql/auth/auth_common.h"
 #include "sql/binlog.h"
+#include "sql/dd/cache/dictionary_client.h"  // dd::cache::Dictionary_client
+#include "sql/dd/dd_table.h"                 // dd::table_storage_engine
+#include "sql/dd/types/abstract_table.h"
 #include "sql/derror.h"
 #include "sql/error_handler.h"  // Ignore_error_handler
 #include "sql/field.h"
@@ -77,16 +87,22 @@
 #include "sql/rpl_rli.h"   // Relay_log_info
 #include "sql/sql_base.h"  // fill_record_n_invoke_before_triggers
 #include "sql/sql_class.h"
+#include "sql/sql_data_change.h"
 #include "sql/sql_error.h"
+#include "sql/sql_exchange.h"
+#include "sql/sql_foreign_key_constraint.h"
 #include "sql/sql_insert.h"  // check_that_all_fields_are_given_values,
 #include "sql/sql_lex.h"
 #include "sql/sql_list.h"
 #include "sql/sql_show.h"
+#include "sql/sql_table.h"
 #include "sql/sql_view.h"  // check_key_in_view
 #include "sql/system_variables.h"
 #include "sql/table.h"
 #include "sql/table_trigger_dispatcher.h"  // Table_trigger_dispatcher
+#include "sql/thd_raii.h"                  // Disable_autocommit_guard
 #include "sql/thr_malloc.h"
+#include "sql/transaction.h"
 #include "sql/transaction_info.h"
 #include "sql/trigger_def.h"
 #include "sql_string.h"
@@ -94,6 +110,8 @@
 #include "strxnmov.h"
 #include "thr_lock.h"
 
+#include <mysql/components/services/bulk_load_service.h>
+
 class READ_INFO;
 
 using std::max;
@@ -191,6 +209,614 @@ class READ_INFO {
   }
 };
 
+/**
+  Truncate to create a new table for BULK LOAD. The transaction is not
+  committed and rolls back if bulk load fails.
+
+  @param thd Current thread
+  @param table_ref table reference
+  @param table_def dd table
+  @returns true if error
+*/
+bool Sql_cmd_load_table::truncate_table_for_bulk_load(
+    THD *thd, Table_ref *const table_ref, dd::Table *table_def) {
+  DBUG_TRACE;
+
+  HA_CREATE_INFO dummy_create_info;
+  // Create a path to the table, but without a extension
+  char path[FN_REFLEN + 1];
+  build_table_filename(path, sizeof(path) - 1, table_ref->db,
+                       table_ref->table_name, "", 0);
+
+  // Truncate table, attempt to reconstruct the table.
+  if (ha_create_table(thd, path, table_ref->db, table_ref->table_name,
+                      &dummy_create_info, true, false, table_def) != 0) {
+    return true;
+  }
+  return false;
+}
+
+/**
+  Check bulk load parameters for limits.
+  @param thd Current thread
+  @returns true if error
+*/
+bool Sql_cmd_load_table::check_bulk_load_parameters(THD *thd) {
+  DBUG_TRACE;
+  /* On error, this function needs to return true. */
+  const bool error = true;
+
+  /* First check if bulk loader component is available. */
+  my_h_service svc = nullptr;
+
+  if (srv_registry->acquire("bulk_load_driver", &svc)) {
+    my_error(ER_NOT_SUPPORTED_YET, MYF(0), "Bulk Load");
+    return error;
+  }
+  srv_registry->release(svc);
+
+  auto sec_ctx = thd->security_context();
+
+  if (m_bulk_source == LOAD_SOURCE_S3) {
+    if (!sec_ctx->has_global_grant(STRING_WITH_LEN("LOAD_FROM_S3")).first) {
+      my_error(ER_SPECIFIC_ACCESS_DENIED_ERROR, MYF(0), "LOAD_FROM_S3");
+      return true;
+    }
+  }
+
+  if (m_bulk_source == LOAD_SOURCE_URL) {
+    if (!sec_ctx->has_global_grant(STRING_WITH_LEN("LOAD_FROM_URL")).first) {
+      my_error(ER_SPECIFIC_ACCESS_DENIED_ERROR, MYF(0), "LOAD_FROM_URL");
+      return true;
+    }
+  }
+  const String *escaped = m_exchange.field.escaped;
+  const String *enclosed = m_exchange.field.enclosed;
+
+  /* Check for single byte separator. */
+  if (escaped->length() > 1 || enclosed->length() > 1) {
+    my_error(ER_WRONG_FIELD_TERMINATORS, MYF(0));
+    return true;
+  }
+
+  /* Report problems with non-ASCII separators */
+  const String *field_term = m_exchange.field.field_term;
+  const String *line_term = m_exchange.line.line_term;
+
+  if (!escaped->is_ascii() || !enclosed->is_ascii() ||
+      !field_term->is_ascii() || !line_term->is_ascii()) {
+    push_warning(thd, Sql_condition::SL_WARNING,
+                 WARN_NON_ASCII_SEPARATOR_NOT_IMPLEMENTED,
+                 ER_THD(thd, WARN_NON_ASCII_SEPARATOR_NOT_IMPLEMENTED));
+  }
+  return false;
+}
+
+/**
+  Validate table for bulk load operation.
+
+  @param       thd         Thread handle.
+  @param       table_ref   Table_ref instance of a table.
+  @param       table_def   DD table instance of a table.
+  @param[out]  hton        Handlerton instance of table's SE.
+
+  @returns false if bulk load can be executed on table.
+*/
+bool Sql_cmd_load_table::validate_table_for_bulk_load(
+    THD *thd, Table_ref *const table_ref, dd::Table *table_def,
+    handlerton **hton) {
+  // Table belongs to engine supporting bulk load.
+  (*hton) = nullptr;
+  if (dd::table_storage_engine(thd, table_def, hton)) {
+    /* Error is already raised. */
+    return true;
+  }
+  if ((*hton) == nullptr ||
+      !ha_check_storage_engine_flag(*hton, HTON_SUPPORTS_BULK_LOAD)) {
+    my_error(ER_NOT_SUPPORTED_YET, MYF(0), "Bulk Load is not supported for SE");
+    return true;
+  }
+
+  /*
+    Engine supporting bulk should also supports atomic DDL and
+    truncate by recreate.
+  */
+  assert((*hton)->flags & HTON_SUPPORTS_ATOMIC_DDL);
+  assert((*hton)->flags & HTON_CAN_RECREATE);
+
+  if (is_temporary_table(table_ref)) {
+    my_error(ER_NOT_SUPPORTED_YET, MYF(0),
+             "Temporary Table with LOAD ALGORITHM = BULK");
+    return true;
+  }
+
+  if (table_ref->is_view() || !table_ref->is_insertable() ||
+      !table_ref->is_updatable()) {
+    my_error(ER_NON_INSERTABLE_TABLE, MYF(0), table_ref->alias, "BULK LOAD");
+    return true;
+  }
+
+  if (table_ref->table->part_info != nullptr) {
+    my_error(ER_NOT_SUPPORTED_YET, MYF(0), "Partitioned Table");
+    return true;
+  }
+
+  if (!table_ref->table->file->bulk_load_check(thd)) {
+    /* SE already raises the error. */
+    return true;
+  }
+
+  /*
+    Skip Foreign Key and Secondary Engine check. They would eventually fail
+    during bulk load execution and rollback.
+  */
+
+  return false;
+}
+
+bool Sql_cmd_load_table::rename_table_for_incremental_bulk_load(
+    THD *thd, const std::string &schema_name, const std::string &old_table_name,
+    const std::string &new_table_name) {
+  close_thread_tables(thd);
+
+  Table_ref old_table_ref{};
+  old_table_ref.table_name = old_table_name.c_str();
+  old_table_ref.table_name_length = old_table_name.length();
+  old_table_ref.db = schema_name.c_str();
+  old_table_ref.db_length = schema_name.length();
+  old_table_ref.alias = old_table_ref.table_name;
+  Table_ref new_table_ref{};
+  new_table_ref.table_name = new_table_name.c_str();
+  new_table_ref.table_name_length = new_table_name.length();
+  new_table_ref.db = schema_name.c_str();
+  new_table_ref.db_length = schema_name.length();
+  new_table_ref.alias = new_table_ref.table_name;
+  new_table_ref.next_local = nullptr;
+  old_table_ref.next_local = &new_table_ref;
+
+  return mysql_rename_tables(thd, &old_table_ref);
+}
+
+bool Sql_cmd_load_table::duplicate_table_for_bulk_load(
+    THD *thd, std::string &temp_name, const std::string &schema_name,
+    Table_ref *new_table_ref) {
+  auto *original_table_ref = thd->lex->query_tables;
+  temp_name = original_table_ref->table->file
+                  ->bulk_load_generate_temporary_table_name();
+
+  new_table_ref->table_name = temp_name.c_str();
+  new_table_ref->table_name_length = temp_name.length();
+  new_table_ref->db = schema_name.c_str();
+  new_table_ref->db_length = schema_name.length();
+  new_table_ref->alias = new_table_ref->table_name;
+  HA_CREATE_INFO info;
+  info.init_create_options_from_share(original_table_ref->table->s, 0);
+
+  new_table_ref->open_strategy = Table_ref::OPEN_FOR_CREATE;
+  MDL_REQUEST_INIT(&new_table_ref->mdl_request, MDL_key::TABLE,
+                   new_table_ref->db, new_table_ref->table_name, MDL_EXCLUSIVE,
+                   MDL_TRANSACTION);
+
+  if (lock_table_names(thd, new_table_ref, nullptr,
+                       thd->variables.lock_wait_timeout, 0)) {
+    return true;
+  }
+  close_thread_tables(thd);
+  original_table_ref->table = nullptr;
+  thd->lex->create_info = &info;
+  thd->lex->sql_command = SQLCOM_CREATE_TABLE;
+  auto res = mysql_create_like_table(thd, new_table_ref, original_table_ref,
+                                     &info, MYSQL_OPEN_HAS_MDL_LOCK, true);
+  thd->lex->sql_command = SQLCOM_LOAD;
+  return res;
+}
+
+/**
+  Execute BULK LOAD DATA
+  @param thd Current thread.
+  @returns true if error
+*/
+bool Sql_cmd_load_table::execute_bulk(THD *thd) {
+  DBUG_TRACE;
+
+  if (check_bulk_load_parameters(thd)) {
+    return true;
+  }
+
+  /* Disallow Bulk Load if the table is locked with LOCK TABLE. */
+  if (thd->locked_tables_mode != LTM_NONE) {
+    my_error(ER_LOCK_OR_ACTIVE_TRANSACTION, MYF(0));
+    return true;
+  }
+
+  Table_ref *table_ref = thd->lex->query_tables;
+
+  uint counter;
+  // Acquire MDL lock on table, BACKUP_LOCK and GLOBAL lock objects.
+  if (open_tables(thd, &table_ref, &counter, 0)) {
+    return true;
+  }
+
+  /* Truncate requires disabling auto commit guard. */
+  const Disable_autocommit_guard autocommit_guard(thd);
+  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+
+  THD_STAGE_INFO(thd, stage_executing);
+
+  // Acquire DD instance for modification. Truncate operation modifies DD
+  // table instance.
+  dd::Table *table_def = nullptr;
+
+  if (thd->dd_client()->acquire_for_modification(
+          table_ref->db, table_ref->table_name, &table_def)) {
+    return true;
+  }
+
+  if (table_def == nullptr) {
+    my_error(ER_NO_SUCH_TABLE, MYF(0), table_ref->db, table_ref->table_name);
+    return true;
+  }
+
+  handlerton *hton = nullptr;
+  if (validate_table_for_bulk_load(thd, table_ref, table_def, &hton)) {
+    return true;
+  }
+
+  Bulk_source src = Bulk_source::LOCAL;
+
+  switch (m_bulk_source) {
+    case LOAD_SOURCE_URL:
+      src = Bulk_source::OCI;
+      break;
+
+    case LOAD_SOURCE_S3:
+      src = Bulk_source::S3;
+      break;
+
+    case LOAD_SOURCE_FILE:
+      src = Bulk_source::LOCAL;
+      break;
+  }
+
+  std::string file_name_arg(m_exchange.file_name);
+
+  Bulk_load_file_info info(src, file_name_arg, m_file_count);
+
+  {
+    std::string error;
+    if (!info.parse(error)) {
+      my_error(ER_BULK_PARSER_ERROR, MYF(0), error.c_str());
+      return false;
+    }
+  }
+
+  if (src == Bulk_source::LOCAL) {
+    char name[FN_REFLEN];
+
+    Table_ref *const table_list = thd->lex->query_tables;
+    const char *db = table_list->db;
+    const char *tdb = thd->db().str ? thd->db().str : db;
+
+    if (!dirname_length(info.m_file_prefix.c_str())) {
+      strxnmov(name, FN_REFLEN - 1, mysql_real_data_home, tdb, NullS);
+      (void)fn_format(name, info.m_file_prefix.c_str(), name, "",
+                      MY_RELATIVE_PATH | MY_UNPACK_FILENAME);
+    } else {
+      (void)fn_format(
+          name, info.m_file_prefix.c_str(), mysql_real_data_home, "",
+          MY_RELATIVE_PATH | MY_UNPACK_FILENAME | MY_RETURN_REAL_PATH);
+    }
+
+    if (!is_secure_file_path(name)) {
+      /* Read only allowed from within dir specified by secure_file_priv */
+      my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0), "--secure-file-priv");
+      return false;
+    }
+    /* Replace relative path. */
+    if (!test_if_hard_path(info.m_file_prefix.c_str())) {
+      info.m_file_prefix.assign(name);
+    }
+  }
+
+  Table_ref new_table_ref{};
+  Table_ref *new_table_ref_ptr = &new_table_ref;
+
+  std::string original_name = table_ref->table_name;
+  std::string temp_name{};
+  std::string schema_name(table_ref->table->s->db.str,
+                          table_ref->table->s->db.length);
+  Disable_binlog_guard disable_binlog(thd);
+
+  bool success = false;
+  bool new_table_opened = false;
+  // Actions needed to cleanup before leaving scope.
+  auto cleanup_guard = create_scope_guard([&]() {
+    THD_STAGE_INFO(thd, stage_end);
+    if (m_non_empty_table && success && !info.m_is_dryrun) {
+      auto final_temp_name =
+          table_ref->table->file->bulk_load_generate_temporary_table_name();
+      Table_ref final_table_ref{};
+      final_table_ref.table_name = final_temp_name.c_str();
+      final_table_ref.table_name_length = final_temp_name.length();
+      final_table_ref.db = schema_name.c_str();
+      final_table_ref.db_length = schema_name.length();
+      MDL_REQUEST_INIT(&final_table_ref.mdl_request, MDL_key::TABLE,
+                       final_table_ref.db, final_table_ref.table_name,
+                       MDL_EXCLUSIVE, MDL_TRANSACTION);
+      if (lock_table_names(thd, &final_table_ref, nullptr,
+                           thd->variables.lock_wait_timeout, 0)) {
+        success = false;
+      } else {
+        // mysql_create_like_table  opens the table under the hood so we close
+        // it here for now.
+        rename_table_for_incremental_bulk_load(thd, schema_name, original_name,
+                                               final_temp_name);
+        rename_table_for_incremental_bulk_load(thd, schema_name, temp_name,
+                                               original_name);
+
+        Table_ref old_table_ref;
+
+        old_table_ref.table_name = final_temp_name.c_str();
+        old_table_ref.table_name_length = final_temp_name.length();
+        old_table_ref.db = schema_name.c_str();
+        old_table_ref.db_length = schema_name.length();
+        old_table_ref.alias = old_table_ref.table_name;
+        close_thread_tables(thd);
+        mysql_rm_table(thd, &old_table_ref, false, false);
+      }
+    }
+
+    close_thread_tables(thd);
+    // End transaction
+    if (success) {
+      success = !(trans_commit_stmt(thd) || trans_commit_implicit(thd));
+    }
+    if (!success) {
+      trans_rollback_stmt(thd);
+      trans_rollback_implicit(thd);
+    }
+    tdc_remove_table(thd, TDC_RT_REMOVE_ALL, table_ref->db,
+                     table_ref->table_name, false);
+    if (m_non_empty_table && !info.m_is_dryrun) {
+      tdc_remove_table(thd, TDC_RT_REMOVE_ALL, new_table_ref.db,
+                       new_table_ref.table_name, false);
+    }
+
+    // Post DDL action for truncate (and rename, create and remove tables during
+    // incremental bulk load). */
+    if (hton != nullptr && hton->post_ddl) {
+      hton->post_ddl(thd);
+    }
+  });
+
+  if (!table_ref->table->file->is_table_empty()) {
+    m_non_empty_table = true;
+  }
+
+  if (m_non_empty_table && !info.m_is_dryrun) {
+    if (duplicate_table_for_bulk_load(thd, temp_name, schema_name,
+                                      &new_table_ref)) {
+      my_error(ER_INTERNAL_ERROR, MYF(0), "BULK LOAD: Duplicate table failed");
+      success = false;
+      return false;
+    }
+  }
+
+  /*
+    We need to close the table and reset the table_ref so that it can be
+    used to re-open the table after truncate.
+  */
+  close_thread_tables(thd);
+  tdc_remove_table(thd, TDC_RT_REMOVE_ALL, table_ref->db, table_ref->table_name,
+                   false);
+  table_ref->table = nullptr;
+  if (m_non_empty_table && !info.m_is_dryrun) {
+    tdc_remove_table(thd, TDC_RT_REMOVE_ALL, new_table_ref.db,
+                     new_table_ref.table_name, false);
+    new_table_ref.table = nullptr;
+  }
+
+  /*
+    We start bulk ingestion by truncating the table. We then continue the same
+    transaction loading data and finishing with a commit or rollback.
+    1. At this point the table is validated to not have any user data. It is
+       thus safe to truncate.
+    2. Truncate command creates a new space for the data to be loaded and the
+       atomicity guarantee is provided by the truncate DDL log.
+       a. Commit (Success case): Old tablespace is removed and we have the new
+          one loaded with user data. The new space id is committed in DD SE
+          private data.
+       b. Rollback (Unsuccessful case): New tablespace with partially loaded
+          data is removed and old tablespace remains. Rollback brings back the
+          old tablespace ID in DD SE private data. Cached objects are removed.
+  */
+  if (!m_non_empty_table &&
+      truncate_table_for_bulk_load(thd, table_ref, table_def)) {
+    my_error(ER_INTERNAL_ERROR, MYF(0), "BULK LOAD: Truncate failed");
+    success = false;
+    return true;
+  }
+
+  /*
+  Open the table after truncate. Here we open the destination table, on which
+  we already have an exclusive metadata lock.
+  */
+  if (open_tables(thd, &table_ref, &counter, MYSQL_OPEN_HAS_MDL_LOCK)) {
+    my_error(ER_INTERNAL_ERROR, MYF(0), "BULK LOAD: open_tables failed");
+    success = false;
+    return true;
+  }
+
+  if (m_non_empty_table && !info.m_is_dryrun) {
+    new_table_opened = !open_tables(thd, &new_table_ref_ptr, &counter,
+                                    MYSQL_OPEN_HAS_MDL_LOCK);
+    if (!new_table_opened) {
+      my_error(ER_INTERNAL_ERROR, MYF(0),
+               "BULK LOAD: open_tables failed on duplicate table");
+      success = false;
+      return true;
+    }
+  }
+
+  size_t affected_rows = 0;
+  if (!m_non_empty_table && !bulk_driver_service(thd, table_ref->table, nullptr,
+                                                 info, src, affected_rows)) {
+    my_error(ER_INTERNAL_ERROR, MYF(0),
+             "BULK LOAD: bulk_driver_service failed");
+    success = false;
+    return true;
+  }
+
+  if (m_non_empty_table) {
+    auto *duplicate_table =
+        info.m_is_dryrun ? nullptr : new_table_ref_ptr->table;
+    if (!bulk_driver_service(thd, table_ref->table, duplicate_table, info, src,
+                             affected_rows)) {
+      my_error(ER_INTERNAL_ERROR, MYF(0),
+               "BULK LOAD: bulk_driver_service failed");
+      success = false;
+      return true;
+    }
+  }
+
+  success = true;
+  char ok_message[512];
+  snprintf(
+      ok_message, sizeof(ok_message), ER_THD(thd, ER_LOAD_INFO),
+      static_cast<long>(affected_rows), 0L, 0L,
+      static_cast<long>(thd->get_stmt_da()->current_statement_cond_count()));
+
+  my_ok(thd, affected_rows, 0LL, ok_message);
+
+  return false;
+}
+
+bool Sql_cmd_load_table::bulk_driver_service(THD *thd, const TABLE *sql_table,
+                                             const TABLE *duplicate_table,
+                                             const Bulk_load_file_info &info,
+                                             Bulk_source src,
+                                             size_t &affected_rows) {
+  std::string lowercase(m_compression_algorithm_string.str,
+                        m_compression_algorithm_string.length);
+
+  std::transform(lowercase.begin(), lowercase.end(), lowercase.begin(),
+                 ::tolower);
+  Bulk_compression_algorithm compression_algorithm =
+      Bulk_compression_algorithm::NONE;
+  if (m_compression_algorithm_string.length == 0) {
+    compression_algorithm = Bulk_compression_algorithm::NONE;
+  } else if (lowercase == "zstd") {
+    compression_algorithm = Bulk_compression_algorithm::ZSTD;
+  } else {
+    std::stringstream ss;
+    ss << "Invalid compression algorithm: "
+       << std::string(m_compression_algorithm_string.str,
+                      m_compression_algorithm_string.length);
+    my_error(ER_WRONG_USAGE, MYF(0), "LOAD DATA with BULK Algorithm",
+             ss.str().c_str());
+    return false;
+  }
+
+  my_h_service svc = nullptr;
+
+  if (srv_registry->acquire("bulk_load_driver", &svc)) {
+    my_error(ER_NOT_SUPPORTED_YET, MYF(0), "Bulk Load");
+    return false;
+  }
+
+  DEBUG_SYNC(thd, "bulk_load_inuse");
+
+  auto load_driver = reinterpret_cast<SERVICE_TYPE(bulk_load_driver) *>(svc);
+
+  auto load_handle = load_driver->create_bulk_loader(
+      thd, thd->thread_id(), sql_table, duplicate_table, src,
+      (m_exchange.file_info.cs != nullptr) ? m_exchange.file_info.cs
+                                           : thd->variables.collation_database);
+
+  /* Set schema, table, file name string options. */
+  std::string schema_name(sql_table->s->db.str, sql_table->s->db.length);
+  load_driver->set_string(load_handle, Bulk_string::SCHEMA_NAME, schema_name);
+
+  std::string original_table_name(sql_table->s->table_name.str,
+                                  sql_table->s->table_name.length);
+  load_driver->set_string(load_handle, Bulk_string::SQL_TABLE_NAME,
+                          original_table_name);
+
+  if (duplicate_table != nullptr) {
+    std::string duplicate_table_name{duplicate_table->s->table_name.str,
+                                     duplicate_table->s->table_name.length};
+    load_driver->set_string(load_handle, Bulk_string::DUPLICATE_TABLE_NAME,
+                            duplicate_table_name);
+  }
+
+  load_driver->set_string(load_handle, Bulk_string::FILE_PREFIX,
+                          info.m_file_prefix);
+
+  if (info.m_file_suffix.has_value()) {
+    load_driver->set_string(load_handle, Bulk_string::FILE_SUFFIX,
+                            info.m_file_suffix.value());
+  }
+
+  if (!info.m_appendtolastprefix.empty()) {
+    load_driver->set_string(load_handle, Bulk_string::APPENDTOLASTPREFIX,
+                            info.m_appendtolastprefix);
+  }
+
+  /* If not set by user, default is assigned from default_field_term. */
+  const String *field_term = m_exchange.field.field_term;
+  std::string col_term(field_term->ptr(), field_term->length());
+
+  load_driver->set_string(load_handle, Bulk_string::COLUMN_TERM, col_term);
+
+  /* If not set by user, default is assigned from default_line_term. */
+  const String *line_term = m_exchange.line.line_term;
+  std::string row_term(line_term->ptr(), line_term->length());
+  load_driver->set_string(load_handle, Bulk_string::ROW_TERM, row_term);
+
+  /* Set boolean options. */
+  load_driver->set_condition(load_handle, Bulk_condition::ORDERED_DATA,
+                             m_ordered_data);
+
+  /* Set boolean options. */
+  load_driver->set_condition(load_handle, Bulk_condition::NON_EMPTY_TABLE,
+                             m_non_empty_table);
+  load_driver->set_condition(load_handle, Bulk_condition::OPTIONAL_ENCLOSE,
+                             m_exchange.field.opt_enclosed);
+  load_driver->set_condition(load_handle, Bulk_condition::DRYRUN,
+                             info.m_is_dryrun);
+
+  /* Set size options. */
+  load_driver->set_size(load_handle, Bulk_size::CONCURRENCY, m_concurrency);
+  load_driver->set_size(load_handle, Bulk_size::COUNT_FILES, m_file_count);
+  load_driver->set_size(load_handle, Bulk_size::START_INDEX,
+                        info.m_start_index);
+  load_driver->set_size(load_handle, Bulk_size::COUNT_ROW_SKIP,
+                        m_exchange.skip_lines);
+  load_driver->set_size(load_handle, Bulk_size::COUNT_COLUMNS,
+                        sql_table->s->fields);
+  load_driver->set_size(load_handle, Bulk_size::MEMORY, m_memory_size);
+
+  /* Set escape and enclosing character options */
+  const String *escaped = m_exchange.field.escaped;
+  const String *enclosed = m_exchange.field.enclosed;
+  if (m_exchange.escaped_given() && !escaped->is_empty()) {
+    load_driver->set_char(load_handle, Bulk_char::ESCAPE_CHAR, (*escaped)[0]);
+  }
+  if (!enclosed->is_empty()) {
+    load_driver->set_char(load_handle, Bulk_char::ENCLOSE_CHAR, (*enclosed)[0]);
+  }
+
+  load_driver->set_compression_algorithm(load_handle, compression_algorithm);
+
+  bool success = load_driver->load(load_handle, affected_rows);
+
+  load_driver->drop_bulk_loader(thd, load_handle);
+
+  srv_registry->release(svc);
+
+  return success;
+}
+
 /**
   Execute LOAD DATA query
 
@@ -266,6 +892,11 @@ bool Sql_cmd_load_table::execute_inner(THD *thd,
           table_list->updatable_base_table()
           : nullptr;
 
+  if (table_list->is_json_duality_view()) {
+    my_error(ER_JDV_CANNOT_BE_USED_WITH, MYF(0), table_list->alias, "LOAD");
+    return true;
+  }
+
   if (insert_table_ref == nullptr ||
       check_key_in_view(thd, table_list, insert_table_ref)) {
     my_error(ER_NON_UPDATABLE_TABLE, MYF(0), table_list->alias, "LOAD");
@@ -464,6 +1095,12 @@ bool Sql_cmd_load_table::execute_inner(THD *thd,
     return true;
   }
 
+  // Statement is fully prepared:
+  thd->lex->unit->set_prepared();
+
+  // Start execution:
+  thd->lex->set_exec_started();
+
   if (m_is_local_file) {
     (void)net_request_file(thd->get_protocol_classic()->get_net(),
                            m_exchange.file_name);
@@ -523,7 +1160,8 @@ bool Sql_cmd_load_table::execute_inner(THD *thd,
 
   READ_INFO read_info(
       file, tot_length,
-      m_exchange.cs ? m_exchange.cs : thd->variables.collation_database,
+      (m_exchange.file_info.cs != nullptr) ? m_exchange.file_info.cs
+                                           : thd->variables.collation_database,
       *field_term, *m_exchange.line.line_start, *m_exchange.line.line_term,
       *enclosed, info.escape_char, m_is_local_file, is_fifo);
   if (read_info.error) {
@@ -542,8 +1180,8 @@ bool Sql_cmd_load_table::execute_inner(THD *thd,
   thd->check_for_truncated_fields = CHECK_FIELD_WARN;
   thd->num_truncated_fields = 0L;
   /* Skip lines if there is a line terminator */
-  if (m_exchange.line.line_term->length() &&
-      m_exchange.filetype != FILETYPE_XML) {
+  if (m_exchange.line.line_term->length() != 0 &&
+      m_exchange.file_info.filetype != FILETYPE_XML) {
     /* m_exchange.skip_lines needs to be preserved for logging */
     while (skip_lines > 0) {
       skip_lines--;
@@ -562,7 +1200,7 @@ bool Sql_cmd_load_table::execute_inner(THD *thd,
       table->file->ha_start_bulk_insert((ha_rows)0);
     table->copy_blobs = true;
 
-    if (m_exchange.filetype == FILETYPE_XML) /* load xml */
+    if (m_exchange.file_info.filetype == FILETYPE_XML) /* load xml */
       error =
           read_xml_field(thd, info, insert_table_ref, read_info, skip_lines);
     else if (!field_term->length() && !enclosed->length())
@@ -723,9 +1361,9 @@ bool Sql_cmd_load_table::write_execute_load_query_log_event(
   Execute_load_query_log_event e(
       thd, query->ptr(), query->length(), fname_start, fname_end,
       (duplicates == DUP_REPLACE)
-          ? binary_log::LOAD_DUP_REPLACE
-          : (thd->lex->is_ignore() ? binary_log::LOAD_DUP_IGNORE
-                                   : binary_log::LOAD_DUP_ERROR),
+          ? mysql::binlog::event::LOAD_DUP_REPLACE
+          : (thd->lex->is_ignore() ? mysql::binlog::event::LOAD_DUP_IGNORE
+                                   : mysql::binlog::event::LOAD_DUP_ERROR),
       transactional_table, false, false, errcode);
 
   return mysql_bin_log.write_event(&e);
@@ -827,6 +1465,7 @@ bool Sql_cmd_load_table::read_fixed_length(THD *thd, COPY_INFO &info,
                             ER_WARN_TOO_FEW_RECORDS,
                             ER_THD(thd, ER_WARN_TOO_FEW_RECORDS),
                             thd->get_stmt_da()->current_row_for_condition());
+        if (thd->is_error()) return true;
         if (field->type() == FIELD_TYPE_TIMESTAMP && !field->is_nullable()) {
           // Specific of TIMESTAMP NOT NULL: set to CURRENT_TIMESTAMP.
           Item_func_now_local::store_in(field);
@@ -872,6 +1511,15 @@ bool Sql_cmd_load_table::read_fixed_length(THD *thd, COPY_INFO &info,
       goto continue_loop;
     }
 
+    if (use_sql_fk_checks_for_table(thd, table)) {
+      if (check_all_parent_fk_ref(thd, table, enum_fk_dml_type::FK_INSERT)) {
+        if (thd->is_error()) return true;
+        // continue when IGNORE clause is used.
+        read_info.next_line();
+        goto continue_loop;
+      }
+    }
+
     err = write_record(thd, table, &info, nullptr);
     if (err) return true;
 
@@ -1064,6 +1712,7 @@ bool Sql_cmd_load_table::read_sep_field(THD *thd, COPY_INFO &info,
                               ER_WARN_TOO_FEW_RECORDS,
                               ER_THD(thd, ER_WARN_TOO_FEW_RECORDS),
                               thd->get_stmt_da()->current_row_for_condition());
+          if (thd->is_error()) return true;
         } else if (item->type() == Item::STRING_ITEM) {
           assert(nullptr != dynamic_cast<Item_user_var_as_out_param *>(item));
           ((Item_user_var_as_out_param *)item)
@@ -1112,6 +1761,15 @@ bool Sql_cmd_load_table::read_sep_field(THD *thd, COPY_INFO &info,
       goto continue_loop;
     }
 
+    if (use_sql_fk_checks_for_table(thd, table)) {
+      if (check_all_parent_fk_ref(thd, table, enum_fk_dml_type::FK_INSERT)) {
+        if (thd->is_error()) return true;
+        // continue when IGNORE clause is used.
+        read_info.next_line();
+        goto continue_loop;
+      }
+    }
+
     err = write_record(thd, table, &info, nullptr);
     if (err) return true;
     /*
@@ -1267,6 +1925,7 @@ bool Sql_cmd_load_table::read_xml_field(THD *thd, COPY_INFO &info,
                               ER_WARN_TOO_FEW_RECORDS,
                               ER_THD(thd, ER_WARN_TOO_FEW_RECORDS),
                               thd->get_stmt_da()->current_row_for_condition());
+          if (thd->is_error()) return true;
         } else {
           assert(nullptr != dynamic_cast<Item_user_var_as_out_param *>(item));
           ((Item_user_var_as_out_param *)item)->set_null_value(cs);
@@ -1292,6 +1951,14 @@ bool Sql_cmd_load_table::read_xml_field(THD *thd, COPY_INFO &info,
       goto continue_loop;
     }
 
+    if (use_sql_fk_checks_for_table(thd, table)) {
+      if (check_all_parent_fk_ref(thd, table, enum_fk_dml_type::FK_INSERT)) {
+        if (thd->is_error()) return true;
+        // continue when IGNORE clause is used.
+        goto continue_loop;
+      }
+    }
+
     if (write_record(thd, table, &info, nullptr)) return true;
 
     /*
@@ -2123,12 +2790,95 @@ found_eof:
   return true;
 }
 
+/** Check if the given string is made up of whitespace characters only.
+@param[in]  s   the input string to be checked
+@return true iff s is made up of whitespace characters */
+static inline bool is_whitespace(const std::string &s) {
+  return s.empty() || std::find_if(s.begin(), s.end(), [](unsigned char c) {
+                        return !std::isspace(c);
+                      }) == s.end();
+}
+
 bool Sql_cmd_load_table::execute(THD *thd) {
   LEX *const lex = thd->lex;
+  bool need_file_acl = false;
+
+  if (m_is_bulk_operation) {
+    if (m_exchange.file_info.filetype == FILETYPE_XML) {
+      my_error(ER_WRONG_USAGE, MYF(0), "LOAD XML", "BULK Algorithm");
+      return true;
+    }
+    if (!m_opt_fields_or_vars.empty()) {
+      my_error(ER_WRONG_USAGE, MYF(0), "LOAD DATA with BULK Algorithm",
+               "column list specification");
+      return true;
+    }
+    if (!m_opt_set_fields.empty()) {
+      my_error(ER_WRONG_USAGE, MYF(0), "LOAD DATA with BULK Algorithm",
+               "assignment to columns or variables");
+      return true;
+    }
+    if (m_is_local_file) {
+      my_error(ER_WRONG_USAGE, MYF(0), "LOAD DATA with BULK Algorithm",
+               "LOCAL client file");
+      return true;
+    }
+    if (!m_exchange.line.line_start->is_empty()) {
+      my_error(ER_WRONG_USAGE, MYF(0), "LOAD DATA with BULK Algorithm",
+               "LINES STARTING BY");
+      return true;
+    }
+    if (m_exchange.field.field_term->length() > 1) {
+      my_error(ER_WRONG_USAGE, MYF(0), "LOAD DATA with BULK Algorithm",
+               "multi-byte column separator");
+      return true;
+    }
+
+    if (thd->lex->is_ignore()) {
+      my_error(ER_WRONG_USAGE, MYF(0), "LOAD DATA with BULK Algorithm",
+               "IGNORE clause");
+      return true;
+    }
+    if (is_whitespace(m_exchange.file_name)) {
+      my_error(ER_WRONG_USAGE, MYF(0), "LOAD DATA with BULK Algorithm",
+               "filename is empty/has only whitespace characters.");
+      return true;
+    }
+
+    /* We need FILE_ACL only if the data source files are in server. */
+    need_file_acl = (m_bulk_source == LOAD_SOURCE_FILE);
+
+  } else {
+    if (m_file_count > 0) {
+      my_error(ER_WRONG_USAGE, MYF(0), "LOAD DATA without BULK Algorithm",
+               "multiple files");
+      return true;
+    }
+    if (m_bulk_source == LOAD_SOURCE_URL) {
+      my_error(ER_WRONG_USAGE, MYF(0), "LOAD DATA without BULK Algorithm",
+               "URI/URL source");
+      return true;
+    }
+    if (m_compression_algorithm_string.length != 0) {
+      my_error(ER_WRONG_USAGE, MYF(0), "LOAD DATA without BULK Algorithm",
+               "COMPRESSION specified!");
+      return true;
+    }
+
+    if (is_json_object(m_exchange.file_name)) {
+      my_error(ER_WRONG_USAGE, MYF(0), "LOAD DATA without BULK Algorithm",
+               "JSON object specified as filename!");
+      return true;
+    }
+
+    /* We need FILE_ACL if data is not streamed by client from client local
+     * file. */
+    need_file_acl = !m_is_local_file;
+  }
 
   const uint privilege =
       (lex->duplicates == DUP_REPLACE ? INSERT_ACL | DELETE_ACL : INSERT_ACL) |
-      (m_is_local_file ? 0 : FILE_ACL);
+      (need_file_acl ? FILE_ACL : 0);
 
   if (m_is_local_file) {
     if (!thd->get_protocol()->has_client_capability(CLIENT_LOCAL_FILES) ||
@@ -2148,10 +2898,16 @@ bool Sql_cmd_load_table::execute(THD *thd) {
   else if (thd->is_strict_mode())
     thd->push_internal_handler(&strict_handler);
 
-  lex->using_hypergraph_optimizer =
-      thd->optimizer_switch_flag(OPTIMIZER_SWITCH_HYPERGRAPH_OPTIMIZER);
+  lex->set_using_hypergraph_optimizer(
+      thd->optimizer_switch_flag(OPTIMIZER_SWITCH_HYPERGRAPH_OPTIMIZER));
 
-  const bool res = execute_inner(thd, lex->duplicates);
+  bool res = true;
+
+  if (m_is_bulk_operation) {
+    res = execute_bulk(thd);
+  } else {
+    res = execute_inner(thd, lex->duplicates);
+  }
 
   /* Pop ignore / strict error handler */
   if (thd->lex->is_ignore() || thd->is_strict_mode())
