@@ -58,6 +58,7 @@ namespace dd {
 class Foreign_key;
 class Schema;
 class Table;
+class View;
 }  // namespace dd
 
 struct HA_CHECK_OPT;
@@ -123,11 +124,13 @@ bool mysql_discard_or_import_tablespace(THD *thd, Table_ref *table_list);
   data-dictionary cache entries and performing such invalidation.
 */
 class Foreign_key_parents_invalidator {
- private:
+ public:
   enum enum_invalidation_type {
     INVALIDATE_AND_CLOSE_TABLE,
     INVALIDATE_AND_MARK_FOR_REOPEN
   };
+
+ private:
   typedef std::map<std::pair<dd::String_type, dd::String_type>,
                    std::pair<handlerton *, enum_invalidation_type>>
       Parent_map;
@@ -475,7 +478,8 @@ bool mysql_compare_tables(THD *thd, TABLE *table, Alter_info *alter_info,
                           HA_CREATE_INFO *create_info, bool *metadata_equal);
 bool mysql_recreate_table(THD *thd, Table_ref *table_list, bool table_copy);
 bool mysql_create_like_table(THD *thd, Table_ref *table, Table_ref *src_table,
-                             HA_CREATE_INFO *create_info);
+                             HA_CREATE_INFO *create_info, uint open_flags = 0,
+                             bool skip_post_ddl_operations = false);
 bool mysql_rename_table(THD *thd, handlerton *base, const char *old_db,
                         const char *old_name, const char *old_fk_db,
                         const char *old_fk_name, const dd::Schema &new_schema,
@@ -487,6 +491,8 @@ bool mysql_rm_table(THD *thd, Table_ref *tables, bool if_exists,
                     bool drop_temporary);
 bool mysql_rm_table_no_locks(THD *thd, Table_ref *tables, bool if_exists,
                              bool drop_temporary, bool drop_database,
+                             const char *database_name,
+                             const bool should_drop_schema_ddl_log,
                              bool *dropped_non_atomic_flag,
                              std::set<handlerton *> *post_ddl_htons,
                              Foreign_key_parents_invalidator *fk_invalidator,
@@ -665,4 +671,30 @@ bool prepare_check_constraints_for_create(THD *thd, const char *db_name,
 extern std::atomic_ulong deprecated_use_fk_on_non_standard_key_count;
 extern std::atomic_ullong deprecated_use_fk_on_non_standard_key_last_timestamp;
 
+/**
+  Unloads the materialized view from a secondary engine, where it might be
+  materialized.
+ */
+bool secondary_engine_unload_materialized_view(THD *thd, const Table_ref *view,
+                                               const dd::View *view_def);
+
+/**
+  Resolves the secondary engine handled via its name and calls the unload
+  table function
+*/
+bool secondary_engine_unload_table_inner(THD *thd, const char *db_name,
+                                         const char *table_name,
+                                         LEX_CSTRING secondary_engine,
+                                         bool is_partitioned,
+                                         bool error_if_not_loaded);
+
+/**
+  Validation on create temporary table statements with secondary engine defined:
+  1. Check that the create table statement contains a query.
+  2. Set the lex state indicating that the query can execute only
+  on the secondary engine.
+*/
+bool validate_secondary_engine_temporary_table(THD *thd,
+                                               HA_CREATE_INFO *create_info);
+
 #endif /* SQL_TABLE_INCLUDED */
