@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_21179566_1759338175_92_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
-# define YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_21179566_1759338175_92_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+#ifndef YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_21724532_1765805899_04_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+# define YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_21724532_1765805899_04_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
 /* Debug traces.  */
 #ifndef MY_SQL_PARSER_DEBUG
 # if defined YYDEBUG
@@ -882,11 +882,13 @@ extern int my_sql_parser_debug;
     EXTERNAL_SYM = 1235,
     MATERIALIZED_SYM = 1236,
     GUIDED_SYM = 1237,
-    KEYWORD_USED_AS_IDENT = 1239,
-    KEYWORD_USED_AS_KEYWORD = 1240,
-    CONDITIONLESS_JOIN = 1241,
-    PREFER_PARENTHESES = 1242,
-    EMPTY_FROM_CLAUSE = 1243
+    SETS_SYM = 1238,
+    VALIDATE_SYM = 1239,
+    KEYWORD_USED_AS_IDENT = 1241,
+    KEYWORD_USED_AS_KEYWORD = 1242,
+    CONDITIONLESS_JOIN = 1243,
+    PREFER_PARENTHESES = 1244,
+    EMPTY_FROM_CLAUSE = 1245
   };
 #endif
 /* Tokens.  */
@@ -1719,11 +1721,13 @@ extern int my_sql_parser_debug;
 #define EXTERNAL_SYM 1235
 #define MATERIALIZED_SYM 1236
 #define GUIDED_SYM 1237
-#define KEYWORD_USED_AS_IDENT 1239
-#define KEYWORD_USED_AS_KEYWORD 1240
-#define CONDITIONLESS_JOIN 1241
-#define PREFER_PARENTHESES 1242
-#define EMPTY_FROM_CLAUSE 1243
+#define SETS_SYM 1238
+#define VALIDATE_SYM 1239
+#define KEYWORD_USED_AS_IDENT 1241
+#define KEYWORD_USED_AS_KEYWORD 1242
+#define CONDITIONLESS_JOIN 1243
+#define PREFER_PARENTHESES 1244
+#define EMPTY_FROM_CLAUSE 1245
 
 /* Value type.  */
 
@@ -1745,4 +1749,4 @@ struct MY_SQL_PARSER_LTYPE
 
 int my_sql_parser_parse (class THD *YYTHD, class Parse_tree_root **parse_tree);
 
-#endif /* !YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_21179566_1759338175_92_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
+#endif /* !YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_21724532_1765805899_04_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
