@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2013, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2013, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -30,13 +30,13 @@
 #include <atomic>
 
 #include "lex_string.h"
-#include "m_ctype.h"
-#include "m_string.h"
 #include "my_psi_config.h"
 #include "mysql/components/services/bits/psi_statement_bits.h"
 #include "mysql/psi/mysql_sp.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysqld_error.h"
 #include "mysys_err.h"          // EE_OUTOFMEMORY
+#include "nulls.h"              // NullS
 #include "sql/derror.h"         // ER_THD
 #include "sql/error_handler.h"  // Internal_error_handler
 #include "sql/sp.h"             // sp_add_used_routine
@@ -47,11 +47,13 @@
 #include "sql/sql_error.h"  // Sql_condition
 #include "sql/sql_lex.h"
 #include "sql/sql_parse.h"  // parse_sql
-#include "sql/sql_show.h"   // append_identifier
+#include "sql/sql_show.h"   // append_identifier_*
 #include "sql/strfunc.h"
 #include "sql/system_variables.h"
 #include "sql/trigger_creation_ctx.h"  // Trigger_creation_ctx
 #include "sql_string.h"
+#include "string_with_len.h"
+#include "strxmov.h"
 
 class sp_rcontext;
 struct MEM_ROOT;
@@ -116,7 +118,7 @@ static bool construct_definer_value(MEM_ROOT *mem_root, LEX_CSTRING *definer,
                                     const LEX_CSTRING &definer_user,
                                     const LEX_CSTRING &definer_host) {
   char definer_buf[USER_HOST_BUFF_SIZE];
-  size_t definer_len =
+  const size_t definer_len =
       strxmov(definer_buf, definer_user.str, "@", definer_host.str, NullS) -
       definer_buf;
 
@@ -251,7 +253,7 @@ Trigger *Trigger::create_from_parser(THD *thd, TABLE *subject_table,
 
   // Create a new Trigger instance.
 
-  my_timeval created_timestamp_not_set = {0, 0};
+  const my_timeval created_timestamp_not_set = {0, 0};
   Trigger *t = new (&subject_table->mem_root) Trigger(
       trigger_name, &subject_table->mem_root, subject_table->s->db,
       subject_table->s->table_name, definition, definition_utf8,
@@ -318,6 +320,22 @@ Trigger *Trigger::create_from_dd(
               trg_time_type, action_order, created_timestamp);
 }
 
+/**
+  Create a new Trigger object as a shallow clone of existing Trigger object.
+
+  @note Allows to produce Trigger objects to be associated with specific TABLE
+        instance from Trigger objects associated with TABLE_SHARE.
+*/
+Trigger *Trigger::clone_shallow(MEM_ROOT *mem_root) const {
+  // Do not create shallow clones of Trigger objects associated with TABLE.
+  assert(m_sp == nullptr);
+  return new (mem_root) Trigger(
+      m_trigger_name, mem_root, m_db_name, m_subject_table_name, m_definition,
+      m_definition_utf8, m_sql_mode, m_definer_user, m_definer_host,
+      m_client_cs_name, m_connection_cl_name, m_db_cl_name, m_event,
+      m_action_time, m_action_order, m_created_timestamp);
+}
+
 /**
   Trigger constructor.
 */
@@ -348,9 +366,7 @@ Trigger::Trigger(
       m_action_order(action_order),
       m_trigger_name(trigger_name),
       m_sp(nullptr),
-      m_has_parse_error(false) {
-  m_parse_error_message[0] = 0;
-
+      m_parse_error_message(nullptr) {
   construct_definer_value(mem_root, &m_definer, definer_user, definer_host);
 }
 
@@ -370,7 +386,7 @@ Trigger::~Trigger() { sp_head::destroy(m_sp); }
 */
 
 bool Trigger::execute(THD *thd) {
-  if (m_has_parse_error) return true;
+  if (has_parse_error()) return true;
 
   bool err_status;
   Sub_statement_state statement_state;
@@ -378,6 +394,12 @@ bool Trigger::execute(THD *thd) {
 
   thd->reset_sub_statement_state(&statement_state, SUB_STMT_TRIGGER);
 
+  Secondary_engine_optimization saved_state =
+      thd->secondary_engine_optimization();
+
+  thd->set_secondary_engine_optimization(
+      Secondary_engine_optimization::PRIMARY_TENTATIVELY);
+
   /*
     Reset current_query_block before call execute_trigger() and
     restore it after return from one. This way error is set
@@ -391,6 +413,8 @@ bool Trigger::execute(THD *thd) {
 
   thd->restore_sub_statement_state(&statement_state);
 
+  thd->set_secondary_engine_optimization(saved_state);
+
   return err_status;
 }
 
@@ -430,7 +454,7 @@ bool Trigger::create_full_trigger_definition(
 */
 
 bool Trigger::parse(THD *thd, bool is_upgrade) {
-  sql_mode_t sql_mode_saved = thd->variables.sql_mode;
+  const sql_mode_t sql_mode_saved = thd->variables.sql_mode;
   thd->variables.sql_mode = m_sql_mode;
 
   Parser_state parser_state;
@@ -469,7 +493,7 @@ bool Trigger::parse(THD *thd, bool is_upgrade) {
   thd->lex = &lex;
   lex_start(thd);
 
-  LEX_CSTRING current_db_name_saved = thd->db();
+  const LEX_CSTRING current_db_name_saved = thd->db();
   thd->reset_db(m_db_name);
 
   Deprecated_trigger_syntax_handler error_handler;
@@ -671,3 +695,13 @@ void Trigger::print_upgrade_warning(THD *thd) {
       ER_THD(thd, ER_WARN_TRIGGER_DOESNT_HAVE_CREATED), get_db_name().str,
       get_subject_table_name().str, get_trigger_name().str);
 }
+
+/**
+  Mark trigger as having a parse error and remember the message for future use.
+*/
+
+void Trigger::set_parse_error_message(const char *error_message) {
+  m_parse_error_message = strdup_root(m_mem_root, error_message);
+  // Play safe even in case of OOM.
+  if (!m_parse_error_message) m_parse_error_message = ER_DEFAULT(ER_DA_OOM);
+}
