@@ -31,7 +31,13 @@ BULK Data Load. Currently treated like DDL */
 #pragma once
 
 #include <list>
+#include "api0api.h"
 #include "btr0mtib.h"
+#include "data0type.h"
+#include "db0err.h"
+#include "dict0types.h"
+#include "mem0mem.h"
+#include "mysql/components/services/bulk_data_service.h"
 #include "row0mysql.h"
 #include "sql/handler.h"
 
@@ -42,6 +48,73 @@ class Loader {
   using Blob_context = void *;
   using byte = unsigned char;
 
+  class Table_reader {
+   public:
+    bool init(const std::string &schema, const std::string &table,
+              const row_prebuilt_t *prebuilt,
+              const std::optional<Rows_mysql> &lower_bound,
+              const std::optional<Rows_mysql> &upper_bound);
+
+    bool is_initialized() const { return m_initialized; }
+    bool more_records_available() const { return m_more_records_available; }
+
+    ib_tpl_t read() { /* Read the current row from cursor position */
+      return m_read_tuple;
+    }
+
+    void next() {
+      ib_err_t err;
+      err = ib_cursor_next(m_read_cursor);
+      if (err != DB_SUCCESS) {
+        m_more_records_available = false;
+        return;
+      }
+      err = ib_cursor_read_row(m_read_cursor, m_read_tuple, m_cmp_tuple,
+                               IB_CUR_L, nullptr, nullptr, nullptr);
+      m_more_records_available = err == DB_SUCCESS;
+    }
+
+    Table_reader() = default;
+    Table_reader(Table_reader &&other) = default;
+
+    ~Table_reader() {
+      if (m_initialized) {
+        ib_cursor_close(m_read_cursor);
+
+        if (!m_prebuilt->index->is_clustered()) {
+          ib_cursor_close(m_table_cursor);
+        }
+
+        if (m_cmp_tuple != nullptr) {
+          ib_tuple_delete(m_cmp_tuple);
+        }
+
+        if (m_read_tuple != nullptr) {
+          ib_tuple_delete(m_read_tuple);
+        }
+
+        trx_commit(m_trx);
+        trx_free_for_background(m_trx);
+      }
+    }
+
+   private:
+    bool m_more_records_available{false};
+    bool m_initialized{false};
+    std::string m_table_name;
+    const row_prebuilt_t *m_prebuilt;
+    std::optional<Rows_mysql> m_lower_bound;
+    std::vector<std::unique_ptr<char[]>> m_lower_bound_data;
+    std::optional<Rows_mysql> m_upper_bound;
+    std::vector<std::unique_ptr<char[]>> m_upper_bound_data;
+    trx_t *m_trx{};
+    ib_crsr_t m_table_cursor{};
+    ib_crsr_t m_read_cursor{};
+    ib_tpl_t m_cmp_tuple{};
+    unsigned char m_cmp_tuple_row_id_data[DATA_ROW_ID_LEN];
+    ib_tpl_t m_read_tuple{};
+  };
+
   class Thread_data {
    public:
     /** Initialize thread specific data.
@@ -58,7 +131,14 @@ class Loader {
                  Btree_multi::Btree_load *sub_tree, const Rows_mysql &rows,
                  Bulk_load::Stat_callbacks &wait_cbk);
 
-   public:
+    bool set_source_table_data(
+        const row_prebuilt_t *prebuilt,
+        const Bulk_load::Source_table_data &source_table);
+
+    dberr_t copy_existing_data(const row_prebuilt_t *prebuilt,
+                               Btree_multi::Btree_load *sub_tree,
+                               Bulk_load::Stat_callbacks &wait_cbk);
+
     /** Create a blob.
     @param[in]   sub_tree  sub tree to load data to
     @param[out]  blob_ctx  pointer to an opaque object representing a blob.
@@ -92,7 +172,6 @@ class Loader {
       return sub_tree->close_blob(blob_ctx, ref);
     }
 
-   public:
     /** Free thread specific data. */
     void free();
 
@@ -112,6 +191,15 @@ class Loader {
     /** Get the last subtree created by this thread. */
     Btree_multi::Btree_load *get_subtree() { return m_list_subtrees.back(); }
 
+    Btree_multi::Btree_load *get_subtree(Blob_context ctx) const {
+      auto iter = std::find_if(m_list_subtrees.begin(), m_list_subtrees.end(),
+                               [ctx](const auto &subtree) {
+                                 return subtree->verify_blob_context(ctx);
+                               });
+      ut_ad(iter != m_list_subtrees.end());
+      return *iter;
+    }
+
     /** Flush queue size used by the Bulk_flusher */
     size_t m_queue_size;
 
@@ -124,47 +212,44 @@ class Loader {
     uint64_t m_last_rowid{0};
 
    private:
-    /** Fill system columns for index entry to be loaded.
-    @param[in]  prebuilt  prebuilt structures from innodb table handler */
-    void fill_system_columns(const row_prebuilt_t *prebuilt);
+    void read_input_entry(const Rows_mysql &rows, size_t &row_index,
+                          const row_prebuilt_t *prebuilt, mem_heap_t *gcol_heap,
+                          bool &gcol_blobs_flushed);
+    void read_table_entry(const row_prebuilt_t *prebuilt);
 
-    /** Fill the tuple to set the column data
-    @param[in]  prebuilt   prebuilt structures from innodb table handler
-    @param[in]  rows       sql rows with column data
-    @param[in]  row_index  current row index
-    @param[in]  gcol_heap  memory heap used for generated columns
-    @param[in,out]  gcol_blobs_flushed true if blobs are flushed, false
-                 otherwise.  This is needed only when we have gcol on blobs.
-    @return innodb error code. */
-    dberr_t fill_tuple(const row_prebuilt_t *prebuilt, const Rows_mysql &rows,
-                       size_t row_index, mem_heap_t *gcol_heap,
-                       bool &gcol_blobs_flushed);
+    void insert_from_input_and_move_to_next(const row_prebuilt_t *prebuilt,
+                                            const Rows_mysql &rows,
+                                            size_t &row_index,
+                                            Btree_multi::Btree_load *sub_tree,
+                                            mem_heap_t *gcol_heap,
+                                            bool &gcol_blobs_flushed);
 
-    dberr_t setup_dfield(const row_prebuilt_t *prebuilt, Field *field,
-                         const Column_mysql &sql_col, dfield_t *src_dfield,
-                         dfield_t *dst_dfield);
+    void insert_from_original_table_and_move_to_next(
+        const row_prebuilt_t *prebuilt, Btree_multi::Btree_load *sub_tree);
 
-    /** Fill he cluster index entry from tuple data.
-    @param[in]  prebuilt  prebuilt structures from innodb table handler */
-    void fill_index_entry(const row_prebuilt_t *prebuilt);
+    void insert_smaller_entry(const row_prebuilt_t *prebuilt,
+                              Btree_multi::Btree_load *sub_tree,
+                              const Rows_mysql &rows, size_t &row_index,
+                              const ddl::Compare_key &compare_key,
+                              mem_heap_t *gcol_heap, bool &gcol_blobs_flushed);
 
-    /** Store integer column in Innodb format.
-    @param[in]      col       sql column data
-    @param[in,out]  data_ptr  data buffer for storing converted data
-    @param[in,out]  data_len  data buffer length
-    @return true if successful. */
-    bool store_int_col(const Column_mysql &col, byte *data_ptr,
-                       size_t &data_len);
-
-   private:
     /** Heap for allocating tuple memory. */
     mem_heap_t *m_heap{};
 
     /** Tuple for converting input data to table row. */
-    dtuple_t *m_row{};
+    dtuple_t *m_input_row{};
 
-    /** Tuple for inserting row to cluster index. */
-    dtuple_t *m_entry{};
+    dtuple_t *m_table_row{};
+
+    /** Tuple for inserting row to index. */
+    dtuple_t *m_input_entry{};
+
+    /** Tuple for inserting row to index. */
+    dtuple_t *m_original_table_entry{};
+
+    /** Contains the tuple we detected an error, either m_original_table_entry
+    or m_input_entry. */
+    dtuple_t *m_error_entry{};
 
     /** Column data for system column transaction ID. */
     unsigned char m_trx_data[DATA_TRX_ID_LEN];
@@ -183,6 +268,11 @@ class Loader {
     std::ostringstream m_sout;
 
     size_t m_nth_index{std::numeric_limits<size_t>::max()};
+
+    Table_reader m_table_reader;
+    bool m_more_available_in_input{};
+    bool m_more_available_in_original_table{};
+    std::optional<std::string> m_original_table_name{};
   };
 
   /** Loader context constructor.
@@ -210,9 +300,16 @@ class Loader {
   dberr_t load(const row_prebuilt_t *prebuilt, size_t thread_index,
                const Rows_mysql &rows, Bulk_load::Stat_callbacks &wait_cbk);
 
+  bool set_source_table_data(
+      const row_prebuilt_t *prebuilt,
+      const std::vector<Bulk_load::Source_table_data> &source_table_data);
+
+  dberr_t copy_existing_data(const row_prebuilt_t *prebuilt,
+                             size_t thread_index,
+                             Bulk_load::Stat_callbacks &wait_cbk);
+
   size_t get_keynr() const { return m_keynr; }
 
- public:
   /** Open a blob.
   @param[in]   thread_index  identifies the thread and the B-tree to use.
   @param[out]  blob_ctx  pointer to an opaque object representing a blob.
@@ -258,7 +355,12 @@ class Loader {
   int get_error_code() const;
 
   /** @return table name where the data is being loaded. */
-  const char *get_table_name() const { return m_table->name.m_name; }
+  const char *get_table_name() const {
+    if (m_original_table_name.has_value()) {
+      return m_original_table_name.value().c_str();
+    }
+    return m_table->name.m_name;
+  }
 
   /** @return index name where the data is being loaded. */
   const char *get_index_name() const { return m_index->name(); }
@@ -308,6 +410,8 @@ class Loader {
   size_t m_queue_size;
 
   std::mutex m_gcol_mutex;
+
+  std::optional<std::string> m_original_table_name;
 };
 
 inline std::string Loader::get_error_string() const {
