@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2024, Oracle and/or its affiliates.
+Copyright (c) 1995, 2025, Oracle and/or its affiliates.
 Copyright (c) 2008, Google Inc.
 Copyright (c) 2012, Facebook Inc.
 
@@ -83,6 +83,7 @@ instrumentation due to their large number of instances. */
 
 #ifdef UNIV_PFS_MUTEX
 /* Key defines to register InnoDB mutexes with performance schema */
+extern mysql_pfs_key_t alter_stage_mutex_key;
 extern mysql_pfs_key_t autoinc_mutex_key;
 extern mysql_pfs_key_t autoinc_persisted_mutex_key;
 #ifndef PFS_SKIP_BUFFER_MUTEX_RWLOCK
@@ -121,6 +122,7 @@ extern mysql_pfs_key_t log_write_notifier_mutex_key;
 extern mysql_pfs_key_t log_flush_notifier_mutex_key;
 extern mysql_pfs_key_t log_limits_mutex_key;
 extern mysql_pfs_key_t log_files_mutex_key;
+extern mysql_pfs_key_t log_governor_mutex_key;
 extern mysql_pfs_key_t log_cmdq_mutex_key;
 extern mysql_pfs_key_t log_sn_lock_key;
 extern mysql_pfs_key_t log_sn_mutex_key;
@@ -140,7 +142,6 @@ extern mysql_pfs_key_t rtr_path_mutex_key;
 extern mysql_pfs_key_t rtr_ssn_mutex_key;
 extern mysql_pfs_key_t temp_space_rseg_mutex_key;
 extern mysql_pfs_key_t undo_space_rseg_mutex_key;
-extern mysql_pfs_key_t trx_sys_rseg_mutex_key;
 extern mysql_pfs_key_t page_zip_stat_per_index_mutex_key;
 #ifdef UNIV_DEBUG
 extern mysql_pfs_key_t rw_lock_debug_mutex_key;
@@ -181,6 +182,7 @@ extern mysql_pfs_key_t clone_snapshot_mutex_key;
 extern mysql_pfs_key_t parallel_read_mutex_key;
 extern mysql_pfs_key_t dblwr_mutex_key;
 extern mysql_pfs_key_t ahi_enabled_mutex_key;
+extern mysql_pfs_key_t dict_table_stats_compute_mutex_key;
 #endif /* UNIV_PFS_MUTEX */
 
 #ifdef UNIV_PFS_RWLOCK
