@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2024, Oracle and/or its affiliates.
+Copyright (c) 1995, 2025, Oracle and/or its affiliates.
 Copyright (c) 2009, Google Inc.
 
 Portions of this file contain modifications contributed and copyrighted by
@@ -697,6 +697,21 @@ static void log_fix_first_rec_group(lsn_t block_lsn,
 
 dberr_t log_start(log_t &log, lsn_t checkpoint_lsn, lsn_t start_lsn,
                   bool allow_checkpoints) {
+  /* Declare function that figures out which of the two header locations stores
+  the checkpoint_lsn. We don't expose it through header, because the header is
+  used in various places of InnoDB, while this is a technical detail of the
+  low-level log implementation. */
+  Log_checkpoint_header_no recv_find_checkpoint_header_no(log_t & log,
+                                                          lsn_t checkpoint_lsn);
+
+  /* Make the preservation of max checkpoint info on disk certain by writing
+  the next checkpoint to the other checkpoint header. This is an extra
+  protection in case next checkpoint write will become corrupted because of
+  crash during the write. */
+
+  log.next_checkpoint_header_no = log_next_checkpoint_header(
+      recv_find_checkpoint_header_no(log, checkpoint_lsn));
+
   ut_a(log_sys != nullptr);
   ut_a(checkpoint_lsn >= OS_FILE_LOG_BLOCK_SIZE);
   ut_a(checkpoint_lsn >= LOG_START_LSN);
@@ -1150,6 +1165,10 @@ void log_print(const log_t &log, FILE *file) {
   log_files_mutex_exit(log);
 
   fprintf(file,
+          "Log capacity                 " LSN_PF
+          "\n"
+          "Log capacity used            " LSN_PF
+          "\n"
           "Log sequence number          " LSN_PF
           "\n"
           "Log buffer assigned up to    " LSN_PF
@@ -1169,6 +1188,7 @@ void log_print(const log_t &log, FILE *file) {
           "Log minimum file id is       " UINT64PF
           "\n"
           "Log maximum file id is       " UINT64PF "\n",
+          (uint64_t)srv_redo_log_capacity, (uint64_t)srv_redo_log_capacity_used,
           current_lsn, max_assigned_lsn, ready_for_write_lsn, write_lsn,
           flush_lsn, dirty_pages_added_up_to_lsn, oldest_lsn,
           last_checkpoint_lsn, file_min_id, file_max_id);
