@@ -1031,8 +1031,7 @@ static byte *trx_undo_report_blob_update(page_t *undo_page, dict_index_t *index,
   const ulint bytes_changed = upd_t::get_total_modified_bytes(*bdiff_v);
 
   /* Whether the update to the LOB can be considered as a small change. */
-  const bool small_change =
-      (bytes_changed <= lob::ref_t::LOB_SMALL_CHANGE_THRESHOLD);
+  const bool small_change = lob::ref_t::is_small_change(bytes_changed, ref);
 
   if (!small_change) {
     /* This is not a small change.  So write the size of the vector as
@@ -2379,7 +2378,7 @@ err_exit:
     bool is_temp)        /*!< in: true if temp undo rec. */
 {
   trx_undo_rec_t *undo_rec;
-  ulint rseg_id;
+  ulint undo_num;
   space_id_t space_id;
   page_no_t page_no;
   ulint offset;
@@ -2387,8 +2386,8 @@ err_exit:
   bool is_insert;
   mtr_t mtr;
 
-  trx_undo_decode_roll_ptr(roll_ptr, &is_insert, &rseg_id, &page_no, &offset);
-  space_id = trx_rseg_id_to_space_id(rseg_id, is_temp);
+  trx_undo_decode_roll_ptr(roll_ptr, &is_insert, &undo_num, &page_no, &offset);
+  space_id = trx_undo_num_to_space_id(undo_num, is_temp);
 
   bool found;
   const page_size_t &page_size = fil_space_get_page_size(space_id, &found);
