@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,15 +25,15 @@
 
 #include "my_config.h"
 
-#include <stdlib.h>
 #include <sys/types.h>
+#include <cstdlib>
 
-#include "m_ctype.h"
 #include "my_byteorder.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
+#include "mysql/strings/m_ctype.h"
 #include "storage/myisam/fulltext.h"
 #include "storage/myisam/myisamdef.h"
 
@@ -328,7 +328,7 @@ int _mi_prefix_search(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *page,
   len = 0;     /* length of previous key unpacked */
 
   while (page < end) {
-    uint packed = *page & 128;
+    uint const packed = *page & 128;
 
     vseg = page;
     if (keyinfo->seg->length >= 127) {
@@ -777,16 +777,16 @@ uint _mi_get_pack_key(MI_KEYDEF *keyinfo, uint nod_flag, uchar **page_pos,
         page += rest_length;
         key += rest_length;
         continue;
-      } else {
-        if (keyseg->flag & HA_NULL_PART) {
-          if (!length--) /* Null part */
-          {
-            *key++ = 0;
-            continue;
-          }
-          *key++ = 1; /* Not null */
-        }
       }
+      if (keyseg->flag & HA_NULL_PART) {
+        if (!length--) /* Null part */
+        {
+          *key++ = 0;
+          continue;
+        }
+        *key++ = 1; /* Not null */
+      }
+
       if (length > (uint)keyseg->length) {
         DBUG_PRINT("error", ("Found too long packed key: %u of %u at %p",
                              length, keyseg->length, *page_pos));
@@ -958,18 +958,18 @@ uchar *_mi_get_key(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *page, uchar *key,
   if (!(keyinfo->flag & (HA_VAR_LENGTH_KEY | HA_BINARY_PACK_KEY))) {
     memmove((uchar *)key, (uchar *)keypos, keyinfo->keylength + nod_flag);
     return keypos + keyinfo->keylength + nod_flag;
-  } else {
-    page += 2 + nod_flag;
-    key[0] = 0; /* safety */
-    while (page <= keypos) {
-      *return_key_length = (*keyinfo->get_key)(keyinfo, nod_flag, &page, key);
-      if (*return_key_length == 0) {
-        mi_print_error(info->s, HA_ERR_CRASHED);
-        set_my_errno(HA_ERR_CRASHED);
-        return nullptr;
-      }
+  }
+  page += 2 + nod_flag;
+  key[0] = 0; /* safety */
+  while (page <= keypos) {
+    *return_key_length = (*keyinfo->get_key)(keyinfo, nod_flag, &page, key);
+    if (*return_key_length == 0) {
+      mi_print_error(info->s, HA_ERR_CRASHED);
+      set_my_errno(HA_ERR_CRASHED);
+      return nullptr;
     }
   }
+
   DBUG_PRINT("exit", ("page: %p  length: %u", page, *return_key_length));
   return page;
 } /* _mi_get_key */
@@ -989,18 +989,18 @@ static bool _mi_get_prev_key(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *page,
     memmove((uchar *)key, (uchar *)keypos - *return_key_length - nod_flag,
             *return_key_length);
     return false;
-  } else {
-    page += 2 + nod_flag;
-    key[0] = 0; /* safety */
-    while (page < keypos) {
-      *return_key_length = (*keyinfo->get_key)(keyinfo, nod_flag, &page, key);
-      if (*return_key_length == 0) {
-        mi_print_error(info->s, HA_ERR_CRASHED);
-        set_my_errno(HA_ERR_CRASHED);
-        return true;
-      }
+  }
+  page += 2 + nod_flag;
+  key[0] = 0; /* safety */
+  while (page < keypos) {
+    *return_key_length = (*keyinfo->get_key)(keyinfo, nod_flag, &page, key);
+    if (*return_key_length == 0) {
+      mi_print_error(info->s, HA_ERR_CRASHED);
+      set_my_errno(HA_ERR_CRASHED);
+      return true;
     }
   }
+
   return false;
 } /* _mi_get_key */
 
@@ -1055,7 +1055,7 @@ uint _mi_keylength(MI_KEYDEF *keyinfo, const uchar *key) {
     if (keyseg->flag & HA_NULL_PART)
       if (!*key++) continue;
     if (keyseg->flag & (HA_SPACE_PACK | HA_BLOB_PART | HA_VAR_LENGTH_PART)) {
-      uint length = get_key_length(&key);
+      uint const length = get_key_length(&key);
       key += length;
     } else
       key += keyseg->length;
@@ -1079,7 +1079,7 @@ uint _mi_keylength_part(MI_KEYDEF *keyinfo, const uchar *key, HA_KEYSEG *end) {
     if (keyseg->flag & HA_NULL_PART)
       if (!*key++) continue;
     if (keyseg->flag & (HA_SPACE_PACK | HA_BLOB_PART | HA_VAR_LENGTH_PART)) {
-      uint length = get_key_length(&key);
+      uint const length = get_key_length(&key);
       key += length;
     } else
       key += keyseg->length;
@@ -1090,7 +1090,7 @@ uint _mi_keylength_part(MI_KEYDEF *keyinfo, const uchar *key, HA_KEYSEG *end) {
 /* Move a key */
 
 uchar *_mi_move_key(MI_KEYDEF *keyinfo, uchar *to, const uchar *from) {
-  size_t length = _mi_keylength(keyinfo, from);
+  size_t const length = _mi_keylength(keyinfo, from);
   memcpy(to, from, length);
   return to + length;
 }
@@ -1134,7 +1134,7 @@ int _mi_search_next(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key,
 
   if (nextflag & SEARCH_BIGGER) /* Next key */
   {
-    my_off_t tmp_pos = _mi_kpos(nod_flag, info->int_keypos);
+    my_off_t const tmp_pos = _mi_kpos(nod_flag, info->int_keypos);
     if (tmp_pos != HA_OFFSET_ERROR) {
       if ((error = _mi_search(info, keyinfo, key, USE_WHOLE_KEY,
                               nextflag | SEARCH_SAVE_BUFF, tmp_pos)) <= 0)
@@ -1492,7 +1492,7 @@ int _mi_calc_var_pack_key_length(MI_KEYDEF *keyinfo, uint nod_flag,
           return (int)length + ref_length - next_length_pack;
         }
         if (ref_length + pack_marker > new_ref_length) {
-          uint new_pack_length = new_ref_length - pack_marker;
+          uint const new_pack_length = new_ref_length - pack_marker;
           /* We must copy characters from the original key to the next key */
           s_temp->part_of_prev_key = new_ref_length;
           s_temp->prev_length = ref_length - new_pack_length;
@@ -1580,7 +1580,7 @@ int _mi_calc_bin_pack_key_length(MI_KEYDEF *keyinfo, uint nod_flag,
   {
     /* pack key against next key */
     uint next_length_pack;
-    uint next_length = get_key_pack_length(&next_key, &next_length_pack);
+    uint const next_length = get_key_pack_length(&next_key, &next_length_pack);
 
     /* If first key and next key is packed (only on delete) */
     if (!prev_key && org_key && next_length) {
