@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -728,36 +728,41 @@ class NdbTransaction {
       const NdbRecord *key_rec, const char *key_row,
       const NdbRecord *result_rec, char *result_row,
       NdbOperation::LockMode lock_mode = NdbOperation::LM_Read,
-      const unsigned char *result_mask = 0,
-      const NdbOperation::OperationOptions *opts = 0, Uint32 sizeOfOptions = 0);
+      const unsigned char *result_mask = nullptr,
+      const NdbOperation::OperationOptions *opts = nullptr,
+      Uint32 sizeOfOptions = 0);
   const NdbOperation *insertTuple(
       const NdbRecord *key_rec, const char *key_row, const NdbRecord *attr_rec,
-      const char *attr_row, const unsigned char *mask = 0,
-      const NdbOperation::OperationOptions *opts = 0, Uint32 sizeOfOptions = 0);
+      const char *attr_row, const unsigned char *mask = nullptr,
+      const NdbOperation::OperationOptions *opts = nullptr,
+      Uint32 sizeOfOptions = 0);
   const NdbOperation *insertTuple(
       const NdbRecord *combined_rec, const char *combined_row,
-      const unsigned char *mask = 0,
-      const NdbOperation::OperationOptions *opts = 0, Uint32 sizeOfOptions = 0);
+      const unsigned char *mask = nullptr,
+      const NdbOperation::OperationOptions *opts = nullptr,
+      Uint32 sizeOfOptions = 0);
   const NdbOperation *updateTuple(
       const NdbRecord *key_rec, const char *key_row, const NdbRecord *attr_rec,
-      const char *attr_row, const unsigned char *mask = 0,
-      const NdbOperation::OperationOptions *opts = 0, Uint32 sizeOfOptions = 0);
-  const NdbOperation *writeTuple(const NdbRecord *key_rec, const char *key_row,
-                                 const NdbRecord *attr_rec,
-                                 const char *attr_row,
-                                 const unsigned char *mask = 0,
-                                 const NdbOperation::OperationOptions *opts = 0,
-                                 Uint32 sizeOfOptions = 0);
+      const char *attr_row, const unsigned char *mask = nullptr,
+      const NdbOperation::OperationOptions *opts = nullptr,
+      Uint32 sizeOfOptions = 0);
+  const NdbOperation *writeTuple(
+      const NdbRecord *key_rec, const char *key_row, const NdbRecord *attr_rec,
+      const char *attr_row, const unsigned char *mask = nullptr,
+      const NdbOperation::OperationOptions *opts = nullptr,
+      Uint32 sizeOfOptions = 0);
   const NdbOperation *deleteTuple(
       const NdbRecord *key_rec, const char *key_row,
-      const NdbRecord *result_rec, char *result_row = 0,
-      const unsigned char *result_mask = 0,
-      const NdbOperation::OperationOptions *opts = 0, Uint32 sizeOfOptions = 0);
+      const NdbRecord *result_rec, char *result_row = nullptr,
+      const unsigned char *result_mask = nullptr,
+      const NdbOperation::OperationOptions *opts = nullptr,
+      Uint32 sizeOfOptions = 0);
 
 #ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
   const NdbOperation *refreshTuple(
       const NdbRecord *key_rec, const char *key_row,
-      const NdbOperation::OperationOptions *opts = 0, Uint32 sizeOfOptions = 0);
+      const NdbOperation::OperationOptions *opts = nullptr,
+      Uint32 sizeOfOptions = 0);
 #endif
 
   /**
@@ -785,8 +790,8 @@ class NdbTransaction {
   NdbScanOperation *scanTable(
       const NdbRecord *result_record,
       NdbOperation::LockMode lock_mode = NdbOperation::LM_Read,
-      const unsigned char *result_mask = 0,
-      const NdbScanOperation::ScanOptions *options = 0,
+      const unsigned char *result_mask = nullptr,
+      const NdbScanOperation::ScanOptions *options = nullptr,
       Uint32 sizeOfOptions = 0);
 
   /**
@@ -830,9 +835,9 @@ class NdbTransaction {
   NdbIndexScanOperation *scanIndex(
       const NdbRecord *key_record, const NdbRecord *result_record,
       NdbOperation::LockMode lock_mode = NdbOperation::LM_Read,
-      const unsigned char *result_mask = 0,
-      const NdbIndexScanOperation::IndexBound *bound = 0,
-      const NdbScanOperation::ScanOptions *options = 0,
+      const unsigned char *result_mask = nullptr,
+      const NdbIndexScanOperation::IndexBound *bound = nullptr,
+      const NdbScanOperation::ScanOptions *options = nullptr,
       Uint32 sizeOfOptions = 0);
 
   /**
@@ -847,7 +852,7 @@ class NdbTransaction {
    * Note, query uses LM_CommittedRead. The LockMode parameter is ignored.
    */
   NdbQuery *createQuery(
-      const NdbQueryDef *query, const NdbQueryParamValue paramValue[] = 0,
+      const NdbQueryDef *query, const NdbQueryParamValue paramValue[] = nullptr,
       NdbOperation::LockMode = NdbOperation::LM_CommittedRead);
 
   /* LockHandle methods */
@@ -1055,12 +1060,12 @@ class NdbTransaction {
   static Uint32 getMagicNumber() { return (Uint32)0x37412619; }
 
   NdbOperation *getNdbOperation(const class NdbTableImpl *aTable,
-                                NdbOperation *aNextOp = 0);
+                                NdbOperation *aNextOp = nullptr);
 
   NdbIndexScanOperation *getNdbScanOperation(const class NdbTableImpl *aTable);
   NdbIndexOperation *getNdbIndexOperation(const class NdbIndexImpl *anIndex,
                                           const class NdbTableImpl *aTable,
-                                          NdbOperation *aNextOp = 0);
+                                          NdbOperation *aNextOp = nullptr);
   NdbIndexScanOperation *getNdbIndexScanOperation(const NdbIndexImpl *index,
                                                   const NdbTableImpl *table);
 
@@ -1070,7 +1075,7 @@ class NdbTransaction {
       const char *key_row, const NdbRecord *attribute_record,
       const char *attribute_row, const unsigned char *mask,
       const NdbOperation::OperationOptions *opts, Uint32 sizeOfOptions,
-      const NdbLockHandle *lh = 0);
+      const NdbLockHandle *lh = nullptr);
 
   void handleExecuteCompletion();
 
@@ -1157,9 +1162,10 @@ class NdbTransaction {
     InCompletedList
   } theListState;
 
-  Uint32 theDBnode;        // The database node we are connected to
-  Uint32 theNodeSequence;  // The sequence no of the db node
-  bool theReleaseOnClose;
+  Uint32 theDBnode;             // The database node we are connected to
+  Uint32 theNodeSequence;       // The sequence no of the db node
+  bool theReleaseOnClose;       // Data node gone, recycle API object
+  bool theForceReleaseOnClose;  // Data node out of sync, force TC release
 
   /**
    * handle transaction spanning
@@ -1208,7 +1214,8 @@ class NdbTransaction {
   bool checkState_TransId(const Uint32 *transId) const;
 
   // Check table and index objects wrt current connection - for debugging
-  bool checkSchemaObjects(const NdbTableImpl *tab, const NdbIndexImpl *idx = 0);
+  bool checkSchemaObjects(const NdbTableImpl *tab,
+                          const NdbIndexImpl *idx = nullptr);
 
   void remove_list(NdbOperation *&head, NdbOperation *);
   void define_scan_op(NdbIndexScanOperation *);
@@ -1247,7 +1254,6 @@ inline Uint32 NdbTransaction::get_send_size() { return 0; }
 
 inline void NdbTransaction::set_send_size(Uint32 send_size) {
   (void)send_size;  // unused
-  return;
 }
 
 #ifdef NDB_NO_DROPPED_SIGNAL
@@ -1259,14 +1265,11 @@ inline Uint32 NdbTransaction::getMagicNumberFromObject() const {
 }
 
 inline int NdbTransaction::checkMagicNumber() {
-  if (theMagicNumber == getMagicNumber())
-    return 0;
-  else {
+  if (theMagicNumber == getMagicNumber()) return 0;
 #ifdef NDB_NO_DROPPED_SIGNAL
-    abort();
+  abort();
 #endif
-    return -1;
-  }
+  return -1;
 }
 
 inline bool NdbTransaction::checkState_TransId(const Uint32 *transId) const {
