@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,13 +25,15 @@
 
 #include "storage/ndb/plugin/ha_ndb_index_stat.h"
 
-#include <ctype.h>
 #include <mysql/plugin.h>
 #include <mysql/psi/mysql_thread.h>
-#include <time.h>
 #include <atomic>
+#include <cctype>
+#include <ctime>
 
+#include "m_string.h"
 #include "my_dbug.h"
+#include "nulls.h"
 #include "sql/field.h"
 #include "sql/mysqld.h"  // LOCK_global_system_variables
 #include "sql/partition_info.h"
@@ -433,7 +435,7 @@ static int ndb_index_stat_str2opt(const char *str, Ndb_index_stat_opt &opt) {
   strcpy(buf, str);
 
   char *p = buf;
-  while (1) {
+  while (true) {
     while (isspace(*p)) p++;
     if (*p == 0) break;
 
@@ -635,8 +637,8 @@ void Ndb_index_stat_glob::set_status() {
   // cache size
   const uint cache_limit = opt.get(Ndb_index_stat_opt::Icache_limit);
   const uint cache_total = cache_query_bytes + cache_clean_bytes;
-  double cache_pct = (double)0.0;
-  double cache_high_pct = (double)0.0;
+  double cache_pct = 0.0;
+  double cache_high_pct = 0.0;
   if (cache_limit != 0) {
     cache_pct = (double)100.0 * (double)cache_total / (double)cache_limit;
     cache_high_pct =
@@ -863,8 +865,8 @@ struct Ndb_index_stat_snap {
 static Ndb_index_stat *ndb_index_stat_alloc(const NDBINDEX *index,
                                             const NDBTAB *table, int &err_out) {
   err_out = 0;
-  Ndb_index_stat *st = new Ndb_index_stat;
-  NdbIndexStat *is = new NdbIndexStat;
+  auto *st = new Ndb_index_stat;
+  auto *is = new NdbIndexStat;
   if (st != nullptr && is != nullptr) {
     st->is = is;
     st->index_id = index->getObjectId();
@@ -878,8 +880,8 @@ static Ndb_index_stat *ndb_index_stat_alloc(const NDBINDEX *index,
   } else {
     err_out = NdbIndexStat::NoMemError;
   }
-  if (is != nullptr) delete is;
-  if (st != nullptr) delete st;
+  delete is;
+  delete st;
   return nullptr;
 }
 
@@ -956,7 +958,7 @@ static Ndb_index_stat *ndb_index_stat_get_share(NDB_SHARE *share,
     snap.sample_version = st->sample_version;
     snap.error_count = st->error_count;
     st->access_time = now;
-  } while (0);
+  } while (false);
 
   if (err_out == 0) {
     st->acquire_client_ref();
@@ -1342,9 +1344,9 @@ static void ndb_index_stat_proc_idle(Ndb_index_stat_proc &pr,
   const longlong clean_delay = opt.get(Ndb_index_stat_opt::Iclean_delay);
   const longlong check_delay = opt.get(Ndb_index_stat_opt::Icheck_delay);
 
-  const longlong pr_now = (longlong)pr.now;
-  const longlong st_read_time = (longlong)st->read_time;
-  const longlong st_check_time = (longlong)st->check_time;
+  const longlong pr_now = pr.now;
+  const longlong st_read_time = st->read_time;
+  const longlong st_check_time = st->check_time;
 
   const longlong clean_wait = st_read_time + clean_delay - pr_now;
   const longlong check_wait = st_check_time + check_delay - pr_now;
@@ -1487,8 +1489,7 @@ static bool ndb_index_stat_proc_evict() {
 
   const uint cache_lowpct = opt.get(Ndb_index_stat_opt::Icache_lowpct);
   const uint cache_limit = opt.get(Ndb_index_stat_opt::Icache_limit);
-  if (100 * curr_size <= cache_lowpct * cache_limit) return false;
-  return true;
+  return 100 * curr_size > cache_lowpct * cache_limit;
 }
 
 /* Check if st1 is better or as good to evict than st2 */
@@ -1509,7 +1510,7 @@ static void ndb_index_stat_proc_evict(Ndb_index_stat_proc &pr, int lt) {
   const uint batch = opt.get(Ndb_index_stat_opt::Ievict_batch);
   const longlong evict_delay = opt.get(Ndb_index_stat_opt::Ievict_delay);
   pr.now = ndb_index_stat_time();
-  const longlong pr_now = (longlong)pr.now;
+  const longlong pr_now = pr.now;
 
   if (!ndb_index_stat_proc_evict()) return;
 
@@ -1523,7 +1524,7 @@ static void ndb_index_stat_proc_evict(Ndb_index_stat_proc &pr, int lt) {
   while (st_loop != nullptr && st_lru_cnt < batch) {
     Ndb_index_stat *st = st_loop;
     st_loop = st_loop->list_next;
-    const longlong st_read_time = (longlong)st->read_time;
+    const longlong st_read_time = st->read_time;
     if (st_read_time + evict_delay <= pr_now &&
         st->query_bytes + st->clean_bytes != 0 && !st->to_delete) {
       /* Insertion sort into the batch from the end */
@@ -1653,8 +1654,8 @@ static void ndb_index_stat_proc_error(Ndb_index_stat_proc &pr,
   const Ndb_index_stat_opt &opt = ndb_index_stat_opt;
   const longlong error_delay = opt.get(Ndb_index_stat_opt::Ierror_delay);
 
-  const longlong pr_now = (longlong)pr.now;
-  const longlong st_error_time = (longlong)st->error_time;
+  const longlong pr_now = pr.now;
+  const longlong st_error_time = st->error_time;
   const longlong error_wait = st_error_time + error_delay - pr_now;
 
   DBUG_PRINT("index_stat", ("st %s error_wait:%lld error_count:%u"
@@ -1746,7 +1747,7 @@ static void ndb_index_stat_proc_event(Ndb_index_stat_proc &pr) {
   }
   if (ret == 0) return;
 
-  while (1) {
+  while (true) {
     ret = is->next_listener(ndb);
     DBUG_PRINT("index_stat", ("next_listener ret: %d", ret));
     if (ret == -1) {
@@ -2131,7 +2132,7 @@ void Ndb_index_stat_thread::do_run() {
   set_timespec(&abstime, 0);
   for (;;) {
     mysql_mutex_lock(&LOCK_client_waiting);
-    if (client_waiting == false) {
+    if (!client_waiting) {
       const int ret = mysql_cond_timedwait(&COND_client_waiting,
                                            &LOCK_client_waiting, &abstime);
       if (ret == ETIMEDOUT)
@@ -2209,7 +2210,7 @@ void Ndb_index_stat_thread::do_run() {
       ndb_index_stat_set_allow(true);
       pr.busy = false;
       ndb_index_stat_proc(pr);
-    } while (0);
+    } while (false);
 
     /* Calculate new time to wake up */
 
@@ -2460,7 +2461,7 @@ int ha_ndbcluster::ndb_index_stat_query(uint inx, const key_range *min_key,
       mysql_mutex_unlock(&ndb_index_stat_thread.stat_mutex);
       break;
     }
-  } while (0);
+  } while (false);
 
   /* Release reference to st */
   st->release_client_ref();
