@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2011, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -49,7 +50,7 @@ st_row_event_key_info::st_row_event_key_info(const NdbDictionary::Table *_table,
       packed_key(_key_buff),
       packed_key_len(_key_buff_len),
       transaction_id(_transaction_id),
-      hash_next(NULL) {}
+      hash_next(nullptr) {}
 
 Uint64 st_row_event_key_info::getTransactionId() const {
   return transaction_id;
@@ -89,7 +90,7 @@ st_trans_dependency::st_trans_dependency(st_transaction *_target_transaction,
     : target_transaction(_target_transaction),
       dependent_transaction(_dependent_transaction),
       next_entry(_next),
-      hash_next(NULL) {}
+      hash_next(nullptr) {}
 
 st_transaction *st_trans_dependency::getTargetTransaction() const {
   return target_transaction;
@@ -132,8 +133,8 @@ void st_trans_dependency::setNext(st_trans_dependency *_next) {
 st_transaction::st_transaction(Uint64 _transaction_id)
     : transaction_id(_transaction_id),
       in_conflict(false),
-      dependency_list_head(NULL),
-      hash_next(NULL) {}
+      dependency_list_head(nullptr),
+      hash_next(nullptr) {}
 
 Uint64 st_transaction::getTransactionId() const { return transaction_id; }
 
@@ -178,7 +179,7 @@ template class LinkedStack<Uint64, st_mem_root_allocator>;
  * Given a table, key_record and record, this method will
  * determine how many significant bytes the key contains,
  * and if a buffer is passed, will copy the bytes into the
- * buffer.
+ * buffer. Returns -1 on failure and 0 on success
  */
 static int pack_key_to_buffer(const NdbDictionary::Table *table,
                               const NdbRecord *key_rec, const uchar *record,
@@ -190,7 +191,7 @@ static int pack_key_to_buffer(const NdbDictionary::Table *table,
    */
   Uint32 attr_id;
   Uint32 buff_offset = 0;
-  NdbDictionary::getFirstAttrId(key_rec, attr_id);
+  if (!NdbDictionary::getFirstAttrId(key_rec, attr_id)) return -1;
 
   do {
     Uint32 from_offset = 0;
@@ -211,6 +212,9 @@ static int pack_key_to_buffer(const NdbDictionary::Table *table,
         byte_len = uint2korr(&record[from_offset]);
         from_offset += 2;
         break;
+      default:
+        assert(false);
+        return -1;
     };
     assert((buff_offset + byte_len) <= buff_len);
 
@@ -223,23 +227,23 @@ static int pack_key_to_buffer(const NdbDictionary::Table *table,
   return 0;
 }
 
-static Uint32 determine_packed_key_size(const NdbDictionary::Table *table,
-                                        const NdbRecord *key_rec,
-                                        const uchar *record) {
-  Uint32 key_size = ~Uint32(0);
+static bool determine_packed_key_size(const NdbDictionary::Table *table,
+                                      const NdbRecord *key_rec,
+                                      const uchar *record,
+                                      Uint32 &required_buff_size) {
   /* Use pack_key_to_buffer to calculate length required */
-  pack_key_to_buffer(table, key_rec, record, NULL, key_size);
-  return key_size;
+  return pack_key_to_buffer(table, key_rec, record, nullptr,
+                            required_buff_size) != -1;
 }
 
 /* st_mem_root_allocator implementation */
 void *st_mem_root_allocator::alloc(void *ctx, size_t bytes) {
-  st_mem_root_allocator *a = (st_mem_root_allocator *)ctx;
+  auto *a = (st_mem_root_allocator *)ctx;
   return a->mem_root->Alloc(bytes);
 }
 
 void *st_mem_root_allocator::mem_calloc(void *ctx, size_t nelem, size_t bytes) {
-  st_mem_root_allocator *a = (st_mem_root_allocator *)ctx;
+  auto *a = (st_mem_root_allocator *)ctx;
   return a->mem_root->Alloc(nelem * bytes);
 }
 
@@ -255,7 +259,7 @@ st_mem_root_allocator::st_mem_root_allocator(MEM_ROOT *_mem_root)
 /* DependencyTracker implementation */
 
 DependencyTracker *DependencyTracker::newDependencyTracker(MEM_ROOT *mem_root) {
-  DependencyTracker *dt = NULL;
+  DependencyTracker *dt = nullptr;
   // Allocate memory from MEM_ROOT
   void *mem = mem_root->Alloc(sizeof(DependencyTracker));
   if (mem) {
@@ -272,7 +276,7 @@ DependencyTracker::DependencyTracker(MEM_ROOT *mem_root)
       dependency_hash(&mra),
       iteratorTodo(ITERATOR_STACK_BLOCKSIZE, &mra),
       conflicting_trans_count(0),
-      error_text(NULL) {
+      error_text(nullptr) {
   /* TODO Get sizes from somewhere */
   key_hash.setSize(1024);
   trans_hash.setSize(100);
@@ -285,15 +289,20 @@ int DependencyTracker::track_operation(const NdbDictionary::Table *table,
                                        Uint64 transaction_id) {
   DBUG_TRACE;
 
-  Uint32 required_buff_size = determine_packed_key_size(table, key_rec, row);
+  Uint32 required_buff_size = ~Uint32(0);
+  if (!determine_packed_key_size(table, key_rec, row, required_buff_size)) {
+    if (!error_text)
+      error_text = "track_operation : Failed to determine packed key size";
+    return -1;
+  }
   DBUG_PRINT("info", ("Required length for key : %u", required_buff_size));
 
   /* Alloc space for packed key and struct in MEM_ROOT */
-  uchar *packed_key_buff = (uchar *)mra.mem_root->Alloc(required_buff_size);
+  auto *packed_key_buff = (uchar *)mra.mem_root->Alloc(required_buff_size);
   void *element_mem = mra.mem_root->Alloc(sizeof(st_row_event_key_info));
 
   if (pack_key_to_buffer(table, key_rec, row, packed_key_buff,
-                         required_buff_size)) {
+                         required_buff_size) == -1) {
     if (!error_text) error_text = "track_operation : Failed packing key";
     return -1;
   }
@@ -306,7 +315,7 @@ int DependencyTracker::track_operation(const NdbDictionary::Table *table,
     }
   }
 
-  st_row_event_key_info *key_info = new (element_mem) st_row_event_key_info(
+  auto *key_info = new (element_mem) st_row_event_key_info(
       table, packed_key_buff, required_buff_size, transaction_id);
 
   /* Now try to add element to hash */
@@ -331,28 +340,26 @@ int DependencyTracker::track_operation(const NdbDictionary::Table *table,
       */
       existing->updateRowTransactionId(newTransIdOnRow);
 
-      assert(res == 0 || error_text != NULL);
+      assert(res == 0 || error_text != nullptr);
 
       return res;
-    } else {
-      /*
-         How can we have two updates to the same row with the
-         same transaction id?  Only if the transaction id
-         is invalid (e.g. not set)
-         In normal cases with only one upstream master, each
-         distinct master user transaction will have a unique
-         id, and all operations on a row in that transaction
-         will be merged in TUP prior to emitting a SUMA
-         event.
-         This could be relaxed for more complex upstream
-         topologies, but acts as a sanity guard currently.
-      */
-      if (existingTransIdOnRow != InvalidTransactionId) {
-        assert(false);
-        error_text =
-            "Two row operations to same key sharing user transaction id";
-        return -1;
-      }
+    }
+    /*
+      How can we have two updates to the same row with the
+      same transaction id?  Only if the transaction id
+      is invalid (e.g. not set)
+      In normal cases with only one upstream master, each
+      distinct master user transaction will have a unique
+      id, and all operations on a row in that transaction
+      will be merged in TUP prior to emitting a SUMA
+      event.
+      This could be relaxed for more complex upstream
+      topologies, but acts as a sanity guard currently.
+    */
+    if (existingTransIdOnRow != InvalidTransactionId) {
+      assert(false);
+      error_text = "Two row operations to same key sharing user transaction id";
+      return -1;
     }
   }
 
@@ -404,7 +411,7 @@ bool DependencyTracker::in_conflict(Uint64 trans_id) {
   DBUG_TRACE;
   DBUG_PRINT("info", ("trans_id %llu", trans_id));
   st_transaction key(trans_id);
-  const st_transaction *entry = NULL;
+  const st_transaction *entry = nullptr;
 
   /*
     If transaction hash entry exists, check it for
@@ -413,16 +420,16 @@ bool DependencyTracker::in_conflict(Uint64 trans_id) {
   if ((entry = trans_hash.get(&key))) {
     DBUG_PRINT("info", ("in_conflict : %u", entry->getInConflict()));
     return entry->getInConflict();
-  } else {
-    assert(!TRACK_ALL_TRANSACTIONS);
   }
+  assert(!TRACK_ALL_TRANSACTIONS);
+
   return false;
 }
 
 st_transaction *DependencyTracker::get_or_create_transaction(Uint64 trans_id) {
   DBUG_TRACE;
   st_transaction transKey(trans_id);
-  st_transaction *transEntry = NULL;
+  st_transaction *transEntry = nullptr;
 
   if (!(transEntry = trans_hash.get(&transKey))) {
     /*
@@ -440,7 +447,7 @@ st_transaction *DependencyTracker::get_or_create_transaction(Uint64 trans_id) {
 
       if (!trans_hash.add(transEntry)) {
         st_mem_root_allocator::mem_free(&mra, transEntry); /* For show */
-        transEntry = NULL;
+        transEntry = nullptr;
       }
     }
   }
@@ -467,9 +474,9 @@ int DependencyTracker::add_dependency(Uint64 trans_id,
   }
 
   /* Now lookup dependency.  Add it if not already present */
-  st_trans_dependency depKey(targetEntry, dependentEntry, NULL);
-  st_trans_dependency *dep = NULL;
-  if (!(dep = dependency_hash.get(&depKey))) {
+  st_trans_dependency depKey(targetEntry, dependentEntry, nullptr);
+  st_trans_dependency *dep = dependency_hash.get(&depKey);
+  if (dep == nullptr) {
     DBUG_PRINT("info", ("Creating new dependency hash entry for "
                         "dependency of %llu on %llu.",
                         dependentEntry->getTransactionId(),
@@ -533,14 +540,15 @@ st_transaction *DependencyTracker::get_next_dependency(
 
   assert(iteratorTodo.size() == 0);
   DBUG_PRINT("info", ("No more dependencies to visit"));
-  return NULL;
+  return nullptr;
 }
 
+#ifndef NDEBUG
 void DependencyTracker::dump_dependents(Uint64 trans_id) {
   fprintf(stderr, "Dumping dependents of transid %llu : ", trans_id);
 
   st_transaction key(trans_id);
-  const st_transaction *dependent = NULL;
+  const st_transaction *dependent = nullptr;
 
   if ((dependent = trans_hash.get(&key))) {
     reset_dependency_iterator();
@@ -577,7 +585,7 @@ bool DependencyTracker::verify_graph() {
   HashMap2<st_transaction, true, st_mem_root_allocator>::Iterator it(
       trans_hash);
 
-  st_transaction *root = NULL;
+  st_transaction *root = nullptr;
 
   while ((root = it.next())) {
     bool in_conflict = root->getInConflict();
@@ -605,6 +613,7 @@ bool DependencyTracker::verify_graph() {
 
   return true;
 }
+#endif
 
 const char *DependencyTracker::get_error_text() const { return error_text; }
 
