@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
-    Use is subject to license terms.
+   Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -33,7 +32,7 @@ bool printALTER_INDX_REQ(FILE *output, const Uint32 *theData, Uint32 len,
     assert(false);
     return false;
   }
-  const AlterIndxReq *sig = (const AlterIndxReq *)theData;
+  const auto *sig = (const AlterIndxReq *)theData;
   fprintf(output, " clientRef: 0x%x", sig->clientRef);
   fprintf(output, " clientData: %u", sig->clientData);
   fprintf(output, "\n");
@@ -57,7 +56,7 @@ bool printALTER_INDX_CONF(FILE *output, const Uint32 *theData, Uint32 len,
     assert(false);
     return false;
   }
-  const AlterIndxConf *sig = (const AlterIndxConf *)theData;
+  const auto *sig = (const AlterIndxConf *)theData;
   fprintf(output, " senderRef: 0x%x", sig->senderRef);
   fprintf(output, " clientData: %u", sig->clientData);
   fprintf(output, "\n");
@@ -75,7 +74,7 @@ bool printALTER_INDX_REF(FILE *output, const Uint32 *theData, Uint32 len,
     assert(false);
     return false;
   }
-  const AlterIndxRef *sig = (const AlterIndxRef *)theData;
+  const auto *sig = (const AlterIndxRef *)theData;
   fprintf(output, " senderRef: 0x%x", sig->senderRef);
   fprintf(output, " clientData: %u", sig->clientData);
   fprintf(output, "\n");
