@@ -34,10 +34,19 @@ bool printDEFINE_BACKUP_REQ(FILE *out, const Uint32 *data, Uint32 len,
   }
 
   const auto *sig = (const DefineBackupReq *)data;
-  fprintf(out, " backupPtr: %d backupId: %d clientRef: %d clientData: %d\n",
-          sig->backupPtr, sig->backupId, sig->clientRef, sig->clientData);
-  fprintf(out, " backupKey: [ %08x%08x ] DataLength: %d\n", sig->backupKey[0],
-          sig->backupKey[1], sig->backupDataLen);
+  fprintf(out,
+          " backupPtr: %u backupId: %u clientRef: %u clientData: %u "
+          "senderRef: %u senderData: %u\n",
+          sig->backupPtr, sig->backupId, sig->clientRef, sig->clientData,
+          sig->senderRef, sig->senderData);
+  fprintf(out,
+          " backupKey: [ %08x%08x ] DataLength: %u flags: %u masterRef: %u\n",
+          sig->backupKey[0], sig->backupKey[1], sig->backupDataLen, sig->flags,
+          sig->masterRef);
+  if (len == sig->SignalLength_v1) {
+    char buf[NdbNodeBitmask48::TextLength + 1];
+    fprintf(out, " nodes: %s\n", sig->nodes.getText(buf));
+  }
   return true;
 }
 
@@ -49,8 +58,8 @@ bool printDEFINE_BACKUP_REF(FILE *out, const Uint32 *data, Uint32 len,
   }
 
   const auto *sig = (const DefineBackupRef *)data;
-  fprintf(out, " backupPtr: %d backupId: %d errorCode: %d\n", sig->backupPtr,
-          sig->backupId, sig->errorCode);
+  fprintf(out, " backupPtr: %u backupId: %u errorCode: %u nodeId: %u\n",
+          sig->backupPtr, sig->backupId, sig->errorCode, sig->nodeId);
   return true;
 }
 
@@ -61,7 +70,7 @@ bool printDEFINE_BACKUP_CONF(FILE *out, const Uint32 *data, Uint32 l,
     return false;
   }
   const auto *sig = (const DefineBackupConf *)data;
-  fprintf(out, " backupPtr: %d backupId: %d\n", sig->backupPtr, sig->backupId);
+  fprintf(out, " backupPtr: %u backupId: %u\n", sig->backupPtr, sig->backupId);
   return true;
 }
 
@@ -72,7 +81,8 @@ bool printSTART_BACKUP_REQ(FILE *out, const Uint32 *data, Uint32 l,
     return false;
   }
   const auto *sig = (const StartBackupReq *)data;
-  fprintf(out, " backupPtr: %d backupId: %d\n", sig->backupPtr, sig->backupId);
+  fprintf(out, " backupPtr: %u backupId: %u senderRef: %u senderData: %u\n",
+          sig->backupPtr, sig->backupId, sig->senderRef, sig->senderData);
   return true;
 }
 
@@ -84,8 +94,8 @@ bool printSTART_BACKUP_REF(FILE *out, const Uint32 *data, Uint32 len,
   }
 
   const auto *sig = (const StartBackupRef *)data;
-  fprintf(out, " backupPtr: %d backupId: %d errorCode: %d\n", sig->backupPtr,
-          sig->backupId, sig->errorCode);
+  fprintf(out, " backupPtr: %u backupId: %u errorCode: %u nodeId: %u\n",
+          sig->backupPtr, sig->backupId, sig->errorCode, sig->nodeId);
   return true;
 }
 
@@ -97,7 +107,7 @@ bool printSTART_BACKUP_CONF(FILE *out, const Uint32 *data, Uint32 l,
   }
 
   const auto *sig = (const StartBackupConf *)data;
-  fprintf(out, " backupPtr: %d backupId: %d\n", sig->backupPtr, sig->backupId);
+  fprintf(out, " backupPtr: %u backupId: %u\n", sig->backupPtr, sig->backupId);
   return true;
 }
 
@@ -109,9 +119,11 @@ bool printBACKUP_FRAGMENT_REQ(FILE *out, const Uint32 *data, Uint32 l,
   }
 
   const auto *sig = (const BackupFragmentReq *)data;
-  fprintf(out, " backupPtr: %d backupId: %d\n", sig->backupPtr, sig->backupId);
-  fprintf(out, " tableId: %d fragmentNo: %d (count = %d)\n", sig->tableId,
+  fprintf(out, " backupPtr: %u backupId: %u\n", sig->backupPtr, sig->backupId);
+  fprintf(out, " tableId: %u fragmentNo: %u (count = %u)\n", sig->tableId,
           sig->fragmentNo, sig->count);
+  fprintf(out, " senderRef: %u senderData: %u\n", sig->senderRef,
+          sig->senderData);
   return true;
 }
 
@@ -123,8 +135,9 @@ bool printBACKUP_FRAGMENT_REF(FILE *out, const Uint32 *data, Uint32 l,
   }
 
   const auto *sig = (const BackupFragmentRef *)data;
-  fprintf(out, " backupPtr: %d backupId: %d nodeId: %d errorCode: %d\n",
+  fprintf(out, " backupPtr: %u backupId: %u nodeId: %u errorCode: %u\n",
           sig->backupPtr, sig->backupId, sig->nodeId, sig->errorCode);
+  fprintf(out, " tableId: %u fragmentNo: %u\n", sig->tableId, sig->fragmentNo);
   return true;
 }
 
@@ -136,11 +149,11 @@ bool printBACKUP_FRAGMENT_CONF(FILE *out, const Uint32 *data, Uint32 l,
   }
 
   const auto *sig = (const BackupFragmentConf *)data;
-  fprintf(out, " backupPtr: %d backupId: %d\n", sig->backupPtr, sig->backupId);
-  fprintf(out, " tableId: %d fragmentNo: %d records: %llu bytes: %llu\n",
+  fprintf(out, " backupPtr: %u backupId: %u\n", sig->backupPtr, sig->backupId);
+  fprintf(out, " tableId: %u fragmentNo: %u records: %llu bytes: %llu\n",
           sig->tableId, sig->fragmentNo,
-          sig->noOfRecordsLow + (((Uint64)sig->noOfRecordsHigh) << 32),
-          sig->noOfBytesLow + (((Uint64)sig->noOfBytesHigh) << 32));
+          (Uint64(sig->noOfRecordsHigh) << 32) + sig->noOfRecordsLow,
+          (Uint64(sig->noOfBytesHigh) << 32) + sig->noOfBytesLow);
   return true;
 }
 
@@ -152,7 +165,9 @@ bool printSTOP_BACKUP_REQ(FILE *out, const Uint32 *data, Uint32 l,
   }
 
   const auto *sig = (const StopBackupReq *)data;
-  fprintf(out, " backupPtr: %d backupId: %d\n", sig->backupPtr, sig->backupId);
+  fprintf(out, " backupPtr: %u backupId: %u\n", sig->backupPtr, sig->backupId);
+  fprintf(out, " startGCP: %u stopGCP: %u senderRef: %u senderData: %u\n",
+          sig->startGCP, sig->stopGCP, sig->senderRef, sig->senderData);
   return true;
 }
 
@@ -164,8 +179,8 @@ bool printSTOP_BACKUP_REF(FILE *out, const Uint32 *data, Uint32 len,
   }
 
   const auto *sig = (const StopBackupRef *)data;
-  fprintf(out, " backupPtr: %d backupId: %d errorCode: %d\n", sig->backupPtr,
-          sig->backupId, sig->errorCode);
+  fprintf(out, " backupPtr: %u backupId: %u errorCode: %u nodeId: %u\n",
+          sig->backupPtr, sig->backupId, sig->errorCode, sig->nodeId);
   return true;
 }
 
@@ -177,7 +192,10 @@ bool printSTOP_BACKUP_CONF(FILE *out, const Uint32 *data, Uint32 l,
   }
 
   const auto *sig = (const StopBackupConf *)data;
-  fprintf(out, " backupPtr: %d backupId: %d\n", sig->backupPtr, sig->backupId);
+  fprintf(out, " backupPtr: %u backupId: %u\n", sig->backupPtr, sig->backupId);
+  fprintf(out, " noOfLogBytes: %llu noOfLogRecords: %llu\n",
+          (Uint64(sig->noOfLogBytesHigh) << 32) + sig->noOfLogBytesLow,
+          (Uint64(sig->noOfLogRecordsHigh) << 32) + sig->noOfLogRecordsLow);
   return true;
 }
 
