@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2010, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2010, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -116,7 +117,8 @@ bool Loopback_Transporter::doSend(bool need_wakeup) {
       assert(sum >= sum_sent);
       remain = sum - sum_sent;
       break;
-    } else if (nBytesSent > 0)  // Sent some, more pending
+    }
+    if (nBytesSent > 0)  // Sent some, more pending
     {
       sum_sent += nBytesSent;
       remain -= nBytesSent;
