@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,7 +26,7 @@
 #include <ndb_global.h>
 #include "util/require.h"
 
-#include <time.h>
+#include <ctime>
 
 #include <NdbMutex.h>
 #include <NdbSleep.h>
@@ -143,7 +143,7 @@ bool SHM_Transporter::setupBuffers() {
   sizeOfBuffer -= 2 * sharedSize;
   sizeOfBuffer /= 2;
 
-  Uint32 *base1 = (Uint32 *)shmBuf;
+  auto *base1 = (Uint32 *)shmBuf;
 
   Uint32 *sharedReadIndex1 = base1;
   Uint32 *sharedWriteIndex1 = base1 + 1;
@@ -153,7 +153,7 @@ bool SHM_Transporter::setupBuffers() {
   serverMutex = (NdbMutex *)(base1 + 16);
   char *startOfBuf1 = shmBuf + sharedSize;
 
-  Uint32 *base2 = (Uint32 *)(shmBuf + sizeOfBuffer + sharedSize);
+  auto *base2 = (Uint32 *)(shmBuf + sizeOfBuffer + sharedSize);
   Uint32 *sharedReadIndex2 = base2;
   Uint32 *sharedWriteIndex2 = base2 + 1;
   clientStatusFlag = base2 + 4;
@@ -265,75 +265,90 @@ bool SHM_Transporter::connect_server_impl(NdbSocket &&sockfd) {
   SocketOutputStream s_output(sockfd);
   SocketInputStream s_input(sockfd);
 
-  // Create
-  if (!_shmSegCreated) {
-    if (!ndb_shm_create()) {
-      DEBUG_FPRINTF((stderr,
-                     "(%u)connect_server_impl failed LINE:%d,"
-                     " to remote node %d\n",
-                     localNodeId, __LINE__, remoteNodeId));
-      DBUG_RETURN(false);
-    }
-    _shmSegCreated = true;
-    DEBUG_FPRINTF(
-        (stderr, "(%u)ndb_shm_create()(%u)\n", localNodeId, remoteNodeId));
-  }
-
-  // Attach
-  if (!_attached) {
-    if (!ndb_shm_attach()) {
-      DEBUG_FPRINTF((stderr,
-                     "(%u)connect_server_impl failed LINE:%d,"
-                     " to remote node %d\n",
-                     localNodeId, __LINE__, remoteNodeId));
-      DBUG_RETURN(false);
-    }
-    _attached = true;
-    DEBUG_FPRINTF(
-        (stderr, "(%u)ndb_shm_attach()(%u)\n", localNodeId, remoteNodeId));
-  }
-
-  require(!setupBuffersDone);
+  do  // Close socket on errors
   {
-    DEBUG_FPRINTF((stderr, "(%u)setupBuffers(%u) Line:%d\n", localNodeId,
-                   remoteNodeId, __LINE__));
-    if (setupBuffers()) {
-      g_eventLogger->info("Shared memory not supported on this platform");
-      detach_shm(false);
-      DBUG_RETURN(false);
+    // Create
+    if (!_shmSegCreated) {
+      if (!ndb_shm_create()) {
+        DEBUG_FPRINTF((stderr,
+                       "(%u)connect_server_impl failed LINE:%d,"
+                       " to remote node %d\n",
+                       localNodeId, __LINE__, remoteNodeId));
+        break;
+      }
+      _shmSegCreated = true;
+      DEBUG_FPRINTF(
+          (stderr, "(%u)ndb_shm_create()(%u)\n", localNodeId, remoteNodeId));
     }
-    setupBuffersDone = true;
-  }
 
-  // Send ok to client
-  s_output.println("shm server 1 ok: %d", m_transporter_registry.m_shm_own_pid);
+    // Attach
+    if (!_attached) {
+      if (!ndb_shm_attach()) {
+        DEBUG_FPRINTF((stderr,
+                       "(%u)connect_server_impl failed LINE:%d,"
+                       " to remote node %d\n",
+                       localNodeId, __LINE__, remoteNodeId));
+        break;
+      }
+      _attached = true;
+      DEBUG_FPRINTF(
+          (stderr, "(%u)ndb_shm_attach()(%u)\n", localNodeId, remoteNodeId));
+    }
 
-  // Wait for ok from client
-  char buf[256];
-  DBUG_PRINT("info", ("Wait for ok from client"));
-  if (s_input.gets(buf, sizeof(buf)) == nullptr) {
-    DEBUG_FPRINTF((stderr,
-                   "(%u)connect_server_impl failed LINE:%d,"
-                   " to remote node %d\n",
-                   localNodeId, __LINE__, remoteNodeId));
-    detach_shm(false);
-    DBUG_RETURN(false);
-  }
+    require(!setupBuffersDone);
+    {
+      DEBUG_FPRINTF((stderr, "(%u)setupBuffers(%u) Line:%d\n", localNodeId,
+                     remoteNodeId, __LINE__));
+      if (setupBuffers()) {
+        g_eventLogger->info(
+            "Node %u transporter to node %u: SHM not supported on this "
+            "platform; disconnecting",
+            localNodeId, remoteNodeId);
+        detach_shm(false);
+        break;
+      }
+      setupBuffersDone = true;
+    }
 
-  if (sscanf(buf, "shm client 1 ok: %d", &m_remote_pid) != 1) {
-    DEBUG_FPRINTF((stderr,
-                   "(%u)connect_server_impl failed LINE:%d,"
-                   " to remote node %d\n",
-                   localNodeId, __LINE__, remoteNodeId));
-    detach_shm(false);
-    DBUG_RETURN(false);
-  }
+    // Send ok to client
+    s_output.println("shm server 1 ok: %d",
+                     m_transporter_registry.m_shm_own_pid);
 
-  DEBUG_FPRINTF(
-      (stderr, "(%u)connect_common()(%u)\n", localNodeId, remoteNodeId));
-  int r = connect_common();
+    // Wait for ok from client
+    char buf[256];
+    DBUG_PRINT("info", ("Wait for ok from client"));
+    if (s_input.gets(buf, sizeof(buf)) == nullptr) {
+      DEBUG_FPRINTF((stderr,
+                     "(%u)connect_server_impl failed LINE:%d,"
+                     " to remote node %d\n",
+                     localNodeId, __LINE__, remoteNodeId));
+      setupBuffersUndone();
+      detach_shm(false);
+      break;
+    }
+
+    if (sscanf(buf, "shm client 1 ok: %d", &m_remote_pid) != 1) {
+      DEBUG_FPRINTF((stderr,
+                     "(%u)connect_server_impl failed LINE:%d,"
+                     " to remote node %d\n",
+                     localNodeId, __LINE__, remoteNodeId));
+      setupBuffersUndone();
+      detach_shm(false);
+      break;
+    }
+
+    DEBUG_FPRINTF(
+        (stderr, "(%u)connect_common()(%u)\n", localNodeId, remoteNodeId));
+    if (!connect_common()) {
+      DEBUG_FPRINTF((stderr,
+                     "(%u)connect_server_impl failed LINE:%d,"
+                     " to remote node %d\n",
+                     localNodeId, __LINE__, remoteNodeId));
+      setupBuffersUndone();
+      detach_shm(false);
+      break;
+    }
 
-  if (r) {
     // Send ok to client
     s_output.println("shm server 2 ok");
     // Wait for ok from client
@@ -342,15 +357,29 @@ bool SHM_Transporter::connect_server_impl(NdbSocket &&sockfd) {
                      "(%u)connect_server_impl failed LINE:%d,"
                      " to remote node %d\n",
                      localNodeId, __LINE__, remoteNodeId));
+      setupBuffersUndone();
       detach_shm(false);
-      DBUG_RETURN(false);
+      break;
     }
+
     DBUG_PRINT("info",
                ("Successfully connected server to node %d", remoteNodeId));
-  }
-  DEBUG_FPRINTF((stderr, "(%u)set_socket()(%u)\n", localNodeId, remoteNodeId));
-  set_socket(std::move(sockfd));
-  DBUG_RETURN(r);
+
+    DEBUG_FPRINTF(
+        (stderr, "(%u)set_socket()(%u)\n", localNodeId, remoteNodeId));
+    set_socket(std::move(sockfd));
+    DBUG_RETURN(true);
+  } while (false);
+
+  /* Error occurred */
+
+  /* Existing error handling should have cleaned up for retry */
+  assert(!setupBuffersDone);
+  assert(!_attached);
+  assert(!_shmSegCreated);
+
+  sockfd.close();
+  DBUG_RETURN(false);
 }
 
 void SHM_Transporter::set_socket(NdbSocket &&sock) {
@@ -369,78 +398,91 @@ bool SHM_Transporter::connect_client_impl(NdbSocket &&sockfd) {
   SocketOutputStream s_output(sockfd);
   char buf[256];
 
-  // Wait for server to create and attach
-  DBUG_PRINT("info", ("Wait for server to create and attach"));
-  if (s_input.gets(buf, 256) == nullptr) {
-    DEBUG_FPRINTF((stderr,
-                   "(%u)connect_client_impl failed LINE:%d,"
-                   " to remote node %d\n",
-                   localNodeId, __LINE__, remoteNodeId));
-
-    DBUG_PRINT("error", ("Server id %d did not attach", remoteNodeId));
-    DBUG_RETURN(false);
-  }
-
-  if (sscanf(buf, "shm server 1 ok: %d", &m_remote_pid) != 1) {
-    DEBUG_FPRINTF((stderr,
-                   "(%u)connect_client_impl failed LINE:%d,"
-                   " to remote node %d\n",
-                   localNodeId, __LINE__, remoteNodeId));
-    DBUG_RETURN(false);
-  }
-
-  // Create
-  if (!_shmSegCreated) {
-    if (!ndb_shm_get()) {
-      DEBUG_FPRINTF((stderr,
-                     "(%u)connect_client_impl failed LINE:%d,"
-                     " to remote node %d\n",
-                     localNodeId, __LINE__, remoteNodeId));
-      DBUG_PRINT("error",
-                 ("Failed create of shm seg to node %d", remoteNodeId));
-      DBUG_RETURN(false);
-    }
-    _shmSegCreated = true;
-    DEBUG_FPRINTF((stderr, "(%u)ndb_shm_get(%u)\n", localNodeId, remoteNodeId));
-  }
-
-  // Attach
-  if (!_attached) {
-    if (!ndb_shm_attach()) {
-      DEBUG_FPRINTF((stderr,
-                     "(%u)connect_client_impl failed LINE:%d,"
-                     " to remote node %d\n",
-                     localNodeId, __LINE__, remoteNodeId));
-      DBUG_PRINT("error",
-                 ("Failed attach of shm seg to node %d", remoteNodeId));
-      DBUG_RETURN(false);
-    }
-    _attached = true;
-    DEBUG_FPRINTF(
-        (stderr, "(%u)ndb_shm_attach(%u)\n", localNodeId, remoteNodeId));
-  }
-
-  require(!setupBuffersDone);
+  do  // Close socket on errors
   {
-    DEBUG_FPRINTF((stderr, "(%u)setupBuffers(%u) Line:%d\n", localNodeId,
-                   remoteNodeId, __LINE__));
-    if (setupBuffers()) {
-      g_eventLogger->info("Shared memory not supported on this platform");
-      detach_shm(false);
-      DBUG_RETURN(false);
-    } else {
+    // Wait for server to create and attach
+    DBUG_PRINT("info", ("Wait for server to create and attach"));
+    if (s_input.gets(buf, 256) == nullptr) {
+      DEBUG_FPRINTF((stderr,
+                     "(%u)connect_client_impl failed LINE:%d,"
+                     " to remote node %d\n",
+                     localNodeId, __LINE__, remoteNodeId));
+
+      DBUG_PRINT("error", ("Server id %d did not attach", remoteNodeId));
+      break;
+    }
+
+    if (sscanf(buf, "shm server 1 ok: %d", &m_remote_pid) != 1) {
+      DEBUG_FPRINTF((stderr,
+                     "(%u)connect_client_impl failed LINE:%d,"
+                     " to remote node %d\n",
+                     localNodeId, __LINE__, remoteNodeId));
+      break;
+    }
+
+    // Create
+    if (!_shmSegCreated) {
+      if (!ndb_shm_get()) {
+        DEBUG_FPRINTF((stderr,
+                       "(%u)connect_client_impl failed LINE:%d,"
+                       " to remote node %d\n",
+                       localNodeId, __LINE__, remoteNodeId));
+        DBUG_PRINT("error",
+                   ("Failed create of shm seg to node %d", remoteNodeId));
+        break;
+      }
+      _shmSegCreated = true;
+      DEBUG_FPRINTF(
+          (stderr, "(%u)ndb_shm_get(%u)\n", localNodeId, remoteNodeId));
+    }
+
+    // Attach
+    if (!_attached) {
+      if (!ndb_shm_attach()) {
+        DEBUG_FPRINTF((stderr,
+                       "(%u)connect_client_impl failed LINE:%d,"
+                       " to remote node %d\n",
+                       localNodeId, __LINE__, remoteNodeId));
+        DBUG_PRINT("error",
+                   ("Failed attach of shm seg to node %d", remoteNodeId));
+        break;
+      }
+      _attached = true;
+      DEBUG_FPRINTF(
+          (stderr, "(%u)ndb_shm_attach(%u)\n", localNodeId, remoteNodeId));
+    }
+
+    require(!setupBuffersDone);
+    {
+      DEBUG_FPRINTF((stderr, "(%u)setupBuffers(%u) Line:%d\n", localNodeId,
+                     remoteNodeId, __LINE__));
+      if (setupBuffers()) {
+        g_eventLogger->info(
+            "Node %u transporter to node %u: SHM not supported on this "
+            "platform; disconnecting",
+            localNodeId, remoteNodeId);
+        detach_shm(false);
+        break;
+      }
       setupBuffersDone = true;
     }
-  }
 
-  // Send ok to server
-  s_output.println("shm client 1 ok: %d", m_transporter_registry.m_shm_own_pid);
+    // Send ok to server
+    s_output.println("shm client 1 ok: %d",
+                     m_transporter_registry.m_shm_own_pid);
 
-  DEBUG_FPRINTF(
-      (stderr, "(%u)connect_common(%u)\n", localNodeId, remoteNodeId));
-  int r = connect_common();
+    DEBUG_FPRINTF(
+        (stderr, "(%u)connect_common(%u)\n", localNodeId, remoteNodeId));
+    if (!connect_common()) {
+      DEBUG_FPRINTF((stderr,
+                     "(%u)connect_client_impl failed LINE:%d,"
+                     " to remote node %d\n",
+                     localNodeId, __LINE__, remoteNodeId));
+      setupBuffersUndone();
+      detach_shm(false);
+      break;
+    }
 
-  if (r) {
     // Wait for ok from server
     DBUG_PRINT("info", ("Wait for ok from server"));
     if (s_input.gets(buf, 256) == nullptr) {
@@ -449,23 +491,29 @@ bool SHM_Transporter::connect_client_impl(NdbSocket &&sockfd) {
                      " to remote node %d\n",
                      localNodeId, __LINE__, remoteNodeId));
       DBUG_PRINT("error", ("No ok from server node %d", remoteNodeId));
+      setupBuffersUndone();
       detach_shm(false);
-      DBUG_RETURN(false);
+      break;
     }
     // Send ok to server
     s_output.println("shm client 2 ok");
     DBUG_PRINT("info",
                ("Successfully connected client to node %d", remoteNodeId));
-  } else {
-    DEBUG_FPRINTF((stderr,
-                   "(%u)connect_client_impl failed LINE:%d,"
-                   " to remote node %d\n",
-                   localNodeId, __LINE__, remoteNodeId));
-    detach_shm(false);
-  }
-  set_socket(std::move(sockfd));
-  DEBUG_FPRINTF((stderr, "(%u)set_socket(%u)\n", localNodeId, remoteNodeId));
-  DBUG_RETURN(r);
+
+    set_socket(std::move(sockfd));
+    DEBUG_FPRINTF((stderr, "(%u)set_socket(%u)\n", localNodeId, remoteNodeId));
+    DBUG_RETURN(true);
+  } while (false);
+
+  /* Error occurred */
+
+  /* Existing error handling should have cleaned up for retry */
+  assert(!setupBuffersDone);
+  assert(!_attached);
+  assert(!_shmSegCreated);
+
+  sockfd.close();
+  DBUG_RETURN(false);
 }
 
 bool SHM_Transporter::connect_common() {
@@ -539,6 +587,10 @@ void SHM_Transporter::wakeup() {
   if (awake_state) {
     return;
   }
+  if (!theSocket.is_valid()) {
+    /* Wakeup socket not yet ready */
+    return;
+  }
   iov[0].iov_len = 1;
   iov[0].iov_base = &buf[0];
   buf[0] = 0;
@@ -734,21 +786,17 @@ bool SHM_Transporter::handle_reverse_awake_state() {
     if (isServer) {
       if (*clientStatusFlag == 1 || *clientAwakenedFlag == 1) {
         return true;
-      } else {
-        *clientAwakenedFlag = 1;
-        return false;
-      }
-    } else {
-      if (*serverStatusFlag == 1 || *serverAwakenedFlag == 1) {
-        return true;
-      } else {
-        *serverAwakenedFlag = 1;
-        return false;
       }
+      *clientAwakenedFlag = 1;
+      return false;
     }
-  } else {
-    return true;
+    if (*serverStatusFlag == 1 || *serverAwakenedFlag == 1) {
+      return true;
+    }
+    *serverAwakenedFlag = 1;
+    return false;
   }
+  return true;
 }
 
 void SHM_Transporter::updateReceivePtr(TransporterReceiveHandle &recvdata,
@@ -799,9 +847,9 @@ bool SHM_Transporter::send_is_possible(int timeout_millisec) const {
       }
       DEBUG_FPRINTF((stderr, "send_is_possible, timed out\n"));
       return false;
-    } else {
-      break;
     }
-  } while (1);
+    break;
+
+  } while (true);
   return true;
 }
