@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -48,7 +48,7 @@
 #define DEBUG_FPRINTF(a)
 #endif
 
-//#define DEBUG_MULTI_TRP 1
+// #define DEBUG_MULTI_TRP 1
 
 #ifdef DEBUG_MULTI_TRP
 #define DEB_MULTI_TRP(arglist)   \
@@ -61,6 +61,28 @@
   } while (0)
 #endif
 
+namespace {
+/**
+ * @brief Return human-readable transporter type name.
+ *
+ * Maps TransporterType enum to a short string used in logs and diagnostics.
+ *
+ * @param t transporter type.
+ * @return transporter type name.
+ */
+static inline const char *transporter_type_name(TransporterType t) {
+  const int v = static_cast<int>(t);
+  switch (v) {
+    case tt_TCP_TRANSPORTER:
+      return "TCP";
+    case tt_SHM_TRANSPORTER:
+      return "SHM";
+    default:
+      return "UNKNOWN";
+  }
+}
+}  // namespace
+
 Transporter::Transporter(TransporterRegistry &t_reg, TrpId transporter_index,
                          TransporterType _type, const char *lHostName,
                          const char *rHostName, int s_port,
@@ -83,9 +105,15 @@ Transporter::Transporter(TransporterRegistry &t_reg, TrpId transporter_index,
       m_connect_count(0),
       m_overload_count(0),
       m_slowdown_count(0),
+      m_send_buffer_alloc_bytes(0),
+      m_send_buffer_max_alloc_bytes(0),
+      m_send_buffer_used_bytes(0),
+      m_send_buffer_max_used_bytes(0),
       isMgmConnection(_isMgmConnection),
       m_connected(false),
       m_type(_type),
+      m_require_tls(false),
+      m_encrypted(false),
       reportFreq(4096),
       receiveCount(0),
       receiveSize(0),
@@ -110,9 +138,9 @@ Transporter::Transporter(TransporterRegistry &t_reg, TrpId transporter_index,
   } else {
     if (!isServer) {
       g_eventLogger->info(
-          "Unable to setup transporter. Node %u must have hostname."
-          " Update configuration.",
-          rNodeId);
+          "Node %u transporter to node %u: missing hostname. "
+          "Update configuration.",
+          localNodeId, rNodeId);
       exit(-1);
     }
     remoteHostName[0] = 0;
@@ -154,6 +182,14 @@ Transporter::Transporter(TransporterRegistry &t_reg, TrpId transporter_index,
   DBUG_VOID_RETURN;
 }
 
+void Transporter::use_tls_client_auth() {
+  delete m_socket_client;
+  auto *authTls =
+      new SocketAuthTls(&m_transporter_registry.m_tls_keys, m_require_tls);
+  m_socket_client = new SocketClient(authTls);
+  m_socket_client->set_connect_timeout(m_timeOutMillis);
+}
+
 Transporter::~Transporter() { delete m_socket_client; }
 
 bool Transporter::start_disconnecting(int err, bool send_source) {
@@ -165,6 +201,7 @@ bool Transporter::start_disconnecting(int err, bool send_source) {
 
 bool Transporter::configure(const TransporterConfiguration *conf) {
   if (configure_derived(conf) && conf->s_port == m_s_port &&
+      conf->requireTls == m_require_tls &&
       strcmp(conf->remoteHostName, remoteHostName) == 0 &&
       strcmp(conf->localHostName, localHostName) == 0 &&
       conf->remoteNodeId == remoteNodeId && conf->localNodeId == localNodeId &&
@@ -214,6 +251,10 @@ bool Transporter::connect_server(NdbSocket &&sockfd, BaseString &msg) {
   DBUG_RETURN(true);
 }
 
+inline static void tls_error(int code) {
+  g_eventLogger->error("TLS error %d '%s'", code, TlsKeyError::message(code));
+}
+
 bool Transporter::connect_client_mgm(int port) {
   require(!isPartOfMultiTransporter());
   NdbSocket secureSocket =
@@ -239,7 +280,7 @@ bool Transporter::connect_client() {
   }
 
   if (isMgmConnection) {
-    return connect_client_mgm(port);
+    DBUG_RETURN(connect_client_mgm(port));
   } else {
     ndb_sockaddr local;
     if (strlen(localHostName) > 0) {
@@ -283,11 +324,44 @@ bool Transporter::connect_client() {
     }
 
     /** Socket Authentication */
-    if (m_socket_client->authenticate(secureSocket) <
-        SocketAuthSimple::AuthOk) {
+    int auth = m_socket_client->authenticate(secureSocket);
+    g_eventLogger->debug("Transporter client auth result: %d [%s]", auth,
+                         SocketAuthenticator::error(auth));
+    if (auth < SocketAuthenticator::AuthOk) {
       secureSocket.close();
       DBUG_RETURN(false);
     }
+
+    if (auth == SocketAuthTls::negotiate_tls_ok)  // Initiate TLS
+    {
+      struct ssl_ctx_st *ctx = m_transporter_registry.m_tls_keys.ctx();
+      struct ssl_st *ssl = NdbSocket::get_client_ssl(ctx);
+      if (ssl == nullptr) {
+        tls_error(TlsKeyError::no_local_cert);
+        secureSocket.close();
+        DBUG_RETURN(false);
+      }
+      if (!secureSocket.associate(ssl)) {
+        tls_error(TlsKeyError::openssl_error);
+        NdbSocket::free_ssl(ssl);
+        secureSocket.close();
+        DBUG_RETURN(false);
+      }
+      if (!secureSocket.do_tls_handshake()) {
+        tls_error(TlsKeyError::authentication_failure);
+        // secureSocket closed by do_tls_handshake
+        DBUG_RETURN(false);
+      }
+
+      /* Certificate Authorization */
+      auth =
+          TlsKeyManager::check_server_host_auth(secureSocket, remoteHostName);
+      if (auth) {
+        tls_error(auth);
+        secureSocket.close();
+        DBUG_RETURN(false);
+      }
+    }
   }
 
   DBUG_RETURN(connect_client(std::move(secureSocket)));
@@ -349,8 +423,9 @@ bool Transporter::connect_client(NdbSocket &&socket) {
   const int OldMaxHandshakeBytesLimit = 23; /* 24 - 1 for \n */
   if (unlikely(helloLen > OldMaxHandshakeBytesLimit)) {
     /* Cannot send this many bytes to older versions */
-    g_eventLogger->info("Failed handshake string length %u : \"%s\"", helloLen,
-                        helloBuf);
+    g_eventLogger->info(
+        "Node %u transporter to node %u: failed handshake length %u: \"%s\"",
+        localNodeId, remoteNodeId, helloLen, helloBuf);
     abort();
   }
 
@@ -391,8 +466,8 @@ bool Transporter::connect_client(NdbSocket &&socket) {
 
   // Check nodeid
   if (nodeId != remoteNodeId) {
-    g_eventLogger->error("Connected to wrong nodeid: %d, expected: %d", nodeId,
-                         remoteNodeId);
+    g_eventLogger->error("Node %u connected to wrong nodeid: %d, expected: %d",
+                         localNodeId, nodeId, remoteNodeId);
     socket.close();
     DBUG_RETURN(false);
   }
@@ -400,9 +475,13 @@ bool Transporter::connect_client(NdbSocket &&socket) {
   // Check transporter type
   if (remote_transporter_type != -1 && remote_transporter_type != m_type) {
     g_eventLogger->error(
-        "Connection to node: %d uses different transporter "
-        "type: %d, expected type: %d",
-        nodeId, remote_transporter_type, m_type);
+        "Node %u connection to node %d uses different transporter "
+        "type: %s (%d), expected type: %s (%d)",
+        localNodeId, nodeId,
+        transporter_type_name(
+            static_cast<TransporterType>(remote_transporter_type)),
+        remote_transporter_type, transporter_type_name(m_type),
+        static_cast<int>(m_type));
     socket.close();
     DBUG_RETURN(false);
   }
@@ -439,11 +518,7 @@ void Transporter::disconnectImpl() {
   assert(theSocket.is_valid());
   if (theSocket.is_valid()) {
     DEB_MULTI_TRP(("Shutdown socket for trp %u", getTransporterIndex()));
-    if (theSocket.shutdown() < 0) {
-      // Do we care about shutdown failures? It might fail due to e.g.
-      // connection already terminated by other peer.
-      report_error(TE_ERROR_CLOSING_SOCKET);
-    }
+    theSocket.shutdown();
   }
 }
 
@@ -463,6 +538,10 @@ void Transporter::resetCounters() {
   m_bytes_received = 0;
   m_overload_count = 0;
   m_slowdown_count = 0;
+  m_send_buffer_alloc_bytes = 0;
+  m_send_buffer_max_alloc_bytes = 0;
+  m_send_buffer_used_bytes = 0;
+  m_send_buffer_max_used_bytes = 0;
 }
 
 void Transporter::checksum_state::dumpBadChecksumInfo(
@@ -480,9 +559,9 @@ void Transporter::checksum_state::dumpBadChecksumInfo(
    */
   {
     Uint32 pos = 0;
-    Uint32 buf_remain = Uint32(len);
+    auto buf_remain = Uint32(len);
     const char *data = (const char *)buf;
-    const Uint32 firstWordBytes = Uint32((offset + sig_remaining) & 3);
+    const auto firstWordBytes = Uint32((offset + sig_remaining) & 3);
     if (firstWordBytes && (buf_remain >= firstWordBytes)) {
       /* Partial first word */
       Uint32 word = 0;
