@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2017, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -293,7 +294,7 @@ size_t LogBuffer::get(char *buf, size_t buf_size, uint timeout_ms) {
   }
 
   // Wait until there's something in the buffer or until timeout
-  while ((m_size == 0) && (cond_ret == 0) && (m_stop == false)) {
+  while ((m_size == 0) && (cond_ret == 0) && !m_stop) {
     /**
      * Log buffer is empty, block until signal is received
      * or until timeout.
@@ -307,7 +308,8 @@ size_t LogBuffer::get(char *buf, size_t buf_size, uint timeout_ms) {
   if (m_size == 0) {
     // log buffer empty even after timeout, return
     return 0;
-  } else if (m_size < buf_size) {
+  }
+  if (m_size < buf_size) {
     // change num of bytes to be copied to available bytes
     size = m_size;
   }
