@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2022, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2022, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,10 +22,10 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
-#include <assert.h>
-#include <stdlib.h>
 #include <sys/stat.h>
-#include <time.h>
+#include <cassert>
+#include <cstdlib>
+#include <ctime>
 #include <memory>
 
 #include <openssl/err.h>
@@ -41,12 +41,14 @@
 
 #include "debugger/EventLogger.hpp"
 #include "portlib/ndb_localtime.h"
-#include "util/File.hpp"
+#include "util/File.hpp"  // S_IRUSR
 #include "util/ndb_openssl3_compat.h"
 #include "util/require.h"
 
 #include "util/NodeCertificate.hpp"
 
+#include "my_ssl_algo_cache.h"
+
 #ifdef _WIN32
 #include <direct.h>
 #define gmtime_r(A, B) (gmtime_s(B, A) == 0)
@@ -86,9 +88,34 @@ static void handle_pem_error(const char fn_name[]) {
  */
 
 bool PkiFile::remove(const PathName &path) {
-  return File_class::remove(path.c_str());
+  return PkiFile::remove(path.c_str());
 }
 
+#ifdef _WIN32
+bool PkiFile::remove(const char *name) {
+  bool mustSetWriteAccess = false;
+  if (_access(name, 06) != 0) {
+    if (errno == ENOENT) return false;
+    mustSetWriteAccess = (errno == EACCES);
+  }
+  if (mustSetWriteAccess)
+    if (_chmod(name, _S_IREAD | _S_IWRITE) != 0) return false;
+
+  if (::remove(name) == 0) return true;  // success
+
+  /* Unusual error: access() has succeeded but remove() has failed .*/
+  g_eventLogger->error("NDB TLS Error %d removing file %s", errno, name);
+
+  if (mustSetWriteAccess) _chmod(name, _S_IREAD);
+  return false;
+}
+
+#else
+
+bool PkiFile::remove(const char *name) { return (::remove(name) == 0); }
+
+#endif
+
 int PkiFile::assign(PathName &path, const char *dir, const char *file) {
   path.clear();
   if (dir == nullptr || dir[0] == '\0')
@@ -292,12 +319,12 @@ short PkiFilenames::find_file(const TlsSearchPath *path,
 
 static bool promote_file(const char *pending, const char *active,
                          const char *retired) {
+  PkiFile::remove(retired);  // this may fail if retired does not exist
 #ifdef _WIN32
   rename(active, retired);  // this may fail if active does not exist
 #else
-  File_class::remove(retired);  // this may fail if retired does not exist
-  if (link(active, retired)) {
-  }  // this may fail if active does not exist
+  std::ignore =
+      link(active, retired);  // this may fail if active does not exist
 #endif
   return (rename(pending, active) == 0);
 }
@@ -333,12 +360,11 @@ bool PrivateKey::store(EVP_PKEY *key, const PkiFile::PathName &path,
                                 passphrase)) {
     fclose(fp);
     return true;
-  } else {
-    handle_pem_error("PEM_write_PKCS8PrivateKey");
-    fclose(fp);
-    PkiFile::remove(path);
-    return false;
   }
+  handle_pem_error("PEM_write_PKCS8PrivateKey");
+  fclose(fp);
+  PkiFile::remove(path);
+  return false;
 }
 
 bool PrivateKey::store(EVP_PKEY *key, const char *dir, const char *file,
@@ -387,7 +413,6 @@ bool PendingPrivateKey::promote(const PkiFile::PathName &pending_file) {
   retired.append(base);
   retired.append(suffix3);
   if (retired.is_truncated()) return false;
-
   return promote_file(pending_file.c_str(), active.c_str(), retired.c_str());
 }
 
@@ -466,7 +491,7 @@ int SigningRequest::finalise(EVP_PKEY *key) {
   }
 
   /* Sign the CSR with the private key */
-  if (X509_REQ_sign(m_req, key, EVP_sha256()) == 0) return -40;
+  if (X509_REQ_sign(m_req, key, my_EVP_sha256()) == 0) return -40;
 
   m_key = key;
   return 0;
@@ -511,11 +536,10 @@ bool SigningRequest::store(const char *dir) const {
   if (write(fp)) {
     fclose(fp);
     return true;
-  } else {
-    fclose(fp);
-    File_class::remove(path);
-    return false;
   }
+  fclose(fp);
+  PkiFile::remove(path);
+  return false;
 }
 
 bool SigningRequest::write(FILE *fp) const {
@@ -651,11 +675,10 @@ bool Certificate::store(STACK_OF(X509) * certs, const PkiFile::PathName &path) {
   if (Certificate::write(certs, fp)) {
     fclose(fp);
     return true;
-  } else {
-    fclose(fp);
-    PkiFile::remove(path);
-    return false;
   }
+  fclose(fp);
+  PkiFile::remove(path);
+  return false;
 }
 
 bool Certificate::store(STACK_OF(X509) * certs, const char *dir,
@@ -780,7 +803,7 @@ X509 *ClusterCertAuthority::create(EVP_PKEY *key, const CertLifetime &lifetime,
 
 int ClusterCertAuthority::sign(X509 *issuer, EVP_PKEY *key, X509 *cert) {
   if (X509_set_issuer_name(cert, X509_get_subject_name(issuer)) == 0) return 0;
-  return X509_sign(cert, key, EVP_sha256());
+  return X509_sign(cert, key, my_EVP_sha256());
 }
 
 /*
@@ -876,7 +899,7 @@ int CertSubject::bound_hostname(int n, char *buffer, int size) const {
     int name_type;
     if (n < sk_GENERAL_NAME_num(m_bound_hostnames)) {
       GENERAL_NAME *name = sk_GENERAL_NAME_value(m_bound_hostnames, n);
-      ASN1_STRING *str =
+      auto *str =
           static_cast<ASN1_STRING *>(GENERAL_NAME_get0_value(name, &name_type));
       if (name_type == GEN_DNS) {
         if (str->length < size) size = str->length;
@@ -898,7 +921,7 @@ bool CertSubject::bound_localhost() const {
   if (sk_GENERAL_NAME_num(m_bound_hostnames) == 1) {
     int name_type;
     GENERAL_NAME *name = sk_GENERAL_NAME_value(m_bound_hostnames, 0);
-    ASN1_STRING *str =
+    auto *str =
         static_cast<ASN1_STRING *>(GENERAL_NAME_get0_value(name, &name_type));
     if (name_type == GEN_DNS) {
       if ((str->length == 9) &&
@@ -1145,13 +1168,13 @@ void NodeCertificate::init_from_credentials(STACK_OF(X509) * certs,
 
 const NodeCertificate *NodeCertificate::from_credentials(STACK_OF(X509) * certs,
                                                          EVP_PKEY *key) {
-  NodeCertificate *nc = new NodeCertificate();
+  auto *nc = new NodeCertificate();
   nc->init_from_credentials(certs, key);
   return nc;
 }
 
 const NodeCertificate *NodeCertificate::for_peer(X509 *cert) {
-  NodeCertificate *nc = new NodeCertificate();
+  auto *nc = new NodeCertificate();
   nc->init_from_x509(cert);
   return nc;
 }
@@ -1243,7 +1266,7 @@ int NodeCertificate::finalise(X509 *CA_cert, EVP_PKEY *CA_key) {
 
   /* Sign the certificate */
   if (CA_key) {
-    if (!X509_sign(m_x509, CA_key, EVP_sha256())) return -40;
+    if (!X509_sign(m_x509, CA_key, my_EVP_sha256())) return -40;
     m_signed = true;
   }
 
@@ -1358,7 +1381,7 @@ bool NodeCertificate::parse_name(const char *name) {
 #include <openssl/applink.c>
 static constexpr bool isWin32 = 1;
 #else
-static constexpr bool isWin32 = 0;
+static constexpr bool isWin32 = false;
 #endif
 
 static constexpr bool openssl_version_ok =
@@ -1463,7 +1486,7 @@ static int file_subtest_csr(bool output) {
   require(r);
 
   if (output) {
-    NodeCertificate *nc = new NodeCertificate(*csr, key);
+    auto *nc = new NodeCertificate(*csr, key);
     r1 = nc->self_sign();
     require(!r1);
     Certificate::write(nc->all_certs(), stdout);
