@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2010, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -91,7 +91,7 @@ class trp_client : TransporterSendBufferHandle {
   void unlock();
   /* Interface used by Multiple NDB waiter code */
   void lock_client();
-  bool check_if_locked(void) const;
+  bool check_if_locked() const;
 
   NodeId getOwnNodeId() const;
 
