@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -230,9 +230,6 @@
 */
 /* clang-format on */
 
-struct PSI_POS;
-struct PSI_RECORD;
-
 bool plugin_table_service_initialized = false;
 
 /**
@@ -1234,14 +1231,6 @@ void get_field_year_v1(PSI_field *f, PSI_year *value) {
   value->is_null = false;
 }
 
-/**************************************
- * NULL                               *
- **************************************/
-void set_field_null_v1(PSI_field *f) {
-  auto *f_ptr = reinterpret_cast<Field *>(f);
-  f_ptr->set_null();
-}
-
 unsigned int get_parts_found_v1(PSI_key_reader *reader) {
   const auto *pfs_reader = (PFS_key_reader *)reader;
   return pfs_reader->m_parts_found;
