@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -335,14 +335,30 @@ ulonglong get_field_set(Field *f) {
 /* DATE TYPE */
 void set_field_date(Field *f, const char *value, uint len) {
   assert(f->real_type() == MYSQL_TYPE_NEWDATE);
-  auto *f2 = (Field_newdate *)f;
+  auto *f2 = down_cast<Field_date *>(f);
   f2->store(value, len, system_charset_info);
 }
 
+void set_field_date_by_daynr(Field *f, ulong daynr) {
+  assert(f->real_type() == MYSQL_TYPE_NEWDATE);
+  auto *f2 = down_cast<Field_date *>(f);
+
+  MYSQL_TIME date;
+  get_date_from_daynr(daynr, &date.year, &date.month, &date.day);
+  date.hour = 0;
+  date.minute = 0;
+  date.second = 0;
+  date.second_part = 0;
+  date.neg = false;
+  date.time_type = MYSQL_TIMESTAMP_DATE;
+  date.time_zone_displacement = 0;
+  f2->store_time(&date, 0);
+}
+
 char *get_field_date(Field *f, char *val, uint *len) {
   assert(f->real_type() == MYSQL_TYPE_NEWDATE);
   String temp;
-  const auto *f2 = (Field_newdate *)f;
+  const auto *f2 = down_cast<Field_date *>(f);
   f2->val_str(&temp, nullptr);
   *len = temp.length();
   strncpy(val, temp.ptr(), *len);
@@ -352,14 +368,14 @@ char *get_field_date(Field *f, char *val, uint *len) {
 /* TIME TYPE */
 void set_field_time(Field *f, const char *value, uint len) {
   assert(f->real_type() == MYSQL_TYPE_TIME2);
-  auto *f2 = (Field_timef *)f;
+  auto *f2 = down_cast<Field_time *>(f);
   f2->store(value, len, system_charset_info);
 }
 
 char *get_field_time(Field *f, char *val, uint *len) {
   assert(f->real_type() == MYSQL_TYPE_TIME2);
   String temp;
-  const auto *f2 = (Field_timef *)f;
+  const auto *f2 = down_cast<Field_time *>(f);
   f2->val_str(&temp, nullptr);
   *len = temp.length();
   strncpy(val, temp.ptr(), *len);
@@ -369,14 +385,14 @@ char *get_field_time(Field *f, char *val, uint *len) {
 /* DATETIME TYPE */
 void set_field_datetime(Field *f, const char *value, uint len) {
   assert(f->real_type() == MYSQL_TYPE_DATETIME2);
-  auto *f2 = (Field_datetimef *)f;
+  auto *f2 = (Field_datetime *)f;
   f2->store(value, len, system_charset_info);
 }
 
 char *get_field_datetime(Field *f, char *val, uint *len) {
   assert(f->real_type() == MYSQL_TYPE_DATETIME2);
   String temp;
-  const auto *f2 = (Field_datetimef *)f;
+  const auto *f2 = (Field_datetime *)f;
   f2->val_str(&temp, nullptr);
   *len = temp.length();
   strncpy(val, temp.ptr(), *len);
@@ -386,14 +402,14 @@ char *get_field_datetime(Field *f, char *val, uint *len) {
 /* TIMESTAMP TYPE */
 void set_field_timestamp(Field *f, const char *value, uint len) {
   assert(f->real_type() == MYSQL_TYPE_TIMESTAMP2);
-  auto *f2 = (Field_timestampf *)f;
+  auto *f2 = (Field_timestamp *)f;
   f2->store(value, len, system_charset_info);
 }
 
 char *get_field_timestamp(Field *f, char *val, uint *len) {
   assert(f->real_type() == MYSQL_TYPE_TIMESTAMP2);
   String temp;
-  const auto *f2 = (Field_timestampf *)f;
+  const auto *f2 = (Field_timestamp *)f;
   f2->val_str(&temp, nullptr);
   *len = temp.length();
   strncpy(val, temp.ptr(), *len);
@@ -404,7 +420,7 @@ void set_field_timestamp(Field *f, ulonglong value) {
   const my_timeval tm = {static_cast<int64_t>(value / 1000000),
                          static_cast<int64_t>(value % 1000000)};
   assert(f->real_type() == MYSQL_TYPE_TIMESTAMP2);
-  auto *f2 = (Field_timestampf *)f;
+  auto *f2 = (Field_timestamp *)f;
   f2->store_timestamp(&tm);
 }
 
@@ -736,7 +752,7 @@ int PFS_object_row::make_row(PFS_program *pfs) {
 }
 
 int PFS_column_row::make_row(const MDL_key *mdl) {
-  static_assert(MDL_key::NAMESPACE_END == 18,
+  static_assert(MDL_key::NAMESPACE_END == 19,
                 "Adjust performance schema when changing enum_mdl_namespace");
 
   bool with_schema = false;
@@ -774,6 +790,11 @@ int PFS_column_row::make_row(const MDL_key *mdl) {
       with_schema = true;
       with_object = true;
       break;
+    case MDL_key::LIBRARY:
+      m_object_type = OBJECT_TYPE_LIBRARY;
+      with_schema = true;
+      with_object = true;
+      break;
     case MDL_key::TRIGGER:
       m_object_type = OBJECT_TYPE_TRIGGER;
       with_schema = true;
@@ -2110,6 +2131,11 @@ bool PFS_key_user::match(const PFS_setup_actor *pfs) {
                   pfs->m_key.m_user_name.length());
 }
 
+bool PFS_key_user::match(const PFS_user_name *pfs) {
+  const bool record_null = (pfs->length() == 0);
+  return do_match(record_null, pfs->ptr(), pfs->length());
+}
+
 bool PFS_key_host::match(const PFS_thread *pfs) {
   const bool record_null = (pfs->m_host_name.length() == 0);
   return do_match(record_null, pfs->m_host_name.ptr(),
@@ -2134,6 +2160,11 @@ bool PFS_key_host::match(const PFS_setup_actor *pfs) {
                   pfs->m_key.m_host_name.length());
 }
 
+bool PFS_key_host::match(const PFS_host_name *pfs) {
+  const bool record_null = (pfs->length() == 0);
+  return do_match(record_null, pfs->ptr(), pfs->length());
+}
+
 bool PFS_key_host::match(const char *hostname, size_t hostname_length) {
   const bool record_null = (hostname_length == 0);
   return do_match(record_null, hostname, hostname_length);
@@ -2198,7 +2229,7 @@ bool PFS_key_group_name::match(PFS_thread *pfs) {
 }
 
 bool PFS_key_variable_name::match(const System_variable *pfs) {
-  return do_match(false, pfs->m_name, pfs->m_name_length);
+  return do_match(false, pfs->m_name_str, pfs->m_name_length);
 }
 
 bool PFS_key_variable_name::match(const Status_variable *pfs) {
