@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -35,6 +35,7 @@
 #include "storage/perfschema/unittest/stub_pfs_global.h"
 #include "storage/perfschema/unittest/stub_pfs_plugin_table.h"
 #include "storage/perfschema/unittest/stub_pfs_tls_channel.h"
+#include "storage/perfschema/unittest/stub_server_logs.h"
 #include "storage/perfschema/unittest/stub_server_telemetry.h"
 #include "storage/perfschema/unittest/stub_telemetry_metrics.h"
 #include "unittest/mytap/tap.h"
@@ -61,6 +62,7 @@ static void test_oom() {
   PSI_system_bootstrap *system_boot;
   PSI_tls_channel_bootstrap *tls_channel_boot;
   PSI_metric_bootstrap *metric_boot;
+  PSI_logs_client_bootstrap *logs_client_boot;
 
   memset(&param, 0xFF, sizeof(param));
   param.m_enabled = true;
@@ -101,6 +103,7 @@ static void test_oom() {
   param.m_memory_class_sizing = 10;
   param.m_meter_class_sizing = 5;
   param.m_metric_class_sizing = 10;
+  param.m_logger_class_sizing = 10;
   param.m_metadata_lock_sizing = 0;
   param.m_max_digest_length = 0;
   param.m_max_sql_text_length = 0;
@@ -132,7 +135,8 @@ static void test_oom() {
       &param, &thread_boot, &mutex_boot, &rwlock_boot, &cond_boot, &file_boot,
       &socket_boot, &table_boot, &mdl_boot, &idle_boot, &stage_boot,
       &statement_boot, &transaction_boot, &memory_boot, &error_boot,
-      &data_lock_boot, &system_boot, &tls_channel_boot, &metric_boot);
+      &data_lock_boot, &system_boot, &tls_channel_boot, &metric_boot,
+      &logs_client_boot);
   ok(rc == 0, "init ok");
   thread_service = (PSI_thread_service_t *)thread_boot->get_interface(
       PSI_CURRENT_THREAD_VERSION);
