@@ -34,9 +34,9 @@
 #include "mysql/binlog/event/compression/none_comp.h"
 #include "mysql/binlog/event/compression/payload_event_buffer_istream.h"
 #include "mysql/binlog/event/compression/zstd_comp.h"
-#include "mysql/utils/concat.h"
+#include "mysql/utils/concat.h"  // concat
 
-using mysql::utils::concat;
+using mysql::utils::throwing::concat;
 
 namespace mysql::binlog::event::unittests {
 
